package org.processmining.plugins.tracealignment;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XLogImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.tracealignment.clustering.AgglomerativeHierarchicalClustering;
import org.processmining.plugins.tracealignment.clustering.AgglomerativeHierarchicalClustering.ClusterCriteria;
import org.processmining.plugins.tracealignment.clustering.AgglomerativeHierarchicalClustering.ClusterType;
import org.processmining.plugins.tracealignment.msa.MSA;
import org.processmining.plugins.tracealignment.swingx.ErrorDialog;
import org.processmining.plugins.tracealignment.swingx.LazyBoundedRangeModel;
import org.processmining.plugins.tracealignment.swingx.ScrollableGridLayout;
import org.processmining.plugins.tracealignment.swingx.ScrollableViewport;
import org.processmining.plugins.tracealignment.ui.Alignment;
import org.processmining.plugins.tracealignment.ui.AlignmentListener;
import org.processmining.plugins.tracealignment.ui.AlignmentNamePanel;
import org.processmining.plugins.tracealignment.ui.AlignmentPanel;
import org.processmining.plugins.tracealignment.ui.ColorScheme;
import org.processmining.plugins.tracealignment.ui.ColumnAnnotationComponent;
import org.processmining.plugins.tracealignment.ui.ColumnFilter;
import org.processmining.plugins.tracealignment.ui.ColumnFilterComponent;
import org.processmining.plugins.tracealignment.ui.ColumnSort;
import org.processmining.plugins.tracealignment.ui.ColumnSortComponent;
import org.processmining.plugins.tracealignment.ui.ConsensusComponent;
import org.processmining.plugins.tracealignment.ui.ConsensusSequence;
import org.processmining.plugins.tracealignment.ui.DisplayProperties;
import org.processmining.plugins.tracealignment.ui.DisplayPropertiesListener;
import org.processmining.plugins.tracealignment.ui.RulerAnnotationComponent;
import org.processmining.plugins.tracealignment.ui.RulerComponent;
import org.processmining.plugins.tracealignment.ui.Sequence;
import org.processmining.plugins.tracealignment.ui.SequenceComponent;
import org.processmining.plugins.tracealignment.util.ActivityOverFlowException;
import org.processmining.plugins.tracealignment.util.ArrayMatrixFileIO;
import org.processmining.plugins.tracealignment.util.EncodeActivitySet;
import org.processmining.plugins.tracealignment.util.EncodeTraces;
import org.processmining.plugins.tracealignment.util.EncodingNotFoundException;
import org.processmining.plugins.tracealignment.util.EquivalenceClass;
import org.processmining.plugins.tracealignment.util.EuclideanDistance;
import org.processmining.plugins.tracealignment.util.FeatureExtraction;
import org.processmining.plugins.tracealignment.util.FeatureMatrix;
import org.processmining.plugins.tracealignment.util.FeatureSelection;
import org.processmining.plugins.tracealignment.util.FileIO;
import org.processmining.plugins.tracealignment.util.IndelSubstitutionMatrix;
import org.processmining.plugins.tracealignment.util.TraceSimilarity;
import org.processmining.plugins.tracealignment.util.EuclideanDistance.Normalization;
import org.processmining.plugins.tracealignment.util.FeatureExtraction.FeatureType;
import org.processmining.plugins.tracealignment.util.FeatureSelection.FilterType;

@SuppressWarnings( { "serial", "unused" })
public class AlignmentFrame extends JInternalFrame implements AlignmentListener, DisplayPropertiesListener {
	String fileSeparator = System.getProperty("file.separator");
	String outputDir = System.getProperty("java.io.tmpdir")+fileSeparator+"ProM"+fileSeparator+"TraceAlignment";
	String delim = "@";

	UIPluginContext context;
	XLog log;
	List<XTrace> traceList = new ArrayList<XTrace>();
	boolean isLogEncoded = false;

	Set<FeatureType> previousFeatureExtractionTypeSet = new HashSet<FeatureType>();
	Set<FeatureType> featureExtractionTypeSet = new HashSet<FeatureType>();

	int prevMinFrequencyCountThreshold = 0, minFrequencyCountThreshold;
	float prevMinInstanceCountPercentageThreshold = 0f, minInstanceCountPercentageThreshold;

	// The encoding length of activities
	int encodingLength = 1;
	String dash = "-";

	boolean hasOutliers = false;
	/*
	 * Store the original sequence/alphabet feature sets; It is map on the
	 * feature type; K-grams would be stored as kGram e.g., 3Gram, 6Gram etc
	 */
	Map<FeatureType, Set<String>> originalSequenceFeatureSetMap = new HashMap<FeatureType, Set<String>>();
	Map<FeatureType, Map<Set<String>, Set<String>>> originalAlphabetFeatureSetMap = new HashMap<FeatureType, Map<Set<String>, Set<String>>>();

	Map<String, Integer> sequenceFeatureCountMap;
	Map<Set<String>, Integer> alphabetFeatureCountMap;

	Set<String> unionFeatureSet;
	Set<String> filteredFeatureSet;

	Set<String> filteredSequenceFeatureSet;
	Set<Set<String>> filteredAlphabetFeatureSet;

	boolean isSequenceFeatureType = true;

	int[][] featureMatrix;
	float[][] distanceMatrix;
	float[][] similarityMatrix;

	String previousDistanceMetric = "", previousSimilarityMetric = "";
	String distanceMetric = "Euclidean", similarityMetric = "F-Score";
	boolean isPreviousDistanceSelected = true;
	boolean isDistanceSelected = true;

	// Store the size of the k-gram; The default value is 3
	int kGramSize = 3;

	boolean deriveSubstitutionScore = true;
	boolean deriveIndelScore = true;
	boolean isSubstitutionScoreDerived = true;
	boolean isIndelScoreDerived = true;

	Map<String, Integer> substitutionScoreMap = null;
	Map<String, Integer> indelRightGivenLeftScoreMap = null;

	int prevNoClusters = -1;
	private int noClusters = 1;

	int incrementLikeSubstitutionScore = 3; // default value for incrementing like substitution score
	int gapPenalty = Integer.MIN_VALUE; // if set to this, it will be automatically assigned in alignment
	float scaleFactor = 0.1f; // default value

	int prevIncrementLikeSubstitutionScore = -1;
	int prevGapPenalty = -1;
	float prevScaleFactor = -1;

	// List of encoded traces
	List<String> encodedTraceList;

	// List of unique encoded traces
	List<String> uniqueEncodedTraceList;

	// Capture the traces that are identical and keep track of their indices. This would be required to display trace details in the Alignment visualization
	Map<String, TreeSet<Integer>> encodedTraceIdenticalIndicesMap = new HashMap<String, TreeSet<Integer>>();

	// char-activity encoding map
	Map<String, String> activityCharMap;
	Map<String, String> charActivityMap;

	List<String> activityList;
	JTable activityCharMapTable;
	// activity color map

	Map<String, Color> encodedActivityColorMap;

	// Should alignment be enabled for column Filtering
	protected boolean columnFiltering = false;

	// Should alignment be enabled for column Sorting
	protected boolean columnSorting = true;

	// View Menu Check Box Menu Items
	JCheckBoxMenuItem columnFilteringMenuItem, columnSortingMenuItem;

	// whether gaps are to be rendered or not
	JCheckBoxMenuItem renderGapsMenuItem;

	// whether encoded activity is to be rendered or not
	JCheckBoxMenuItem renderEncodedActivityMenuItem;

	JMenu viewMenu, editMenu;
	// whether activity background is square or not
	JCheckBoxMenuItem squareMenuItem;

	// whether activity background is dot or not
	JCheckBoxMenuItem dotMenuItem;

	// Overall Panel for the AlignmentFrame

	JPanel overallPanel;
	JTabbedPane clusterAlignmentTabbedPane;
	JPanel clusterAlignmentTabbedPanePanel;

	// Parameter Settings Panel and all associated panels within it
	JPanel parameterSettingsPanel;
	JPanel preProcessingPanel;
	JPanel featureExtractionPanel;
	JPanel featureSelectionPanel;
	JPanel substitutionIndelScorePanel;
	JPanel distanceSimilarityPanel;
	JPanel clusteringPanel;
	JPanel alignmentParametersPanel;
	JPanel computeAlignmentPanel;

	// Feature Extraction Panel and all associate GUI components within it
	JPanel sequenceFeaturePanel, alphabetFeaturePanel;
	JPanel kGramSizePanel;
	JCheckBox kGramCheckBox, maximalRepeatCheckBox, nearSuperMaximalRepeatCheckBox, superMaximalRepeatCheckBox;
	JCheckBox individualEventCheckBox, maximalRepeatAlphabetCheckBox, nearSuperMaximalRepeatAlphabetCheckBox,
			superMaximalRepeatAlphabetCheckBox;

	// Alignment viewer GUI

	List<Alignment> alignmentList = new ArrayList<Alignment>();
	List<DisplayProperties> displayPropertiesList = new ArrayList<DisplayProperties>();

	JPanel[] clusterAlignmentPanel, leftPanel, topPanel, bottomPanel;
	ScrollableGridLayout[] clusterAlignmentPanelLayout;
	ScrollableViewport[] alignmentViewPort, leftViewPort, bottomViewPort, analysisViewPort, topViewPort,
			bottomLeftViewPort, analysisNameViewPort;
	JScrollBar[] alignmentHorizontalScrollBar, alignmentVerticalScrollBar;

	DisplayProperties[] displayProperties;

	AlignmentPanel[] alignmentPanel;
	AlignmentNamePanel[] alignmentNamePanel;

	ColumnFilterComponent[] columnFilterComponent;
	ColumnSortComponent[] columnSortComponent;
	ConsensusSequence[] consensusSequence;
	ConsensusComponent[] consensusComponent;
	
	int MaxPriority = 10;
	int numColumnSorts;
	Vector<ColumnSort> columnSorts;

	MSA msa;
	private final Set<JMenuItem> alignmentDependentMenuItemSet = new HashSet<JMenuItem>();

	public AlignmentFrame(UIPluginContext context, XLog log) {
		super("Trace Alignment Viewer");
		this.context = context;
		this.log = log;

		createMenu();
		buildParameterSettingsPanel();

		buildClusterAlignmentTabbedPanePanel();

		overallPanel = new JPanel();
		ScrollableGridLayout overallPanelLayout = new ScrollableGridLayout(overallPanel, 2, 1, 0, 0);
		overallPanel.setLayout(overallPanelLayout);

		overallPanelLayout.setColumnFixed(1, true);

		overallPanelLayout.setPosition(clusterAlignmentTabbedPanePanel, 0, 0);
		overallPanel.add(clusterAlignmentTabbedPanePanel);

		JScrollPane parameterSettingsScrollPane = new JScrollPane(parameterSettingsPanel);
		overallPanelLayout.setPosition(parameterSettingsScrollPane, 1, 0);
		overallPanel.add(parameterSettingsScrollPane);

		getContentPane().add(overallPanel);
		getContentPane().validate();
		getContentPane().repaint();
		pack();

		this.show();
	}

	private void createMenu() {
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);

		initializeFileMenu(menuBar);
		initializeEditMenu(menuBar);
		initializeViewMenu(menuBar);
	}

	private void initializeFileMenu(JMenuBar menuBar) {
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');
		menuBar.add(fileMenu);

		JMenu openMenu = new JMenu("Open");
		openMenu.setMnemonic('O');
		//TODO un-comment the following lines once the open functionality is implemented
//		fileMenu.add(openMenu);
//		fileMenu.addSeparator();
		
		JMenuItem openMenuItem = new JMenuItem("Alignment");
		openMenu.add(openMenuItem);

		openMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser fc = new JFileChooser("./");
				fc.setDialogType(JFileChooser.OPEN_DIALOG);
				if (fc.showDialog(AlignmentFrame.this, "Open Alignment") == JFileChooser.APPROVE_OPTION) {
					try {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, "Opening " + "alignment ");
					} catch (Exception e) {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, "Unable to open " + "alignment: ", e);
					}
				}

			}

		});

		/*
		 * Create the Save Menu and Options Add the Save Menu to the FileMenu
		 */

		JMenu saveMenu = new JMenu("Save");
		saveMenu.setMnemonic('S');
		
		//TODO un-comment the following line once the save functionality is implemented
//		fileMenu.add(saveMenu);
//		fileMenu.addSeparator();

		JMenuItem saveMenuItem = new JMenuItem("Save");
		saveMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (JOptionPane.showConfirmDialog(AlignmentFrame.this, "Save ", "Save", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
//					System.out.println("Save Alignment");
				}
			}
		});

		JMenuItem saveAsMenuItem = new JMenuItem("Save As");
		saveAsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser fileChooser = new JFileChooser("./");
				fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
				fileChooser.setSelectedFile(new File("Alignment.txt"));

				if (fileChooser.showDialog(AlignmentFrame.this, "Save Current Alignment As") == JFileChooser.APPROVE_OPTION) {
					try {
						File chosenFile = fileChooser.getSelectedFile();
//						System.out.println("Save To: " + chosenFile.getAbsolutePath());

						if (chosenFile.exists()) {
							if (JOptionPane.showConfirmDialog(AlignmentFrame.this,
									"Warning: this file exists. Overwrite it?", "Save file " + chosenFile,
									JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION) {
								return;
							}
						}
					} catch (Exception e) {

					}
				}

			}
		});

		// Add the Save Menu Items to the Save Menu
		saveMenu.add(saveMenuItem);
		saveMenu.add(saveAsMenuItem);

		JMenu loadMenu = new JMenu("Load");
		loadMenu.setMnemonic('L');
		
		//TODO un-comment the following line once the load functionality is implemented
//		fileMenu.add(loadMenu);
//		fileMenu.addSeparator();

		final JMenuItem loadSubstitutionScoreMenuItem = new JMenuItem("Substitution Scores");
		loadSubstitutionScoreMenuItem.setName("Substitution Scores");
		loadSubstitutionScoreMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				loadSubstitutionIndelScores(loadSubstitutionScoreMenuItem);
			}
		});

		final JMenuItem loadIndelRightGivenLeftScoreMenuItem = new JMenuItem("Indel Right Given Left Scores");
		loadIndelRightGivenLeftScoreMenuItem.setName("Indel Right Given Left Scores");

		loadIndelRightGivenLeftScoreMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				loadSubstitutionIndelScores(loadIndelRightGivenLeftScoreMenuItem);
			}
		});

		final JMenuItem loadIndelLeftGivenRightScoreMenuItem = new JMenuItem("Indel Left Given Right Scores");
		loadIndelLeftGivenRightScoreMenuItem.setName("Indel Left Given Right Scores");
		loadIndelLeftGivenRightScoreMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				loadSubstitutionIndelScores(loadIndelLeftGivenRightScoreMenuItem);
			}
		});

		loadMenu.add(loadSubstitutionScoreMenuItem);
		loadMenu.add(loadIndelRightGivenLeftScoreMenuItem);
		loadMenu.add(loadIndelLeftGivenRightScoreMenuItem);

		JMenu exportMenu = new JMenu("Export");
		exportMenu.setMnemonic('E');
		fileMenu.add(exportMenu);

		JMenuItem exportAlignmentMenuItem = new JMenuItem("Export Alignment to png");
		//		exportAlignmentMenuItem.setMnemonic('L');
		//		fileMenu.addSeparator();
		//		fileMenu.add(exportAlignmentMenuItem);

		exportAlignmentMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser fileChooser = new JFileChooser("./");
				fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
				fileChooser.setSelectedFile(new File("Alignment"));
				if (fileChooser.showDialog(AlignmentFrame.this, "Export Alignment As PNG") == JFileChooser.APPROVE_OPTION) {
					try {
						String fileName;
						File file;
						Image alignmentNamePanelImage, alignmentPanelImage, consensusComponentImage, headerComponentImage;
						BufferedImage appendedImage;
						Graphics2D g2d;
						Dimension dim;
						int alignmentNamePanelWidth, alignmentPanelWidth, alignmentNamePanelHeight, consensusComponentHeight, headerComponentHeight;
						boolean overwrite = false;
						for (int i = 0; i < noClusters; i++) {
							fileName = fileChooser.getSelectedFile().getAbsolutePath() + i + ".png";
							if (!overwrite && new File(fileName).exists()) {
								if (JOptionPane.showConfirmDialog(AlignmentFrame.this,
										"Warning: this file exists. Overwrite it?", "Save file " + fileName,
										JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION) {
									return;
								} else {
									overwrite = true;
								}
							}
							file = new File(fileName);

							if (alignmentList.get(i).getColumnSorting()) {
								dim = columnSortComponent[i].getPreferredSize();
								headerComponentHeight = dim.height;
								headerComponentImage = columnSortComponent[i].createImage(dim.width, dim.height);
								columnSortComponent[i].paint(headerComponentImage.getGraphics());
							} else {
								dim = columnFilterComponent[i].getPreferredSize();
								headerComponentHeight = dim.height;
								headerComponentImage = columnFilterComponent[i].createImage(dim.width, dim.height);
								columnFilterComponent[i].paint(headerComponentImage.getGraphics());
							}

							dim = alignmentNamePanel[i].getPreferredSize();
							alignmentNamePanelWidth = dim.width;
							alignmentNamePanelHeight = dim.height;
							alignmentNamePanelImage = alignmentNamePanel[i].createImage(dim.width, dim.height);
							alignmentNamePanel[i].paint(alignmentNamePanelImage.getGraphics());

							dim = alignmentPanel[i].getPreferredSize();
							alignmentPanelWidth = dim.width;
							alignmentPanelImage = alignmentPanel[i].createImage(dim.width, dim.height);
							alignmentPanel[i].paint(alignmentPanelImage.getGraphics());

							dim = consensusComponent[i].getPreferredSize();
							consensusComponentHeight = dim.height;
							consensusComponentImage = consensusComponent[i].createImage(dim.width, dim.height);
							consensusComponentImage.getGraphics().setClip(0, 0, dim.width, dim.height);
							consensusComponent[i].paint(consensusComponentImage.getGraphics());

							appendedImage = new BufferedImage(alignmentNamePanelWidth + alignmentPanelWidth,
									headerComponentHeight + alignmentNamePanelHeight + consensusComponentHeight,
									BufferedImage.TYPE_3BYTE_BGR);
							g2d = appendedImage.createGraphics();
							g2d.drawImage(headerComponentImage, alignmentNamePanelWidth, 0, AlignmentFrame.this);
							g2d.drawImage(alignmentNamePanelImage, 0, headerComponentHeight, AlignmentFrame.this);
							g2d.drawImage(alignmentPanelImage, alignmentNamePanelWidth, headerComponentHeight,
									AlignmentFrame.this);
							g2d.drawImage(consensusComponentImage, alignmentNamePanelWidth, headerComponentHeight
									+ alignmentNamePanelHeight, AlignmentFrame.this);
							g2d.setColor(Color.white);
							g2d.fillRect(0, 0, alignmentNamePanelWidth, headerComponentHeight);
							g2d.fillRect(0, headerComponentHeight + alignmentNamePanelHeight, alignmentNamePanelWidth,
									consensusComponentHeight);

							ImageIO.write(appendedImage, "png", file);
						}
					} catch (IOException e) {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, e.getMessage());
					}
				}
			}
		});

		JMenuItem exportClusterLogsMenuItem = new JMenuItem("Export Cluster Logs To Framework");
		exportClusterLogsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Sequence[] sequencesInAlignment;
				String encodedTrace;
				XLog[] clusterLog = new XLog[noClusters];
				Set<Integer> identicalTraceSet;
				for (int i = 0; i < noClusters; i++) {
					clusterLog[i] = new XLogImpl(log.getAttributes());
					sequencesInAlignment = alignmentList.get(i).getAllSequences();
					for (Sequence seq : sequencesInAlignment) {
						encodedTrace = seq.getSequenceWithoutGapsString();

						if (encodedTraceIdenticalIndicesMap.containsKey(encodedTrace)) {
							identicalTraceSet = encodedTraceIdenticalIndicesMap.get(encodedTrace);
							for (Integer traceIndex : identicalTraceSet) {
								clusterLog[i].add(log.get(traceIndex));
							}
						} else {
							ErrorDialog.showErrorDialog(AlignmentFrame.this, "EncodedTrace Not Found");
						}
					}
//					System.out.println("No. Traces in Cluster: " + clusterLog[i].size());
					context.getProvidedObjectManager().createProvidedObject("AlignmentClusterLog " + i, clusterLog[i],
							XLog.class, context);
				}
				//				
			}
		});

		exportMenu.add(exportAlignmentMenuItem);
		exportMenu.add(exportClusterLogsMenuItem);
	}

	private void loadSubstitutionIndelScores(Component owner) {
		JFileChooser fc = new JFileChooser("./");
		fc.setDialogType(JFileChooser.OPEN_DIALOG);

		String openText = "Open";
		try {
			if (owner.getName().equalsIgnoreCase("Substitution Scores")) {
				openText = "Open Substitution Scores";
			} else if (owner.getName().equalsIgnoreCase("Indel Right Given Left Scores")) {
				openText = "Open Indel Right Given Left Scores";
			} else if (owner.getName().equalsIgnoreCase("Indel Left Given Right Scores")) {
				openText = "Open Indel Left Given Right Scores";
			}

			if (fc.showDialog(AlignmentFrame.this, openText) == JFileChooser.APPROVE_OPTION) {
				ErrorDialog.showErrorDialog(AlignmentFrame.this, openText);
			}
		} catch (Exception e) {
			ErrorDialog.showErrorDialog(AlignmentFrame.this, "Unable to " + openText, e);
		}
	}

	private void initializeEditMenu(JMenuBar menuBar) {
		editMenu = new JMenu("Edit");
		editMenu.setMnemonic('E');
		menuBar.add(editMenu);

		JMenu deleteMenu = new JMenu("Delete");
		JMenuItem deleteAllGapColumnsMenuItem = new JMenuItem("All Gap Columns");
		deleteAllGapColumnsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//TODO 
				int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
				alignmentList.get(clusterNo).deselectAllColumns(displayProperties[clusterNo]);
				alignmentList.get(clusterNo).removeAllGapColumns();
				//				consensusComponent[clusterNo].setConsensus(new ConsensusSequence(displayProperties[clusterNo]));
			}
		});

		editMenu.add(deleteMenu);
		deleteMenu.add(deleteAllGapColumnsMenuItem);
	}

	private void initializeViewMenu(JMenuBar menuBar) {
		viewMenu = new JMenu("View");
		viewMenu.setMnemonic('V');

		menuBar.add(viewMenu);

		JMenu colorSchemeMenu = new JMenu("Color Scheme");
		JMenuItem colorSchemeMenuItem1 = new JMenuItem("Color Scheme 1");
		colorSchemeMenuItem1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub

			}

		});
		colorSchemeMenu.add(colorSchemeMenuItem1);

		//TODO un-comment the following line once the colorScheme functionality is implemented
//		viewMenu.add(colorSchemeMenu);
//		viewMenu.addSeparator();

		columnFilteringMenuItem = new JCheckBoxMenuItem("Filter by Column", columnFiltering);
		//		addAlignmentDependentMenuItem(columnFilteringMenuItem);
		columnFilteringMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				//TODO 
				setColumnFiltering(columnFilteringMenuItem.isSelected(), true);
			}
		});
		viewMenu.add(columnFilteringMenuItem);

		columnSortingMenuItem = new JCheckBoxMenuItem("Sort by Column", columnSorting);
		//		addAlignmentDependentMenuItem(columnSortingMenuItem);
		columnSortingMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				//TODO
				setColumnSorting(columnSortingMenuItem.isSelected(), true);
			}
		});
		viewMenu.add(columnSortingMenuItem);

		viewMenu.addSeparator();

		renderGapsMenuItem = new JCheckBoxMenuItem("Render Gaps", true);
		renderGapsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
				displayProperties[clusterNo].setRenderGaps(renderGapsMenuItem.getState());
			}
		});
		viewMenu.add(renderGapsMenuItem);

		viewMenu.addSeparator();
		renderEncodedActivityMenuItem = new JCheckBoxMenuItem("Render Encoded Activity", true);
		renderEncodedActivityMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
				displayProperties[clusterNo].setRenderEncodedActivity(renderEncodedActivityMenuItem.getState());
			}
		});
		viewMenu.add(renderEncodedActivityMenuItem);

		viewMenu.addSeparator();
		JMenu backgroundMenu = new JMenu("Activity Background");
		squareMenuItem = new JCheckBoxMenuItem("Square");
		dotMenuItem = new JCheckBoxMenuItem("Dot");
		squareMenuItem.setSelected(true);
		dotMenuItem.setSelected(false);

		viewMenu.add(backgroundMenu);
		backgroundMenu.add(squareMenuItem);
		backgroundMenu.add(dotMenuItem);

		squareMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
				displayProperties[clusterNo].setActivityBackgroundSquare(squareMenuItem.isSelected());
				dotMenuItem.setSelected(false);
			}
		});

		dotMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
				displayProperties[clusterNo].setActivityBackgroundSquare(!dotMenuItem.isSelected());
				squareMenuItem.setSelected(false);
			}
		});

	}

	private void buildParameterSettingsPanel() {
		// build the panel with pre-processing options
		buildPreProcessingPanel();

		// build the panel with feature extraction options 
		buildFeatureExtractionPanel();

		// build the panel for filtering features		
		buildFeatureSelectionPanel();

		// build the panel for substitution/indel score options
		buildSubstitutionIndelScorePanel();

		// build the panel with options for computing distance/similarity between traces
		buildDistanceSimilarityPanel();

		// build the panel with clustering options
		buildClusteringPanel();

		// build the panel with alignment parameters
		buildAlignmentParametersPanel();

		// build the panel with compute alignment button
		buildComputeAlignmentPanel();

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.VERTICAL;

		parameterSettingsPanel = new JPanel();
		ScrollableGridLayout parameterSettingsPanelLayout = new ScrollableGridLayout(parameterSettingsPanel, 1, 8, 0, 0);

		// Set the rows of the panel to be fixed
		parameterSettingsPanelLayout.setRowFixed(0, true);
		parameterSettingsPanelLayout.setRowFixed(1, true);
		parameterSettingsPanelLayout.setRowFixed(2, true);
		parameterSettingsPanelLayout.setRowFixed(3, true);
		parameterSettingsPanelLayout.setRowFixed(4, true);
		parameterSettingsPanelLayout.setRowFixed(5, true);
		parameterSettingsPanelLayout.setRowFixed(6, true);
		parameterSettingsPanelLayout.setRowFixed(7, true);

		parameterSettingsPanel.setLayout(parameterSettingsPanelLayout);

		parameterSettingsPanelLayout.setPosition(preProcessingPanel, 0, 0);
		parameterSettingsPanel.add(preProcessingPanel);

		parameterSettingsPanelLayout.setPosition(featureExtractionPanel, 0, 1);
		parameterSettingsPanel.add(featureExtractionPanel);

		parameterSettingsPanelLayout.setPosition(featureSelectionPanel, 0, 2);
		parameterSettingsPanel.add(featureSelectionPanel);

		parameterSettingsPanelLayout.setPosition(substitutionIndelScorePanel, 0, 3);
		parameterSettingsPanel.add(substitutionIndelScorePanel);

		parameterSettingsPanelLayout.setPosition(distanceSimilarityPanel, 0, 4);
		parameterSettingsPanel.add(distanceSimilarityPanel);

		parameterSettingsPanelLayout.setPosition(clusteringPanel, 0, 5);
		parameterSettingsPanel.add(clusteringPanel);

		parameterSettingsPanelLayout.setPosition(alignmentParametersPanel, 0, 6);
		parameterSettingsPanel.add(alignmentParametersPanel);

		parameterSettingsPanelLayout.setPosition(computeAlignmentPanel, 0, 7);
		parameterSettingsPanel.add(computeAlignmentPanel);
	}

	private void buildPreProcessingPanel() {
		preProcessingPanel = new JPanel();
		preProcessingPanel.setBorder(BorderFactory.createEtchedBorder());

		GridBagLayout preProcessingGBLayout = new GridBagLayout();
		preProcessingPanel.setLayout(preProcessingGBLayout);

		final GridBagConstraints c = new GridBagConstraints();

		JLabel preprocessingLabel = new JLabel("PreProcessing");

		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 150);
		preProcessingPanel.add(preprocessingLabel, c);

		JCheckBox removeLoopsCheckBox = new JCheckBox("Remove Loops");
		removeLoopsCheckBox.setEnabled(false);

		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		preProcessingPanel.add(removeLoopsCheckBox, c);

		removeLoopsCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//TODO 
			}
		});

		JCheckBox removeOutliersCheckBox = new JCheckBox("Remove Outliers");
		removeOutliersCheckBox.setEnabled(false);

		c.gridx = 0;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		preProcessingPanel.add(removeOutliersCheckBox, c);

		removeOutliersCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//TODO 
			}
		});
	}

	private void buildFeatureExtractionPanel() {
		featureExtractionPanel = new JPanel();
		featureExtractionPanel.setBorder(BorderFactory.createEtchedBorder());

		GridBagLayout featureExtractionGBLayout = new GridBagLayout();
		featureExtractionPanel.setLayout(featureExtractionGBLayout);

		final GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 10);

		JLabel featureTypeLabel = new JLabel("Feature Type");
		featureExtractionPanel.add(featureTypeLabel, c);

		final JRadioButton sequenceFeatureRadioButton = new JRadioButton("Sequence");
		sequenceFeatureRadioButton.setSelected(true);
		sequenceFeatureRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (sequenceFeatureRadioButton.isSelected()) {
					/*
					 * Uncheck all selections in alphabetFeaturePanel
					 */
					maximalRepeatAlphabetCheckBox.setSelected(false);
					superMaximalRepeatAlphabetCheckBox.setSelected(false);
					nearSuperMaximalRepeatAlphabetCheckBox.setSelected(false);

					featureExtractionTypeSet.clear();

					isSequenceFeatureType = true;

					featureExtractionPanel.remove(alphabetFeaturePanel);
					c.gridwidth = 3;
					c.gridx = 0;
					c.gridy = 1;
					c.insets = new Insets(5, 0, 0, 0);
					featureExtractionPanel.add(sequenceFeaturePanel, c);
					featureExtractionPanel.revalidate();
					featureExtractionPanel.repaint();
				}
			}
		});

		c.gridx = 1;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 10);
		featureExtractionPanel.add(sequenceFeatureRadioButton, c);

		final JRadioButton alphabetFeatureRadioButton = new JRadioButton("Alphabet");
		alphabetFeatureRadioButton.setSelected(false);
		alphabetFeatureRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (alphabetFeatureRadioButton.isSelected()) {
					/*
					 * Uncheck all selections in sequenceFeaturePanel
					 */
					individualEventCheckBox.setSelected(false);
					kGramCheckBox.setSelected(false);
					maximalRepeatCheckBox.setSelected(false);
					superMaximalRepeatCheckBox.setSelected(false);
					nearSuperMaximalRepeatCheckBox.setSelected(false);

					featureExtractionTypeSet.clear();

					isSequenceFeatureType = false;

					featureExtractionPanel.remove(sequenceFeaturePanel);
					featureExtractionPanel.remove(kGramSizePanel);
					c.gridwidth = 3;
					c.gridx = 0;
					c.gridy = 1;
					c.insets = new Insets(5, 0, 0, 0);
					featureExtractionPanel.add(alphabetFeaturePanel, c);
					featureExtractionPanel.revalidate();
					featureExtractionPanel.repaint();
				}
			}

		});
		c.gridx = 2;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 10);
		featureExtractionPanel.add(alphabetFeatureRadioButton, c);

		ButtonGroup featureTypeButtonGroup = new ButtonGroup();
		featureTypeButtonGroup.add(sequenceFeatureRadioButton);
		featureTypeButtonGroup.add(alphabetFeatureRadioButton);

		c.gridwidth = 3;
		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 0);

		buildSequenceFeaturePanel();
		buildAlphabetFeaturePanel();

		featureExtractionPanel.add(sequenceFeaturePanel, c);
	}

	private void buildSequenceFeaturePanel() {
		if (sequenceFeaturePanel == null) {
			sequenceFeaturePanel = new JPanel();
			sequenceFeaturePanel.setBorder(BorderFactory.createEtchedBorder());

			ScrollableGridLayout sequenceFeatureLayout = new ScrollableGridLayout(sequenceFeaturePanel, 1, 5, 0, 0);
			sequenceFeaturePanel.setLayout(sequenceFeatureLayout);

			individualEventCheckBox = new JCheckBox("Individual Event");
			kGramCheckBox = new JCheckBox("K Gram");
			maximalRepeatCheckBox = new JCheckBox("Maximal Repeats");
			superMaximalRepeatCheckBox = new JCheckBox("Super Maximal Repeats");
			nearSuperMaximalRepeatCheckBox = new JCheckBox("Near Super Maximal Repeats");

			sequenceFeatureLayout.setPosition(individualEventCheckBox, 0, 0);
			sequenceFeaturePanel.add(individualEventCheckBox);

			sequenceFeatureLayout.setPosition(kGramCheckBox, 0, 1);
			sequenceFeaturePanel.add(kGramCheckBox);

			sequenceFeatureLayout.setPosition(maximalRepeatCheckBox, 0, 2);
			sequenceFeaturePanel.add(maximalRepeatCheckBox);

			sequenceFeatureLayout.setPosition(nearSuperMaximalRepeatCheckBox, 0, 3);
			sequenceFeaturePanel.add(nearSuperMaximalRepeatCheckBox);

			sequenceFeatureLayout.setPosition(superMaximalRepeatCheckBox, 0, 4);
			sequenceFeaturePanel.add(superMaximalRepeatCheckBox);

			buildKGramSizePanel();

			individualEventCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (individualEventCheckBox.isSelected()) {
						featureExtractionTypeSet.add(FeatureType.INDIVIDUAL_EVENT);
					} else {
						featureExtractionTypeSet.remove(FeatureType.INDIVIDUAL_EVENT);
					}
				}
			});

			kGramCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					if (kGramCheckBox.isSelected()) {
						GridBagConstraints c = new GridBagConstraints();
						c.gridx = 0;
						c.gridy = 2;
						c.gridwidth = 3;
						c.insets = new Insets(5, 0, 0, 0);
						featureExtractionPanel.add(kGramSizePanel, c);
						featureExtractionTypeSet.add(FeatureType.KGRAM);
					} else {
						featureExtractionPanel.remove(kGramSizePanel);
						featureExtractionTypeSet.remove(FeatureType.KGRAM);
					}
					featureExtractionPanel.revalidate();
					featureExtractionPanel.repaint();
				}
			});

			maximalRepeatCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					if (maximalRepeatCheckBox.isSelected()) {
						featureExtractionTypeSet.add(FeatureType.MAXIMAL_REPEAT);
						featureExtractionTypeSet.remove(FeatureType.SUPERMAXIMAL_REPEAT);
						featureExtractionTypeSet.remove(FeatureType.NEARSUPERMAXIMAL_REPEAT);
						superMaximalRepeatCheckBox.setSelected(false);
						nearSuperMaximalRepeatCheckBox.setSelected(false);
						superMaximalRepeatCheckBox.setEnabled(false);
						nearSuperMaximalRepeatCheckBox.setEnabled(false);
					} else {
						featureExtractionTypeSet.remove(FeatureType.MAXIMAL_REPEAT);
						superMaximalRepeatCheckBox.setEnabled(true);
						nearSuperMaximalRepeatCheckBox.setEnabled(true);
					}
				}
			});

			superMaximalRepeatCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					if (superMaximalRepeatCheckBox.isSelected()) {
						featureExtractionTypeSet.add(FeatureType.SUPERMAXIMAL_REPEAT);
						
					} else {
						featureExtractionTypeSet.remove(FeatureType.SUPERMAXIMAL_REPEAT);
					}
				}
			});

			nearSuperMaximalRepeatCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					if (nearSuperMaximalRepeatCheckBox.isSelected()) {
						featureExtractionTypeSet.add(FeatureType.NEARSUPERMAXIMAL_REPEAT);
						featureExtractionTypeSet.remove(FeatureType.SUPERMAXIMAL_REPEAT);
						superMaximalRepeatCheckBox.setSelected(false);
						superMaximalRepeatCheckBox.setEnabled(false);
					} else {
						featureExtractionTypeSet.remove(FeatureType.NEARSUPERMAXIMAL_REPEAT);
						superMaximalRepeatCheckBox.setEnabled(true);
					}
				}
			});
		}
	}

	private void buildAlphabetFeaturePanel() {
		if (alphabetFeaturePanel == null) {
			alphabetFeaturePanel = new JPanel();
			alphabetFeaturePanel.setBorder(BorderFactory.createEtchedBorder());

			ScrollableGridLayout alphabetFeatureLayout = new ScrollableGridLayout(alphabetFeaturePanel, 1, 3, 0, 0);
			alphabetFeaturePanel.setLayout(alphabetFeatureLayout);

			maximalRepeatAlphabetCheckBox = new JCheckBox("Maximal Repeat Alphabet");

			superMaximalRepeatAlphabetCheckBox = new JCheckBox("Super Maximal Repeat Alphabet");
			nearSuperMaximalRepeatAlphabetCheckBox = new JCheckBox("Near Super Maximal Repeat Alphabet");

			alphabetFeatureLayout.setPosition(maximalRepeatAlphabetCheckBox, 0, 0);
			alphabetFeaturePanel.add(maximalRepeatAlphabetCheckBox);

			alphabetFeatureLayout.setPosition(nearSuperMaximalRepeatAlphabetCheckBox, 0, 1);
			alphabetFeaturePanel.add(nearSuperMaximalRepeatAlphabetCheckBox);

			alphabetFeatureLayout.setPosition(superMaximalRepeatAlphabetCheckBox, 0, 2);
			alphabetFeaturePanel.add(superMaximalRepeatAlphabetCheckBox);

			maximalRepeatAlphabetCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (maximalRepeatAlphabetCheckBox.isSelected()) {
						featureExtractionTypeSet.add(FeatureType.MAXIMAL_REPEAT_ALPHABET);
						featureExtractionTypeSet.remove(FeatureType.SUPERMAXIMAL_REPEAT_ALPHABET);
						featureExtractionTypeSet.remove(FeatureType.NEARSUPERMAXIMAL_REPEAT_ALPHABET);
						superMaximalRepeatAlphabetCheckBox.setSelected(false);
						nearSuperMaximalRepeatAlphabetCheckBox.setSelected(false);
						superMaximalRepeatAlphabetCheckBox.setEnabled(false);
						nearSuperMaximalRepeatAlphabetCheckBox.setEnabled(false);
					} else {
						featureExtractionTypeSet.remove(FeatureType.MAXIMAL_REPEAT_ALPHABET);
						superMaximalRepeatAlphabetCheckBox.setEnabled(true);
						nearSuperMaximalRepeatAlphabetCheckBox.setEnabled(true);
					}
				}
			});

			superMaximalRepeatAlphabetCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (superMaximalRepeatAlphabetCheckBox.isSelected()) {
						featureExtractionTypeSet.add(FeatureType.SUPERMAXIMAL_REPEAT_ALPHABET);
					} else {
						featureExtractionTypeSet.remove(FeatureType.SUPERMAXIMAL_REPEAT_ALPHABET);
					}
				}
			});

			nearSuperMaximalRepeatAlphabetCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (nearSuperMaximalRepeatAlphabetCheckBox.isSelected()) {
						featureExtractionTypeSet.add(FeatureType.NEARSUPERMAXIMAL_REPEAT_ALPHABET);
						featureExtractionTypeSet.remove(FeatureType.SUPERMAXIMAL_REPEAT_ALPHABET);
						superMaximalRepeatAlphabetCheckBox.setSelected(false);
						superMaximalRepeatAlphabetCheckBox.setEnabled(false);
					} else {
						featureExtractionTypeSet.remove(FeatureType.NEARSUPERMAXIMAL_REPEAT_ALPHABET);
						superMaximalRepeatAlphabetCheckBox.setEnabled(true);
					}
				}
			});
		}
	}

	private void buildKGramSizePanel() {
		kGramSizePanel = new JPanel();
		GridBagLayout kGramGBLayout = new GridBagLayout();
		kGramSizePanel.setLayout(kGramGBLayout);
		GridBagConstraints c = new GridBagConstraints();

		JLabel kGramSizeLabel = new JLabel("Gram Size");
		final JSlider kGramSizeSlider = new JSlider(JSlider.HORIZONTAL, 2, 6, 3);
		final JLabel kGramValueLabel = new JLabel("3");
		kGramSizeSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				int kGramValue = kGramSizeSlider.getValue();
				kGramValueLabel.setText(kGramValue + "");
				kGramSizePanel.revalidate();
				kGramSizePanel.repaint();
			}

		});
		Dimension d = kGramSizeSlider.getPreferredSize();
		kGramSizeSlider.setPreferredSize(new Dimension(d.width / 2, d.height));
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 10);
		kGramSizePanel.add(kGramSizeLabel, c);

		c.gridx = 1;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 10);
		kGramSizePanel.add(kGramSizeSlider, c);

		c.gridx = 2;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 10);
		kGramSizePanel.add(kGramValueLabel, c);
	}

	private void buildFeatureSelectionPanel() {
		featureSelectionPanel = new JPanel();
		featureSelectionPanel.setBorder(BorderFactory.createEtchedBorder());

		GridBagLayout featureSelectionGBLayout = new GridBagLayout();
		featureSelectionPanel.setLayout(featureSelectionGBLayout);

		final GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;

		JLabel featureSelectionLabel = new JLabel("Feature Selection");
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 3;
		c.insets = new Insets(5, 0, 0, 10);
		featureSelectionPanel.add(featureSelectionLabel, c);

		c.gridwidth = 1;
		final JCheckBox frequencyCountCheckbox = new JCheckBox("Frequency");
		frequencyCountCheckbox.setSelected(false);
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.insets = new Insets(5, 0, 0, 10);
		featureSelectionPanel.add(frequencyCountCheckbox, c);

		final JSlider frequencyCountSlider = new JSlider(JSlider.HORIZONTAL, 1, 100, 20);
		Dimension d = frequencyCountSlider.getPreferredSize();
		frequencyCountSlider.setPreferredSize(new Dimension(d.width / 2, d.height));
		frequencyCountSlider.setEnabled(false);
		c.gridx = 1;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		featureSelectionPanel.add(frequencyCountSlider, c);

		final JLabel frequencyCountLabel = new JLabel("20");
		frequencyCountLabel.setEnabled(false);
		c.gridx = 2;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		featureSelectionPanel.add(frequencyCountLabel, c);

		frequencyCountCheckbox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (frequencyCountCheckbox.isSelected()) {
					frequencyCountSlider.setEnabled(true);
					frequencyCountLabel.setEnabled(true);
					minFrequencyCountThreshold = frequencyCountSlider.getValue();
				} else {
					frequencyCountSlider.setEnabled(false);
					frequencyCountLabel.setEnabled(false);
					minFrequencyCountThreshold = 0;
				}
			}

		});

		frequencyCountSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				frequencyCountLabel.setText(frequencyCountSlider.getValue() + "");
				minFrequencyCountThreshold = frequencyCountSlider.getValue();
			}
		});

		final JCheckBox instanceCountCheckbox = new JCheckBox("Instance %");
		instanceCountCheckbox.setSelected(false);
		c.gridx = 0;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		featureSelectionPanel.add(instanceCountCheckbox, c);

		final JSlider instanceSlider = new JSlider(JSlider.HORIZONTAL, 1, 100, 10);
		d = instanceSlider.getPreferredSize();
		instanceSlider.setPreferredSize(new Dimension(d.width / 2, d.height));
		instanceSlider.setEnabled(false);
		c.gridx = 1;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		featureSelectionPanel.add(instanceSlider, c);

		final JLabel instanceThresholdLabel = new JLabel("10");
		instanceThresholdLabel.setEnabled(false);
		c.gridx = 2;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		featureSelectionPanel.add(instanceThresholdLabel, c);

		instanceCountCheckbox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (instanceCountCheckbox.isSelected()) {
					instanceSlider.setEnabled(true);
					instanceThresholdLabel.setEnabled(true);
					minInstanceCountPercentageThreshold = instanceSlider.getValue() / 100f;
				} else {
					instanceSlider.setEnabled(false);
					instanceThresholdLabel.setEnabled(false);
					minInstanceCountPercentageThreshold = 0;
				}
			}
		});

		instanceSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				instanceThresholdLabel.setText(instanceSlider.getValue() + "");
				minInstanceCountPercentageThreshold = instanceSlider.getValue() / 100f;
			}
		});

		//		nonOverlappingCheckBox = new JCheckBox("Non Overlapping");
		//		nonOverlappingCheckBox.setSelected(false);
		//		c.gridx = 0;
		//		c.gridy = 3;
		//		c.gridwidth = 3;
		//		c.insets = new Insets(5, 0, 0, 10);
		//		featureSelectionPanel.add(nonOverlappingCheckBox, c);
	}

	private void buildSubstitutionIndelScorePanel() {
		substitutionIndelScorePanel = new JPanel();
		substitutionIndelScorePanel.setBorder(BorderFactory.createEtchedBorder());
		GridBagLayout substitutionIndelScoreGBLayout = new GridBagLayout();
		substitutionIndelScorePanel.setLayout(substitutionIndelScoreGBLayout);

		final GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 10);

		JLabel scoringMatricesLabel = new JLabel("Scoring Matrices");
		substitutionIndelScorePanel.add(scoringMatricesLabel, c);

		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);

		JLabel substitutionScoreLabel = new JLabel("Substitution Score");
		substitutionIndelScorePanel.add(substitutionScoreLabel, c);

		final JLabel substitutionScoreFileLabel = new JLabel("Subsitution Score File");
		substitutionScoreFileLabel.setEnabled(false);

		final JButton selectSubstitutionScoreFileButton = new JButton("Select File");
		selectSubstitutionScoreFileButton.setEnabled(false);
		selectSubstitutionScoreFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser substitutionScoreFileChooser = new JFileChooser(".");
				substitutionScoreFileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				if (substitutionScoreFileChooser.showDialog(AlignmentFrame.this, "Open Alignment") == JFileChooser.APPROVE_OPTION) {
					try {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, "Unable to open ABC" + "alignment: ");
					} catch (Exception e) {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, "Unable to open " + "alignment: ", e);
					}
				}
			}
		});

		final JRadioButton deriveSubstitutionScoreRadioButton = new JRadioButton("Derive");
		deriveSubstitutionScoreRadioButton.setSelected(true);
		deriveSubstitutionScoreRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (deriveSubstitutionScoreRadioButton.isSelected()) {
					substitutionScoreFileLabel.setEnabled(false);
					selectSubstitutionScoreFileButton.setEnabled(false);

					deriveSubstitutionScore = true;

					substitutionIndelScorePanel.revalidate();
					substitutionIndelScorePanel.repaint();
				}
			}
		});

		final JRadioButton loadSubstitutionScoreRadioButton = new JRadioButton("Load");
		loadSubstitutionScoreRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (loadSubstitutionScoreRadioButton.isSelected()) {
					substitutionScoreFileLabel.setEnabled(true);
					selectSubstitutionScoreFileButton.setEnabled(true);

					deriveSubstitutionScore = false;

					substitutionIndelScorePanel.revalidate();
					substitutionIndelScorePanel.repaint();
				}
			}
		});

		ButtonGroup substitutionScoreButtonGroup = new ButtonGroup();
		substitutionScoreButtonGroup.add(deriveSubstitutionScoreRadioButton);
		substitutionScoreButtonGroup.add(loadSubstitutionScoreRadioButton);

		c.gridx = 1;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(deriveSubstitutionScoreRadioButton, c);

		c.gridx = 2;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(loadSubstitutionScoreRadioButton, c);

		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 2;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(substitutionScoreFileLabel, c);

		c.gridx = 2;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(selectSubstitutionScoreFileButton, c);

		final JLabel indelRightGivenLeftScoreLabel = new JLabel("Indel Right Given Left Score");
		indelRightGivenLeftScoreLabel.setEnabled(false);

		final JButton selectIndelRightGivenLeftScoreFileButton = new JButton("Select File");
		selectIndelRightGivenLeftScoreFileButton.setEnabled(false);
		selectIndelRightGivenLeftScoreFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser indelRightGivenLeftScoreFileChooser = new JFileChooser(".");
				indelRightGivenLeftScoreFileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				if (indelRightGivenLeftScoreFileChooser.showDialog(AlignmentFrame.this, "Open Alignment") == JFileChooser.APPROVE_OPTION) {
					try {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, "Unable to open ABC" + "alignment: ");
					} catch (Exception e) {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, "Unable to open " + "alignment: ", e);
					}
				}
			}
		});

		final JLabel indelLeftGivenRightScoreLabel = new JLabel("Indel Left Given Right Score");
		indelLeftGivenRightScoreLabel.setEnabled(false);

		final JButton selectIndelLeftGivenRightScoreFileButton = new JButton("Select File");
		selectIndelLeftGivenRightScoreFileButton.setEnabled(false);
		selectIndelLeftGivenRightScoreFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser indelLeftGivenRightScoreFileChooser = new JFileChooser(".");
				indelLeftGivenRightScoreFileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				if (indelLeftGivenRightScoreFileChooser.showDialog(AlignmentFrame.this, "Open Alignment") == JFileChooser.APPROVE_OPTION) {
					try {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, "Unable to open ABC" + "alignment: ");
					} catch (Exception e) {
						ErrorDialog.showErrorDialog(AlignmentFrame.this, "Unable to open " + "alignment: ", e);
					}
				}
			}
		});

		final JRadioButton deriveIndelScoreRadioButton = new JRadioButton("Derive");
		deriveIndelScoreRadioButton.setSelected(true);
		deriveIndelScoreRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (deriveIndelScoreRadioButton.isSelected()) {
					indelRightGivenLeftScoreLabel.setEnabled(false);
					indelLeftGivenRightScoreLabel.setEnabled(false);

					deriveIndelScore = true;

					selectIndelRightGivenLeftScoreFileButton.setEnabled(false);
					selectIndelLeftGivenRightScoreFileButton.setEnabled(false);
				}
			}
		});

		final JRadioButton loadIndelScoreRadioButton = new JRadioButton("Load");
		loadIndelScoreRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (loadIndelScoreRadioButton.isSelected()) {
					indelRightGivenLeftScoreLabel.setEnabled(true);
					indelLeftGivenRightScoreLabel.setEnabled(true);

					deriveIndelScore = false;

					selectIndelRightGivenLeftScoreFileButton.setEnabled(true);
					selectIndelLeftGivenRightScoreFileButton.setEnabled(true);
				}
			}
		});

		ButtonGroup indelScoreButtonGroup = new ButtonGroup();
		indelScoreButtonGroup.add(deriveIndelScoreRadioButton);
		indelScoreButtonGroup.add(loadIndelScoreRadioButton);

		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 3;
		c.insets = new Insets(5, 0, 0, 10);
		JLabel indelScoreLabel = new JLabel("Indel Score");
		substitutionIndelScorePanel.add(indelScoreLabel, c);

		c.gridx = 1;
		c.gridy = 3;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(deriveIndelScoreRadioButton, c);

		c.gridx = 2;
		c.gridy = 3;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(loadIndelScoreRadioButton, c);

		c.gridx = 0;
		c.gridy = 4;
		c.gridwidth = 2;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(indelRightGivenLeftScoreLabel, c);

		c.gridx = 2;
		c.gridy = 4;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(selectIndelRightGivenLeftScoreFileButton, c);

		c.gridx = 0;
		c.gridy = 5;
		c.gridwidth = 2;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(indelLeftGivenRightScoreLabel, c);

		c.gridx = 2;
		c.gridy = 5;
		c.insets = new Insets(5, 0, 0, 10);
		substitutionIndelScorePanel.add(selectIndelLeftGivenRightScoreFileButton, c);

		substitutionIndelScorePanel.revalidate();
		substitutionIndelScorePanel.repaint();
	}

	private void buildDistanceSimilarityPanel() {
		distanceSimilarityPanel = new JPanel();
		distanceSimilarityPanel.setBorder(BorderFactory.createEtchedBorder());

		GridBagLayout distanceSimilarityGBLayout = new GridBagLayout();
		distanceSimilarityPanel.setLayout(distanceSimilarityGBLayout);

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;

		JLabel distanceSimilartiyLabel = new JLabel("Distance/Similarity");
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 60);
		distanceSimilarityPanel.add(distanceSimilartiyLabel, c);

		final JRadioButton distanceRadioButton = new JRadioButton("Distance");
		final JRadioButton similarityRadioButton = new JRadioButton("Similarity");

		ButtonGroup distanceSimilarityButtonGroup = new ButtonGroup();
		distanceSimilarityButtonGroup.add(distanceRadioButton);
		distanceSimilarityButtonGroup.add(similarityRadioButton);

		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		distanceSimilarityPanel.add(distanceRadioButton, c);

		c.gridx = 0;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		distanceSimilarityPanel.add(similarityRadioButton, c);

		String[] distanceMetrics = new String[] { "Euclidean" };
		String[] similarityMetrics = new String[] { "F-Score" };

		final JComboBox distanceComboBox = new JComboBox(distanceMetrics);
		distanceComboBox.setEnabled(false);

		c.gridx = 1;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		distanceSimilarityPanel.add(distanceComboBox, c);

		final JComboBox similarityComboBox = new JComboBox(similarityMetrics);
		similarityComboBox.setEnabled(false);

		c.gridx = 1;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		distanceSimilarityPanel.add(similarityComboBox, c);

		distanceRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (distanceRadioButton.isSelected()) {
					distanceComboBox.setEnabled(true);
					similarityComboBox.setEnabled(false);
					isDistanceSelected = true;
				}
			}
		});

		distanceComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
//				System.out.println("Distance Metric: " + distanceComboBox.getSelectedItem());
				distanceMetric = (String) distanceComboBox.getSelectedItem();
			}
		});

		similarityRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (similarityRadioButton.isEnabled()) {
					distanceComboBox.setEnabled(false);
					similarityComboBox.setEnabled(true);
					isDistanceSelected = false;
				}
			}
		});

		similarityComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
//				System.out.println("Similarity: " + similarityComboBox.getSelectedItem());
				similarityMetric = (String) similarityComboBox.getSelectedItem();
			}
		});
	}

	private void buildClusteringPanel() {
		clusteringPanel = new JPanel();
		clusteringPanel.setBorder(BorderFactory.createEtchedBorder());

		GridBagLayout clusteringPanelGBLayout = new GridBagLayout();
		clusteringPanel.setLayout(clusteringPanelGBLayout);

		JLabel clusteringLabel = new JLabel("Clustering");

		JLabel noClustersLabel = new JLabel("No. Clusters");
		noClusters = 4;
		prevNoClusters = 4;
		final JLabel noClustersValueLabel = new JLabel("4");

		final JSlider noClustersSlider = new JSlider(JSlider.HORIZONTAL, 1, 15, 4);
		Dimension d = noClustersSlider.getPreferredSize();
		noClustersSlider.setPreferredSize(new Dimension(d.width / 2 + 10, d.height));
		noClustersSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				noClusters = noClustersSlider.getValue();
				noClustersValueLabel.setText(noClustersSlider.getValue() + "");
			}
		});

		GridBagConstraints c = new GridBagConstraints();

		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(5, 0, 0, 40);
		clusteringPanel.add(clusteringLabel, c);

		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 30);
		clusteringPanel.add(noClustersLabel, c);

		c.gridx = 1;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		clusteringPanel.add(noClustersSlider, c);

		c.gridx = 2;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		clusteringPanel.add(noClustersValueLabel, c);
	}

	private void buildAlignmentParametersPanel() {
		alignmentParametersPanel = new JPanel();
		alignmentParametersPanel.setBorder(BorderFactory.createEtchedBorder());

		GridBagLayout alignmentParametersGBLayout = new GridBagLayout();
		alignmentParametersPanel.setLayout(alignmentParametersGBLayout);

		final GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;

		JLabel alignmentParametersLabel = new JLabel("Alignment Parameters");
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 3;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(alignmentParametersLabel, c);

		c.gridwidth = 1;
		final JCheckBox incrementLikeSubstitutionScoreCheckBox = new JCheckBox(
				"<HTML>Increment like <BR>substitution score</HTML>");
		incrementLikeSubstitutionScoreCheckBox.setSelected(false);
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(incrementLikeSubstitutionScoreCheckBox, c);

		final JSlider incrementLikeSubsitutionScoreSlider = new JSlider(JSlider.HORIZONTAL, 1, 10, 3);
		Dimension d = incrementLikeSubsitutionScoreSlider.getPreferredSize();
		incrementLikeSubsitutionScoreSlider.setPreferredSize(new Dimension(d.width / 2, d.height));
		incrementLikeSubsitutionScoreSlider.setEnabled(false);
		c.gridx = 1;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(incrementLikeSubsitutionScoreSlider, c);

		final JLabel incrementLikeSubstitutionScoreValueLabel = new JLabel("3");
		incrementLikeSubstitutionScoreValueLabel.setEnabled(false);
		c.gridx = 2;
		c.gridy = 1;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(incrementLikeSubstitutionScoreValueLabel, c);

		incrementLikeSubstitutionScoreCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (incrementLikeSubstitutionScoreCheckBox.isSelected()) {
					incrementLikeSubsitutionScoreSlider.setEnabled(true);
					incrementLikeSubstitutionScoreValueLabel.setEnabled(true);
				} else {
					incrementLikeSubsitutionScoreSlider.setEnabled(false);
					incrementLikeSubstitutionScoreValueLabel.setEnabled(false);
				}
			}
		});

		incrementLikeSubsitutionScoreSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				incrementLikeSubstitutionScoreValueLabel.setText(incrementLikeSubsitutionScoreSlider.getValue() + "");
			}
		});

		final JCheckBox indelScoreScaleFactorCheckBox = new JCheckBox("<HTML>Indel Score <BR>Scale Factor</HTML>");
		indelScoreScaleFactorCheckBox.setSelected(false);
		c.gridx = 0;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(indelScoreScaleFactorCheckBox, c);

		final JSlider indelScoreScaleFactorSlider = new JSlider(JSlider.HORIZONTAL, 1, 20, 2);
		d = indelScoreScaleFactorSlider.getPreferredSize();
		indelScoreScaleFactorSlider.setPreferredSize(new Dimension(d.width / 2, d.height));
		indelScoreScaleFactorSlider.setEnabled(false);
		c.gridx = 1;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(indelScoreScaleFactorSlider, c);

		final JLabel indelScoreScaleFactorValueLabel = new JLabel("2");
		indelScoreScaleFactorValueLabel.setEnabled(false);
		c.gridx = 2;
		c.gridy = 2;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(indelScoreScaleFactorValueLabel, c);

		indelScoreScaleFactorCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (indelScoreScaleFactorCheckBox.isSelected()) {
					indelScoreScaleFactorSlider.setEnabled(true);
					indelScoreScaleFactorValueLabel.setEnabled(true);
				} else {
					indelScoreScaleFactorSlider.setEnabled(false);
					indelScoreScaleFactorValueLabel.setEnabled(false);
				}
			}
		});

		indelScoreScaleFactorSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				indelScoreScaleFactorValueLabel.setText(indelScoreScaleFactorSlider.getValue() + "");
			}
		});

		final JCheckBox gapPenaltyCheckBox = new JCheckBox("Gap Penalty");
		gapPenaltyCheckBox.setSelected(false);
		c.gridx = 0;
		c.gridy = 3;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(gapPenaltyCheckBox, c);

		final JSlider gapPenaltySlider = new JSlider(JSlider.HORIZONTAL, 1, 10, 2);
		d = gapPenaltySlider.getPreferredSize();
		gapPenaltySlider.setPreferredSize(new Dimension(d.width / 2, d.height));
		gapPenaltySlider.setEnabled(false);
		c.gridx = 1;
		c.gridy = 3;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(gapPenaltySlider, c);

		final JLabel gapPenaltyValueLabel = new JLabel("-2");
		gapPenaltyValueLabel.setEnabled(false);
		c.gridx = 2;
		c.gridy = 3;
		c.insets = new Insets(5, 0, 0, 10);
		alignmentParametersPanel.add(gapPenaltyValueLabel, c);

		gapPenaltyCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gapPenaltyCheckBox.isSelected()) {
					gapPenaltySlider.setEnabled(true);
					gapPenaltyValueLabel.setEnabled(true);
				} else {
					gapPenaltySlider.setEnabled(false);
					gapPenaltyValueLabel.setEnabled(false);
				}
			}
		});

		gapPenaltySlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				gapPenaltyValueLabel.setText("-" + gapPenaltySlider.getValue());
			}
		});

	}

	private void buildComputeAlignmentPanel() {
		computeAlignmentPanel = new JPanel();
		JButton computeAlignmentButton = new JButton("Compute Alignment");

		computeAlignmentPanel.add(computeAlignmentButton);

		computeAlignmentButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//Reset all column sorts
				numColumnSorts = 0;
				if(columnSorts != null)
					columnSorts.clear();
				else
					columnSorts = new Vector<ColumnSort>();
				for (int i = 0; i < MaxPriority; i++) {
					columnSorts.add(null);
				}
//				System.out.println("Column Sorts Size: "+columnSorts.size());
				
				if (clusterAlignmentTabbedPane != null) {
					ChangeListener[] changeListeners = clusterAlignmentTabbedPane.getChangeListeners();
					for (ChangeListener c : changeListeners) {
						clusterAlignmentTabbedPane.removeChangeListener(c);
					}
					clusterAlignmentTabbedPane.removeAll();
				}

				boolean performClustering = false;
				//If no feature set is selected, then we cannot proceed; Inform user and then return
				if (featureExtractionTypeSet.size() < 1) {
					ErrorDialog.showErrorDialog(AlignmentFrame.this,
							"No feature set is selected. Need to select at least one to proceed");
					return;
				}

				//				checkOutliers();
				if (hasOutliers) {
					return;
				}

				if (!isLogEncoded) {
					// Log is not yet encoded; computeAlignmentButton is clicked for the first time; Encode the log first
					encodeLog();
					isLogEncoded = true;
					generateEncodedActivityRandomColorMap();
					FileIO io = new FileIO();
					io.writeToFile(outputDir, "encodedActivityColorMap.txt", encodedActivityColorMap, delim);
					performClustering = true;
				}

				// Log is already encoded; Have to check what changes were made in the parameters and recompute the alignment

				if (isFeatureExtractionTypeModified()) {
					if (JOptionPane
							.showConfirmDialog(
									AlignmentFrame.this,
									"Multiple feature sets are selected. Union of these feature sets would be considered. Do you want to proceed?",
									"Confirm", JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION) {
						return;
					}
					computeFeatureSets();
					performClustering = true;
				}

				// Check if feature selection options have been changed
				if (isFeatureSelectionOptionsChanged()) {
//					System.out.println("Filtering Features");
					
					filteredFeatureSet.clear();

					// Filter frequency count first
					FeatureSelection featureSelection = new FeatureSelection(encodingLength, uniqueEncodedTraceList);
					Set<String> frequencyFilteredFeatureSet = featureSelection.getFilteredSequenceFeatureSet(
							unionFeatureSet, FilterType.FREQUENCY, (float) minFrequencyCountThreshold,
							2 * uniqueEncodedTraceList.size()).keySet();

					Set<String> frequencyInstanceFilteredFeatureSet = featureSelection
							.getFilteredSequenceFeatureSet(frequencyFilteredFeatureSet, FilterType.INSTANCE,
									minInstanceCountPercentageThreshold, 1.0f).keySet();

					filteredFeatureSet.addAll(frequencyInstanceFilteredFeatureSet);
					prevMinFrequencyCountThreshold = minFrequencyCountThreshold;
					prevMinInstanceCountPercentageThreshold = minInstanceCountPercentageThreshold;
					FileIO io = new FileIO();
					io.writeToFile(outputDir, "filteredUnionFeatureSet.txt", filteredFeatureSet);
					performClustering = true;
				}

				if (isDistanceSimilarityOptionsChanged()) {
					computeDistanceSimilarity();
					performClustering = true;
				}

				// Compute substitution/indel scores
				if ((substitutionScoreMap == null) || (indelRightGivenLeftScoreMap == null)) {
					computeSubstitutionIndelScores();
					performClustering = true;
				} else if ((!isSubstitutionScoreDerived && deriveSubstitutionScore)
						|| (!isIndelScoreDerived && deriveIndelScore)) {
					computeSubstitutionIndelScores();
					performClustering = true;
				}

				// Perform clustering if required
				if (performClustering) {
					clusterTraces();
				} else {
					if (prevNoClusters != noClusters) {
						setAlignment(msa.getAlignments(noClusters));
						prevNoClusters = noClusters;
					}
				}

			}
		});
	}

	private void buildClusterAlignmentTabbedPanePanel() {
		clusterAlignmentTabbedPane = new JTabbedPane(JTabbedPane.TOP);
		clusterAlignmentTabbedPanePanel = new JPanel();

		ScrollableGridLayout clusterAlignmentTabbedPanePanelLayout = new ScrollableGridLayout(
				clusterAlignmentTabbedPanePanel, 1, 1, 0, 0);
		clusterAlignmentTabbedPanePanel.setLayout(clusterAlignmentTabbedPanePanelLayout);

		clusterAlignmentTabbedPanePanelLayout.setPosition(clusterAlignmentTabbedPane, 0, 0);
		clusterAlignmentTabbedPanePanel.add(clusterAlignmentTabbedPane);

	}

	private void checkOutliers() {
		checkLengthBasedOutliers();
	}

	private void checkLengthBasedOutliers() {
		float averageTraceLength = 0f;
		int maxTraceLength = 0;
		int minTraceLength = Integer.MAX_VALUE;
		for (XTrace trace : log) {
			averageTraceLength += trace.size();
			if (maxTraceLength < trace.size()) {
				maxTraceLength = trace.size();
			}
			if (minTraceLength > trace.size()) {
				minTraceLength = trace.size();
			}
		}
		averageTraceLength /= log.size();
		if ((maxTraceLength > 4 * averageTraceLength) || (minTraceLength < 0.1 * averageTraceLength)) {
			ErrorDialog
					.showErrorDialog(
							this,
							"There are some outlier traces whose length differs vastly from the average length. Remove those traces with length above 4*averageTraceLength or less than 0.1*averageTraceLength and apply trace alignment.");
			hasOutliers = true;
		}
	}

	private void encodeLog() {
		/*
		 * traceIndex is used to keep track of the number of traces;
		 * totalNoEvents counts the number of events/activities in the entire
		 * event log. activitySet accumulates the set of distinct
		 * activities/events in the event log; it doesn't store the trace
		 * identifier for encoding; Encoding trace identifier is required only
		 * when any of the maximal repeat (alphabet) features is selected
		 */

		int traceIndex = 0, totalNoEvents = 0;
		Set<String> activitySet = new HashSet<String>();
		XAttributeMap attributeMap;

		for (XTrace trace : log) {
			totalNoEvents += trace.size();
//			System.out.println("Trace Size: "+trace.size());
			for (XEvent event : trace) {
				attributeMap = event.getAttributes();
				activitySet.add(attributeMap.get("concept:name").toString() + "-"
						+ attributeMap.get("lifecycle:transition").toString());
			}
		}

		try {
			EncodeActivitySet encodeActivitySet = new EncodeActivitySet(activitySet);
			encodingLength = encodeActivitySet.getEncodingLength();

			for (int i = 1; i < encodingLength; i++) {
				dash += "-";
			}

			activityCharMap = encodeActivitySet.getActivityCharMap();
			charActivityMap = encodeActivitySet.getCharActivityMap();

			/*
			 * Encode each trace to a charStream
			 */
			EncodeTraces encodeTraces = new EncodeTraces(activityCharMap, log);
			encodedTraceList = encodeTraces.getCharStreamList();

			uniqueEncodedTraceList = new ArrayList<String>();
			traceIndex = 0;
			TreeSet<Integer> encodedTraceIdenticalIndicesSet;
			for (String encodedTrace : encodedTraceList) {
				if (encodedTraceIdenticalIndicesMap.containsKey(encodedTrace)) {
					encodedTraceIdenticalIndicesSet = encodedTraceIdenticalIndicesMap.get(encodedTrace);
				} else {
					encodedTraceIdenticalIndicesSet = new TreeSet<Integer>();
					uniqueEncodedTraceList.add(encodedTrace);
				}
				encodedTraceIdenticalIndicesSet.add(traceIndex);
				encodedTraceIdenticalIndicesMap.put(encodedTrace, encodedTraceIdenticalIndicesSet);

				traceIndex++;
			}

			String[] columnNames = { "Activity Name", "Char Encoding" };
			String[][] activityCharValues = new String[activityCharMap.size()][2];
			int index = 0;
			for (String activity : activityCharMap.keySet()) {
				activityCharValues[index][0] = activity;
				activityCharValues[index++][1] = activityCharMap.get(activity);
			}
			activityCharMapTable = new JTable(activityCharValues, columnNames);
			activityCharMapTable.setAutoCreateRowSorter(true);

			FileIO io = new FileIO();
			io.writeToFile(outputDir, "charActivityMap.txt", charActivityMap, delim);
			io.writeToFile(outputDir, "activityCharMap.txt", activityCharMap, delim);
			io.writeToFile(outputDir, "encodedTraceList.txt", encodedTraceList);
			io.writeToFile(outputDir, "uniqueEncodedTraceList.txt", uniqueEncodedTraceList);
			io.writeToFile(outputDir, "encodedTraceIdenticalIndicesMap.txt", encodedTraceIdenticalIndicesMap, delim);
		} catch (ActivityOverFlowException e) {
			ErrorDialog.showErrorDialog(this, "Exception while encoding log", e);
		} catch (EncodingNotFoundException e) {
			ErrorDialog.showErrorDialog(this, "Exception while encoding log", e);
		}
	}

	private void generateEncodedActivityRandomColorMap() {
		encodedActivityColorMap = new HashMap<String, Color>();
		Color c;
		Random random = new Random();
		activityList = new ArrayList<String>();
		Set<Color> chosenColorSet = new HashSet<Color>();
		for (String encodedActivity : charActivityMap.keySet()) {
			do {
				c = new Color(random.nextInt()).brighter();
			} while ((c == Color.white) || (c == Color.black) || chosenColorSet.contains(c));
			chosenColorSet.add(c);
			encodedActivityColorMap.put(encodedActivity, c);
			activityList.add(encodedActivity);
		}

		encodedActivityColorMap.put(dash, Color.white);
		activityList.add(dash);

	}

	/**
	 * Check whether there is any change in the features to be considered for
	 * alignment;
	 * 
	 * @return boolean
	 */
	private boolean isFeatureExtractionTypeModified() {
		// If the previous selection size and current selection sizes differ, then there is a change
		if (previousFeatureExtractionTypeSet.size() != featureExtractionTypeSet.size()) {
			return true;
		} else {
			/*
			 * Now that the previous selection size and current selection size
			 * are the same, check for the selections; If there is a previous
			 * selection that is not present in the current selection, then
			 * there is a change
			 */
			for (FeatureType previousFeatureType : previousFeatureExtractionTypeSet) {
				if (!featureExtractionTypeSet.contains(previousFeatureType)) {
					return true;
				}
			}
		}

		return false;
	}

	private void computeFeatureSets() {
		context.log("Computing Feature Sets");
//		printSelectedFeatures();

		List<FeatureType> repeatFeatureTypeList = new ArrayList<FeatureType>();
		List<FeatureType> repeatAlphabetFeatureTypeList = new ArrayList<FeatureType>();
		for (FeatureType featureType : featureExtractionTypeSet) {
			switch (featureType) {
				case INDIVIDUAL_EVENT :
					originalSequenceFeatureSetMap.put(FeatureType.INDIVIDUAL_EVENT, charActivityMap.keySet());
					break;
				case KGRAM :
					originalSequenceFeatureSetMap.put(FeatureType.KGRAM, (new FeatureExtraction(encodingLength,
							encodedTraceList, featureType, kGramSize)).getSequenceFeatureSet());
					break;
				case MAXIMAL_REPEAT :
				case SUPERMAXIMAL_REPEAT :
				case NEARSUPERMAXIMAL_REPEAT :
					repeatFeatureTypeList.add(featureType);
					break;
				case MAXIMAL_REPEAT_ALPHABET :
				case SUPERMAXIMAL_REPEAT_ALPHABET :
				case NEARSUPERMAXIMAL_REPEAT_ALPHABET :
					repeatAlphabetFeatureTypeList.add(featureType);
					break;
				default :
					break;
			}
		}

		if ((repeatFeatureTypeList.size() > 0) || (repeatAlphabetFeatureTypeList.size() > 0)) {
			/*
			 * Create a new encoded trace set so that the trace indices are
			 * considered for encoding; This would be used as a distinct
			 * delimiter for combining the streams
			 */

			int traceIndex = 0, totalNoEvents = 0;
			int encodingLength = 1;
			Set<String> activitySet = new HashSet<String>();
			XAttributeMap attributeMap;
			Map<String, String> extendedActivityCharMap = null;
			Map<String, String> extendedCharActivityMap = null;

			StringBuilder combinedStringBuilder = new StringBuilder();
			for (XTrace trace : log) {
				totalNoEvents += trace.size();
				for (XEvent event : trace) {
					attributeMap = event.getAttributes();
					activitySet.add(attributeMap.get("concept:name").toString() + "-"
							+ attributeMap.get("lifecycle:transition").toString());
				}
				activitySet.add("Trace" + traceIndex);
				traceIndex++;
			}

			try {
				EncodeActivitySet encodeActivitySet = new EncodeActivitySet(activitySet);
				encodingLength = encodeActivitySet.getEncodingLength();

				extendedActivityCharMap = encodeActivitySet.getActivityCharMap();
				extendedCharActivityMap = encodeActivitySet.getCharActivityMap();

				/*
				 * Encode each trace to a charStream
				 */
				EncodeTraces encodeTraces = new EncodeTraces(extendedActivityCharMap, log);
				List<String> extendedEncodedTraceList = encodeTraces.getCharStreamList();

				Set<String> extendedEncodedTraceSet = new HashSet<String>();
				extendedEncodedTraceSet.addAll(extendedEncodedTraceList);

				traceIndex = 0;
				for (String charStream : extendedEncodedTraceSet) {
					combinedStringBuilder.append(charStream);
					combinedStringBuilder.append(extendedActivityCharMap.get("Trace" + traceIndex));
					traceIndex++;
				}
			} catch (ActivityOverFlowException e) {
				ErrorDialog.showErrorDialog(this, "Error while encoding Log", e);
			} catch (EncodingNotFoundException e) {
				ErrorDialog.showErrorDialog(this, "Error while encoding Log", e);
			}

			FeatureExtraction featureExtraction = new FeatureExtraction();
			if (repeatFeatureTypeList.size() > 0) {
				Map<FeatureType, Set<String>> repeatFeatureTypeFeatureSetMap = featureExtraction
						.getRepeatFeatureTypeFeatureSetMap(encodingLength, combinedStringBuilder.toString(),
								repeatFeatureTypeList);
				for (FeatureType featureType : repeatFeatureTypeFeatureSetMap.keySet()) {
					originalSequenceFeatureSetMap.put(featureType, getOriginalEncodedFeatureSet(encodingLength,
							repeatFeatureTypeFeatureSetMap.get(featureType), extendedCharActivityMap));
				}
			} else if (repeatAlphabetFeatureTypeList.size() > 0) {
				Map<FeatureType, Map<Set<String>, Set<String>>> repeatAlphabetFeatureTypeFeatureSetMap = featureExtraction
						.getRepeatAlphabetFeatureTypeFeatureSetMap(encodingLength, combinedStringBuilder.toString(),
								repeatAlphabetFeatureTypeList);
				for (FeatureType featureType : repeatAlphabetFeatureTypeFeatureSetMap.keySet()) {
					originalAlphabetFeatureSetMap.put(featureType, getOriginalEncodedAlphabetFeatureSetMap(
							encodingLength, repeatAlphabetFeatureTypeFeatureSetMap.get(featureType),
							extendedCharActivityMap));
				}
			}

		}

		printFeatureSets();

		computeUnionFeatureSet();

		previousFeatureExtractionTypeSet.clear();
		previousFeatureExtractionTypeSet.addAll(featureExtractionTypeSet);
	}

	private void printSelectedFeatures() {
		for (FeatureType featureType : featureExtractionTypeSet) {
			switch (featureType) {
				case INDIVIDUAL_EVENT :
					System.out.println("Individual Event");
					break;
				case KGRAM :
					System.out.println("K-Gram");
					break;
				case MAXIMAL_REPEAT :
					System.out.println("Maximal Repeat");
					break;
				case SUPERMAXIMAL_REPEAT :
					System.out.println("Super Maximal Repeat");
					break;
				case NEARSUPERMAXIMAL_REPEAT :
					System.out.println("Near Super Maximal Repeat");
					break;
				case MAXIMAL_REPEAT_ALPHABET :
					System.out.println("Maximal Repeat Alphabet");
					break;
				case SUPERMAXIMAL_REPEAT_ALPHABET :
					System.out.println("Super Maximal Repeat Alphabet");
					break;
				case NEARSUPERMAXIMAL_REPEAT_ALPHABET :
					System.out.println("Near Super Maximal Repeat Alphabet");
					break;
				default :
					System.out.println("Other");
			}
		}

	}

	private Set<String> getOriginalEncodedFeatureSet(int extendedEncodingLength, Set<String> featureSet,
			Map<String, String> extendedCharActivityMap) {
		Set<String> originalEncodedFeatureSet = new HashSet<String>();

		int featureLength;
		String symbol, originalEncodedFeature;
		for (String feature : featureSet) {
			featureLength = feature.length() / extendedEncodingLength;
			originalEncodedFeature = "";
			for (int i = 0; i < featureLength; i++) {
				symbol = feature.substring(i * extendedEncodingLength, (i + 1) * extendedEncodingLength);
				originalEncodedFeature += activityCharMap.get(extendedCharActivityMap.get(symbol));
			}
			originalEncodedFeatureSet.add(originalEncodedFeature);
		}

		return originalEncodedFeatureSet;
	}

	private Map<Set<String>, Set<String>> getOriginalEncodedAlphabetFeatureSetMap(int extendedEncodingLength,
			Map<Set<String>, Set<String>> alphabetFeatureSetMap, Map<String, String> extendedCharActivityMap) {
		Set<String> unionRepeatSet = new HashSet<String>();
		Set<String> repeatAlphabetEquivalenceClassSet;
		int repeatLength;
		String originalEncodedRepeat;
		for (Set<String> repeatAlphabet : alphabetFeatureSetMap.keySet()) {
			repeatAlphabetEquivalenceClassSet = alphabetFeatureSetMap.get(repeatAlphabet);
			for (String repeat : repeatAlphabetEquivalenceClassSet) {
				repeatLength = repeat.length() / extendedEncodingLength;
				originalEncodedRepeat = "";
				for (int i = 0; i < repeatLength; i++) {
					originalEncodedRepeat += activityCharMap.get(extendedCharActivityMap.get(repeat.substring(i
							* extendedEncodingLength, (i + 1) * extendedEncodingLength)));
				}
				unionRepeatSet.add(originalEncodedRepeat);
			}
		}

		EquivalenceClass equivalenceClass = new EquivalenceClass();
		return equivalenceClass.getAlphabetEquivalenceClassMap(encodingLength, unionRepeatSet);
	}

	private void printFeatureSets() {
		FileIO io = new FileIO();
		for (FeatureType featureType : featureExtractionTypeSet) {
			switch (featureType) {
				case INDIVIDUAL_EVENT :
					io.writeToFile(outputDir, "individualEventFeatureSet.txt", originalSequenceFeatureSetMap
							.get(FeatureType.INDIVIDUAL_EVENT));
					break;
				case KGRAM :
					io.writeToFile(outputDir, kGramSize + "GramFeatureSet.txt", originalSequenceFeatureSetMap
							.get(FeatureType.KGRAM));
					break;
				case MAXIMAL_REPEAT :
					io.writeToFile(outputDir, "maximalRepeatFeatureSet.txt", originalSequenceFeatureSetMap
							.get(FeatureType.MAXIMAL_REPEAT));
					break;
				case SUPERMAXIMAL_REPEAT :
					io.writeToFile(outputDir, "superMaximalRepeatFeatureSet.txt", originalSequenceFeatureSetMap
							.get(FeatureType.SUPERMAXIMAL_REPEAT));
					break;
				case NEARSUPERMAXIMAL_REPEAT :
					io.writeToFile(outputDir, "nearSuperMaximalRepeatFeatureSet.txt", originalSequenceFeatureSetMap
							.get(FeatureType.NEARSUPERMAXIMAL_REPEAT));
					break;
				case MAXIMAL_REPEAT_ALPHABET :
					io.writeToFile(outputDir, "maximalRepeatAlphabetFeatureSetMap.txt", originalAlphabetFeatureSetMap
							.get(FeatureType.MAXIMAL_REPEAT_ALPHABET), delim);
					break;
				case SUPERMAXIMAL_REPEAT_ALPHABET :
					io.writeToFile(outputDir, "superMaximalRepeatAlphabetFeatureSetMap.txt",
							originalAlphabetFeatureSetMap.get(FeatureType.SUPERMAXIMAL_REPEAT_ALPHABET), delim);
					break;
				case NEARSUPERMAXIMAL_REPEAT_ALPHABET :
					io.writeToFile(outputDir, "nearSuperMaximalRepeatAlphabetFeatureSetMap.txt",
							originalAlphabetFeatureSetMap.get(FeatureType.NEARSUPERMAXIMAL_REPEAT_ALPHABET), delim);
					break;
				default :
					System.out.println("Other");
			}
		}
	}

	private void computeUnionFeatureSet() {
		if (unionFeatureSet == null) {
			unionFeatureSet = new HashSet<String>();
		}

		if (filteredFeatureSet == null) {
			filteredFeatureSet = new HashSet<String>();
		}

		unionFeatureSet.clear();
		filteredFeatureSet.clear();

		if (featureExtractionTypeSet.size() > 0) {
			if (isSequenceFeatureType) {
				Set<String> unionSequenceFeatureSet = new HashSet<String>();
				for (FeatureType featureType : originalSequenceFeatureSetMap.keySet()) {
					unionSequenceFeatureSet.addAll(originalSequenceFeatureSetMap.get(featureType));
				}
				unionFeatureSet.addAll(unionSequenceFeatureSet);
			} else {
				Map<Set<String>, Set<String>> unionAlphabetFeatureSetMap = new HashMap<Set<String>, Set<String>>();
				Map<Set<String>, Set<String>> alphabetFeatureSetMap;
				Set<String> alphabetEquivalenceClassFeatureSet;
				for (FeatureType featureType : originalAlphabetFeatureSetMap.keySet()) {
					alphabetFeatureSetMap = originalAlphabetFeatureSetMap.get(featureType);
					for (Set<String> alphabet : alphabetFeatureSetMap.keySet()) {
						if (unionAlphabetFeatureSetMap.containsKey(alphabet)) {
							alphabetEquivalenceClassFeatureSet = unionAlphabetFeatureSetMap.get(alphabet);
						} else {
							alphabetEquivalenceClassFeatureSet = new TreeSet<String>();
						}
						alphabetEquivalenceClassFeatureSet.addAll(alphabetFeatureSetMap.get(alphabet));
						unionAlphabetFeatureSetMap.put(alphabet, alphabetEquivalenceClassFeatureSet);
					}
				}

				for (Set<String> alphabet : unionAlphabetFeatureSetMap.keySet()) {
					unionFeatureSet.addAll(unionAlphabetFeatureSetMap.get(alphabet));
				}
			}
		}

		filteredFeatureSet.addAll(unionFeatureSet);
		FileIO io = new FileIO();
		io.writeToFile(outputDir, "unionFeatureSet.txt", unionFeatureSet);
	}

	private boolean isFeatureSelectionOptionsChanged() {
		if ((prevMinFrequencyCountThreshold != minFrequencyCountThreshold)
				|| (prevMinInstanceCountPercentageThreshold != minInstanceCountPercentageThreshold)) {
			return true;
		}
		return false;
	}

	private boolean isDistanceSimilarityOptionsChanged() {
		//		if(isPreviousDistanceSelected && !isDistanceSelected || !isPreviousDistanceSelected && isDistanceSelected){
		//			return true;
		//		}else{

		if (isDistanceSelected && !previousDistanceMetric.equals(distanceMetric)) {
			return true;
		} else {
			if (!isDistanceSelected && !previousSimilarityMetric.equals(similarityMetric)) {
				return true;
			}
		}
		//		}
		return false;
	}

	private void computeDistanceSimilarity() {
		if (isDistanceSelected) {
			if (distanceMetric.equals("Euclidean")) {
				previousDistanceMetric = distanceMetric;
				if (isSequenceFeatureType) {
					distanceMatrix = new EuclideanDistance(encodingLength, uniqueEncodedTraceList, filteredFeatureSet,
							Normalization.NONE).getDistanceMatrix();
				} else {
					EquivalenceClass equivalenceClass = new EquivalenceClass();
					distanceMatrix = new EuclideanDistance(encodingLength, uniqueEncodedTraceList, equivalenceClass
							.getAlphabetEquivalenceClassMap(encodingLength, filteredFeatureSet), Normalization.NONE)
							.getDistanceMatrix();
				}

			}
			ArrayMatrixFileIO io = new ArrayMatrixFileIO();
			io.writeToFile(outputDir, "distanceMatrix.txt", distanceMatrix);
		} else {
			// For trace similarity, we need to pass the feature matrix;
			if (similarityMetric.equals("F-Score")) {
				previousSimilarityMetric = similarityMetric;
				if (isSequenceFeatureType) {
					featureMatrix = (new FeatureMatrix(encodingLength, uniqueEncodedTraceList, filteredFeatureSet))
							.getFeatureMatrix();
				} else {
					EquivalenceClass equivalenceClass = new EquivalenceClass();
					featureMatrix = (new FeatureMatrix(encodingLength, uniqueEncodedTraceList, equivalenceClass
							.getAlphabetEquivalenceClassMap(encodingLength, filteredFeatureSet))).getFeatureMatrix();
				}

				similarityMatrix = new TraceSimilarity(encodingLength).getFScore(featureMatrix);
			}
			ArrayMatrixFileIO io = new ArrayMatrixFileIO();
			io.writeToFile(outputDir, "similarityMatrix.txt", similarityMatrix);
		}

	}

	private void computeSubstitutionIndelScores() {
		IndelSubstitutionMatrix indelSubstitutionMatrix = new IndelSubstitutionMatrix(encodingLength, encodedTraceList);

		if ((substitutionScoreMap == null) || (!isSubstitutionScoreDerived && deriveSubstitutionScore)) {
			substitutionScoreMap = indelSubstitutionMatrix.getSubstitutionScoreMap();
			isSubstitutionScoreDerived = true;
		}

		if ((indelRightGivenLeftScoreMap == null) || (!isIndelScoreDerived && deriveIndelScore)) {
			indelRightGivenLeftScoreMap = indelSubstitutionMatrix.getIndelRightGivenLeftScoreMap();
			isIndelScoreDerived = true;
		}

		FileIO io = new FileIO();
		io.writeToFile(outputDir, "substitutionScoreMap.txt", substitutionScoreMap, delim);
		io.writeToFile(outputDir, "indelRightGivenLeftScoreMap.txt", indelRightGivenLeftScoreMap, delim);
	}

	private void clusterTraces() {
		AgglomerativeHierarchicalClustering ahc;
		String outFileName = "distance";
		if (isDistanceSelected) {
			ahc = new AgglomerativeHierarchicalClustering(distanceMatrix, ClusterType.DISTANCE,
					ClusterCriteria.MIN_VARIANCE);
		} else {
			ahc = new AgglomerativeHierarchicalClustering(similarityMatrix, ClusterType.SIMILARITY,
					ClusterCriteria.MIN_VARIANCE);
			outFileName = "similarity";
		}

		ArrayMatrixFileIO io = new ArrayMatrixFileIO();
		io.writeToFile(outputDir, outFileName + "ItemsJoined.txt", ahc.getItemsJoined());
		computeAlignment(ahc.getItemsJoined());
	}

	private boolean hasAlignmentParametersChanged() {
		if ((prevIncrementLikeSubstitutionScore != incrementLikeSubstitutionScore) || (prevGapPenalty != gapPenalty)
				|| (prevScaleFactor != scaleFactor)) {
			return true;
		}
		return false;
	}

	private void computeAlignment(int[][] itemsJoined) {
		msa = new MSA(encodingLength, uniqueEncodedTraceList, itemsJoined, substitutionScoreMap,
				indelRightGivenLeftScoreMap, incrementLikeSubstitutionScore, scaleFactor, gapPenalty, 1.2f);
		List<String[]> clusterAlignmentList = msa.getAlignments(noClusters);

		// Populate the clusterAlignment Tabbed Pane
		setAlignment(clusterAlignmentList);
	}

	@SuppressWarnings("deprecation")
	private void setAlignment(List<String[]> clusterAlignmentList) {
		alignmentList.clear();
		ChangeListener[] changeListeners = clusterAlignmentTabbedPane.getChangeListeners();
		for (ChangeListener c : changeListeners) {
			clusterAlignmentTabbedPane.removeChangeListener(c);
		}
		clusterAlignmentTabbedPane.removeAll();

		displayProperties = new DisplayProperties[noClusters];
		alignmentPanel = new AlignmentPanel[noClusters];
		alignmentNamePanel = new AlignmentNamePanel[noClusters];

		columnFilterComponent = new ColumnFilterComponent[noClusters];
		columnSortComponent = new ColumnSortComponent[noClusters];

		consensusSequence = new ConsensusSequence[noClusters];
		consensusComponent = new ConsensusComponent[noClusters];

		leftPanel = new JPanel[noClusters];
		topPanel = new JPanel[noClusters];
		bottomPanel = new JPanel[noClusters];

		clusterAlignmentPanel = new JPanel[noClusters];
		clusterAlignmentPanelLayout = new ScrollableGridLayout[noClusters];
		alignmentViewPort = new ScrollableViewport[noClusters];
		leftViewPort = new ScrollableViewport[noClusters];
		bottomViewPort = new ScrollableViewport[noClusters];
		analysisViewPort = new ScrollableViewport[noClusters];
		topViewPort = new ScrollableViewport[noClusters];
		bottomLeftViewPort = new ScrollableViewport[noClusters];
		analysisNameViewPort = new ScrollableViewport[noClusters];
		alignmentHorizontalScrollBar = new JScrollBar[noClusters];
		alignmentVerticalScrollBar = new JScrollBar[noClusters];

		LazyBoundedRangeModel vsbModel, hsbModel;

		for (int i = 0; i < noClusters; i++) {
			clusterAlignmentPanel[i] = new JPanel();
			clusterAlignmentPanel[i].setBackground(Color.white);
			clusterAlignmentPanelLayout[i] = new ScrollableGridLayout(clusterAlignmentPanel[i], 3, 4, 0, 0);

			clusterAlignmentPanelLayout[i].setRowFixed(0, true);
			clusterAlignmentPanelLayout[i].setRowFixed(2, true);
			clusterAlignmentPanelLayout[i].setRowFixed(3, true);
			//			clusterAlignmentPanelLayout[i].setRowFixed(4, true);
			clusterAlignmentPanelLayout[i].setColumnFixed(0, true);
			clusterAlignmentPanelLayout[i].setColumnFixed(2, true);

			clusterAlignmentPanel[i].setLayout(clusterAlignmentPanelLayout[i]);

			alignmentViewPort[i] = new ScrollableViewport();
			alignmentViewPort[i].setBackingStoreEnabled(false);
			alignmentViewPort[i].putClientProperty("EnableWindowBlit", null);
			alignmentViewPort[i].setBackground(Color.white);
			alignmentViewPort[i].add(new JLabel("Cluster" + i));
			clusterAlignmentPanelLayout[i].setPosition(alignmentViewPort[i], 1, 1);
			clusterAlignmentPanel[i].add(alignmentViewPort[i]);

			// add other panels
			leftViewPort[i] = new ScrollableViewport();
			leftViewPort[i].setBackingStoreEnabled(false);
			leftViewPort[i].putClientProperty("EnableWindowBlit", null);
			leftViewPort[i].setBackground(Color.white);
			leftViewPort[i].add(new JLabel("Left ViewPort"));
			clusterAlignmentPanelLayout[i].setPosition(leftViewPort[i], 0, 1);
			clusterAlignmentPanel[i].add(leftViewPort[i]);

			bottomLeftViewPort[i] = new ScrollableViewport();
			bottomLeftViewPort[i].setBackingStoreEnabled(false);
			bottomLeftViewPort[i].putClientProperty("EnableWindowBlit", null);
			bottomLeftViewPort[i].setBackground(Color.white);
			bottomLeftViewPort[i].setAlignmentX(Component.RIGHT_ALIGNMENT);
			//			bottomLeftViewPort[i].add(new JLabel("BottomLeft ViewPort"));
			clusterAlignmentPanelLayout[i].setPosition(bottomLeftViewPort[i], 0, 3);
			clusterAlignmentPanel[i].add(bottomLeftViewPort[i]);

			topViewPort[i] = new ScrollableViewport();
			topViewPort[i].setBackingStoreEnabled(false);
			topViewPort[i].putClientProperty("EnableWindowBlit", null);
			topViewPort[i].setBackground(Color.white);
			topViewPort[i].add(new JLabel("Top ViewPort"));
			clusterAlignmentPanelLayout[i].setPosition(topViewPort[i], 1, 0);
			clusterAlignmentPanel[i].add(topViewPort[i]);

			bottomViewPort[i] = new ScrollableViewport();
			bottomViewPort[i].setBackingStoreEnabled(false);
			bottomViewPort[i].putClientProperty("EnableWindowBlit", null);
			bottomViewPort[i].setBackground(Color.white);
			bottomViewPort[i].add(new JLabel("Bottom ViewPort"));
			clusterAlignmentPanelLayout[i].setPosition(bottomViewPort[i], 1, 3);
			clusterAlignmentPanel[i].add(bottomViewPort[i]);

			// Analysis name view port is dummy in original ; so removing it
			// TODO
			// AnalysisPanel is dummy in original; so removing it
			// TODO

			// add scrollbars
			alignmentHorizontalScrollBar[i] = new JScrollBar(JScrollBar.HORIZONTAL);
			hsbModel = new LazyBoundedRangeModel(false);
			alignmentHorizontalScrollBar[i].setModel(hsbModel);
			clusterAlignmentPanelLayout[i].setPosition(alignmentHorizontalScrollBar[i], 1, 2);
			clusterAlignmentPanel[i].add(alignmentHorizontalScrollBar[i]);

			alignmentViewPort[i].setHorizontalScrollbar(alignmentHorizontalScrollBar[i]);
			topViewPort[i].setHorizontalViewport(alignmentViewPort[i]);
			bottomViewPort[i].setHorizontalViewport(alignmentViewPort[i]);

			alignmentVerticalScrollBar[i] = new JScrollBar(JScrollBar.VERTICAL);
			vsbModel = new LazyBoundedRangeModel(false);
			alignmentVerticalScrollBar[i].setModel(vsbModel);
			clusterAlignmentPanelLayout[i].setPosition(alignmentVerticalScrollBar[i], 2, 1);
			clusterAlignmentPanel[i].add(alignmentVerticalScrollBar[i]);

			alignmentViewPort[i].setVerticalScrollbar(alignmentVerticalScrollBar[i]);
			leftViewPort[i].setVerticalViewport(alignmentViewPort[i]);

			clusterAlignmentTabbedPane.addTab("Cluster " + (i + 1) + " (" + clusterAlignmentList.get(i).length + ")",
					new ImageIcon(), clusterAlignmentPanel[i], "Cluster " + (i + 1));

		}
		JScrollPane scrollPane = new JScrollPane(activityCharMapTable);
		clusterAlignmentTabbedPane.addTab("Activity Char Encoding", new ImageIcon(), scrollPane,
				"Activity Char Encoding");

		clusterAlignmentTabbedPane.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if (clusterAlignmentTabbedPane.getComponents().length > 0) {
					int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
					if (alignmentList.size() > clusterNo) {
						editMenu.setEnabled(true);
						viewMenu.setEnabled(true);
						if (alignmentList.get(clusterNo).getColumnFiltering()) {
							columnFilteringMenuItem.setSelected(true);
						} else {
							columnFilteringMenuItem.setSelected(false);
						}
						if (alignmentList.get(clusterNo).getColumnSorting()) {
							columnSortingMenuItem.setSelected(true);
						} else {
							columnSortingMenuItem.setSelected(false);
						}

						if (displayProperties[clusterNo].isGapRendered()) {
							renderGapsMenuItem.setSelected(true);
						} else {
							renderGapsMenuItem.setSelected(false);
						}

						if (displayProperties[clusterNo].isEncodedActivityRendered()) {
							renderEncodedActivityMenuItem.setSelected(true);
						} else {
							renderEncodedActivityMenuItem.setSelected(false);
						}

						if (displayProperties[clusterNo].isActivityBackgroundSquare()) {
							squareMenuItem.setSelected(true);
							dotMenuItem.setSelected(false);
						} else {
							squareMenuItem.setSelected(false);
							dotMenuItem.setSelected(true);
						}
					} else {
						viewMenu.setEnabled(false);
						editMenu.setEnabled(false);
					}
				}
			}

		});
		StringBuilder seqWithoutGaps = new StringBuilder();
		String traceName;
		Set<Integer> identicalTraceIndicesSet;
		List<String> traceNameList;
		for (int i = 0; i < noClusters; i++) {
			String[] clusterAlignmentStringArray = clusterAlignmentList.get(i);
			Sequence[] sequencesInArray = new Sequence[clusterAlignmentStringArray.length];
			int clusterAlignmentLength = clusterAlignmentStringArray[0].length() / encodingLength;
			String[] seq;

			for (int j = 0; j < clusterAlignmentStringArray.length; j++) {
				seq = new String[clusterAlignmentLength];
				seqWithoutGaps.setLength(0);
				for (int k = 0; k < clusterAlignmentLength; k++) {
					seq[k] = clusterAlignmentStringArray[j].substring(k * encodingLength, (k + 1) * encodingLength);
					if (!seq[k].equals(dash)) {
						seqWithoutGaps.append(seq[k]);
					}
				}
				// Setting the name of the trace here

				traceNameList = new ArrayList<String>();
				if (encodedTraceIdenticalIndicesMap.containsKey(seqWithoutGaps.toString())) {
					identicalTraceIndicesSet = encodedTraceIdenticalIndicesMap.get(seqWithoutGaps.toString());
					for (Integer traceIndex : identicalTraceIndicesSet) {
						traceNameList.add(log.get(traceIndex).getAttributes().get("concept:name").toString().trim());
					}
					//					traceIndex = encodedTraceIdenticalIndicesMap.get(seqWithoutGaps.toString()).first();
					//					traceName = log.get(traceIndex).getAttributes().get("concept:name").toString();
				} else {
//					System.out.println(encodedTraceIdenticalIndicesMap.size()+" @ "+seqWithoutGaps.toString());
					ErrorDialog.showErrorDialog(this,
							"Couldn't find encoded trace in the traceIdenticalIndicesMap in cluster: " + i);
				}
				if (traceNameList.size() == 0) {
					traceNameList.add("No Name");
				}
				traceName = traceNameList.get(0);
				//				sequencesInArray[j] = new Sequence(encodingLength, "Trace "
				//						+ (i + 1) * (j + 1), seq);

				sequencesInArray[j] = new Sequence(encodingLength, traceName, traceNameList, seq);
				// Add the duplicate trace names etc here
			}
			Alignment alignment = new Alignment(encodingLength, sequencesInArray);
			alignment.addListener(this);

			Font font = new Font("Sans Serif", Font.PLAIN, alignment.getFontSize());
			displayProperties[i] = new DisplayProperties(encodingLength, alignment, font, getFontMetrics(font), true,
					true);
			displayProperties[i].addListener(this);

			alignmentPanel[i] = new AlignmentPanel(AlignmentFrame.this, alignment, displayProperties[i]);
			alignmentViewPort[i].setView(alignmentPanel[i]);

			alignmentNamePanel[i] = new AlignmentNamePanel(this, alignment, displayProperties[i]);
			// alignmentAnnPanel[i] = new AlignmentAnnPanel(alignment,
			// displayProperties[i]);
			// alignmentAnnPanel[i].setVisible(true);

			leftPanel[i] = new JPanel();
			leftPanel[i].setBackground(Color.white);

			// TODO the number of columns is 4 in the original
			ScrollableGridLayout layout = new ScrollableGridLayout(leftPanel[i], 1, 1, 50, 50);

			layout.setRowFixed(0, true);
			layout.setColumnFixed(0, true);
			// layout.setColumnFixed(1, true);

			leftPanel[i].setLayout(layout);
			layout.setPosition(alignmentNamePanel[i], 0, 0);
			leftPanel[i].add(alignmentNamePanel[i]);

			// layout.setPosition(alignmentAnnPanel[i], 1, 0);
			// leftPanel[i].add(alignmentAnnPanel[i]);

			leftViewPort[i].setView(leftPanel[i]);

			// The top panel contains the ruler and ruler annotations
			topPanel[i] = new JPanel();
			topPanel[i].setBackground(Color.white);
			topPanel[i].setLayout(new BoxLayout(topPanel[i], BoxLayout.Y_AXIS));

			topPanel[i].add(new ColumnAnnotationComponent(alignment, displayProperties[i]));
			topPanel[i].add(new RulerComponent(alignment, displayProperties[i]));
			topPanel[i].add(new RulerAnnotationComponent(alignment, displayProperties[i], AlignmentFrame.this));

			columnFilterComponent[i] = new ColumnFilterComponent(this, alignment, displayProperties[i], this);
			columnFilterComponent[i].setVisible(columnFiltering);
			topPanel[i].add(columnFilterComponent[i]);

			columnSortComponent[i] = new ColumnSortComponent(JOptionPane.getFrameForComponent(this), alignment,
					displayProperties[i], this);
			columnSortComponent[i].setVisible(columnSorting);
			topPanel[i].add(columnSortComponent[i]);

			topViewPort[i].setView(topPanel[i]);

			bottomPanel[i] = new JPanel();
			bottomPanel[i].setBackground(Color.white);
			bottomPanel[i].setLayout(new BoxLayout(bottomPanel[i], BoxLayout.Y_AXIS));

			consensusSequence[i] = new ConsensusSequence(this, displayProperties[i]);
			consensusComponent[i] = new ConsensusComponent(consensusSequence[i], displayProperties[i]);
			//			consensusComponent[i] = new ConsensusComponent(new ConsensusSequence(displayProperties[i]), displayProperties[i]);

			bottomPanel[i].add(consensusComponent[i]);
			bottomViewPort[i].setView(bottomPanel[i]);

			bottomLeftViewPort[i].add(new JLabel("Information Score"));
			alignmentList.add(alignment);
		}

		addAlignmentDependentMenuItem(columnFilteringMenuItem);
		addAlignmentDependentMenuItem(columnSortingMenuItem);
	}

	private void addAlignmentDependentMenuItem(JMenuItem menuItem) {
		menuItem.setEnabled(alignmentList.size() > 0);
		alignmentDependentMenuItemSet.add(menuItem);
	}

	public void setColumnFiltering(boolean columnFiltering) {
		setColumnFiltering(columnFiltering, true);
	}

	public void setColumnFiltering(boolean columnFiltering, boolean setMenu) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		// if we already are do nothing
		// if (this.columnFiltering == columnFiltering)
		// return;

		alignmentList.get(clusterNo).setColumnFiltering(columnFiltering);
		// this.columnFiltering = columnFiltering;

		if (setMenu) {
			columnFilteringMenuItem.setSelected(columnFiltering);
		}

		// if we turn column filtering on then we must turn column sorting off
		if (columnFiltering) {
			setColumnSorting(false);
		}

		// if not init do nothing else...
		if (columnFilterComponent[clusterNo] == null) {
			return;
		}

		updateCollapsed();
		columnFilterComponent[clusterNo].setVisible(columnFiltering);
	}

	public void setColumnSorting(boolean columnSorting) {
		setColumnSorting(columnSorting, true);
	}

	public void setColumnSorting(boolean columnSorting, boolean setMenu) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		// if we already are do nothing
		// if (this.columnSorting == columnSorting)
		// return;

		alignmentList.get(clusterNo).setColumnSorting(columnSorting);
		// this.columnSorting = columnSorting;

		if (setMenu) {
			columnSortingMenuItem.setSelected(columnSorting);
		}

		// if we turn column sorting on then we must turn column filtering off
		if (columnSorting) {
			setColumnFiltering(false);
		}

		// if we are before alignment loads...do nothing else...
		if (columnSortComponent[clusterNo] == null) {
			return;
		}

		updateSort();
		columnSortComponent[clusterNo].setVisible(columnSorting);
	}

	public void addColumnSort(ColumnSort columnSort, int column) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		if ((columnSort == null) || (column < 0) || (column > alignmentList.get(clusterNo).maxLength())) {
			return;
		}

		// call on alignment
		alignmentList.get(clusterNo).addColumnSort(columnSort, column);
		updateSort();
	}

	public void removeColumnSort(int column) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		// call on alignment
		alignmentList.get(clusterNo).removeColumnSort(column);
		updateSort();
	}

	public void updateSort() {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		// tell alignment to sort sequence components based on column sorts
		try {
			int originalCursorRow = displayProperties[clusterNo].getCursorRow();
			Sequence sequenceWithCursor = alignmentList.get(clusterNo).getSequence(originalCursorRow);
			alignmentList.get(clusterNo).sortBasedOnColumnSorts();
			int newCursorRow = alignmentList.get(clusterNo).getIndex(sequenceWithCursor);
			displayProperties[clusterNo].updateCursor(newCursorRow, displayProperties[clusterNo].getCursorColumn());
		} catch (Exception e) {
			ErrorDialog.showErrorDialog(this, "Error swapping sequences while soring");
		}

		// for debug print total
		// System.out.println(ColumnSort.getNumColumnSorts());
	}

	public void removeColumnFilter(int column) {
//		System.out.println("Selected in Cluster: " + clusterAlignmentTabbedPane.getSelectedIndex());
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		// call on alignment
		alignmentList.get(clusterNo).removeColumnFilter(column);

		// update collapsed based on this
		updateCollapsed();
	}

	public void addColumnFilter(ColumnFilter filter, int column) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		if ((filter == null) || (column < 0) || (column > alignmentList.get(clusterNo).maxLength())) {
			return;
		}

		// call on alignment
		alignmentList.get(clusterNo).addColumnFilter(filter, column);

		// update collapsed based on this
		updateCollapsed();
	}

	public void updateCollapsed() {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		// reset collapsed sequences based on current criteria..
		Sequence[] seqs = alignmentList.get(clusterNo).getAllSequences();

		// start by setting all to NOT collapsed and NOT hidden
		setAllSequenceCollapsed(false, false);
		// setAllSequencesHidden(false); //Cannot set as not hidden as hide
		// selected uses it.

		// if (columnFiltering) {
		if (alignmentList.get(clusterNo).getColumnFiltering()) {
			setAllSequencesHidden(false);
			Iterator<Integer> iter = alignmentList.get(clusterNo).getColumnFilterColumnIterator();
			while (iter.hasNext()) {
				Integer key = iter.next();
				int column = key.intValue();
				ColumnFilter filter = alignmentList.get(clusterNo).getColumnFilter(key);
				for (int i = 0; i < seqs.length; i++) {
					Sequence seq = seqs[i];
					String encodedActivity = seq.getEncodedActivity(column);

					if (!filter.optionTest(encodedActivity)) {
						setSequenceCollapsed(i, true, false);
						displayProperties[clusterNo].setSeqHidden(seqs[i], true);
					}
					/*
					 * else{ props.setSeqHidden(seqs[i],false); }
					 */
				}
			}
		}

		// now hide those that are hidden
		// need to repeat setSequenceCollapsed as this is also used by hide
		// selected seqs
		for (int i = 0; i < seqs.length; i++) {
			if (displayProperties[clusterNo].isSequenceHidden(i)) {
				setSequenceCollapsed(i, true, false);
			}
		}

		revalidateAndRepaintAll();
	}

	public void setAllSequenceCollapsed(boolean collapsed, boolean repaint) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		alignmentPanel[clusterNo].setAllSequenceCollapsed(collapsed, repaint);
		alignmentNamePanel[clusterNo].setAllSequenceCollapsed(collapsed, repaint);
		// TODO alignmentAnnPanel code removed
	}

	public void setAllSequencesHidden(boolean hidden) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		Sequence[] seqs = alignmentList.get(clusterNo).getAllSequences();
		for (int i = 0; i < seqs.length; i++) {
			displayProperties[clusterNo].setSeqHidden(seqs[i], hidden);
		}
	}

	public void setSequenceCollapsed(int idx, boolean collapsed, boolean repaint) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		alignmentPanel[clusterNo].setSequenceCollapsed(idx, collapsed, repaint);
		alignmentNamePanel[clusterNo].setSequenceCollapsed(idx, collapsed, repaint);
		// TODO alignmentAnnPanel code removed
	}

	public void revalidateAndRepaintAll() {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		// System.out.println("revalidateAndRepaintAll().....");
		// and NOW revalidate and repaint
		alignmentPanel[clusterNo].revalidate();
		alignmentPanel[clusterNo].repaint();

		alignmentNamePanel[clusterNo].revalidate();
		alignmentNamePanel[clusterNo].repaint();

		alignmentViewPort[clusterNo].revalidate();
		alignmentViewPort[clusterNo].repaint();

		// TODO removed alignmentAnnPanel

		columnFilterComponent[clusterNo].revalidate();
		columnFilterComponent[clusterNo].repaint();

		bottomViewPort[clusterNo].revalidate();
		bottomViewPort[clusterNo].repaint();

		clusterAlignmentPanel[clusterNo].revalidate();
		clusterAlignmentPanel[clusterNo].repaint();
	}

	public Dimension getPreferredSize() {
		return new Dimension(600, 600);
	}

	// Alignment Listener Interface

	public void alignmentNameChanged(Alignment alignment) {
		// TODO
//		System.out.println("In alignmentNameChanged");
	}

	public void alignmentSeqInserted(Alignment align, int i) {
		// TODO
//		System.out.println("In alignmentSeqInserted");
	}

	public void alignmentSeqDeleted(Alignment align, int i, Sequence sequence) {
		// TODO
//		System.out.println("In alignmentSeqDeleted");
	}

	public void alignmentSeqSwapped(Alignment align, int i, int j) {
	}

	public void alignmentSeqActivityChanged(Alignment align, Sequence sequence) {
		// TODO
		//		System.out.println("In alignmentSeqActivityChanged");
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		consensusSequence[clusterNo].recalc();
		consensusComponent[clusterNo].setConsensus(consensusSequence[clusterNo]);
	}

	// Display Properties Listener
	public void displayAnnViewChanged(DisplayProperties dp, Sequence sequence, boolean show) {
		// TODO Auto-generated method stub
//		System.out.println(" In AlignmentFrame: displayAnnViewChanged");
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		alignmentPanel[clusterNo].revalidate();
		alignmentPanel[clusterNo].repaint();
		revalidateAndRepaintAll();
		// clusterAlignmentPanel[clusterNo].revalidate();
		// clusterAlignmentPanel[clusterNo].repaint();
	}

	public void displayColorSchemeChanged(DisplayProperties dp, ColorScheme old) {
		// Nothing to be done here
//		System.out.println("In displayColorSchemeChanged");
	}

	public void displayFontChanged(DisplayProperties dp) {
		// TODO Auto-generated method stub
//		System.out.println("In displayFontChanged");
		for (int i = 0; i < displayProperties.length; i++) {
			// Font font = displayProperties[i].getFont();
			// bottomLeftViewPort[i].setView(bottomLeftPanel[i]);
			clusterAlignmentPanel[i].revalidate();
			clusterAlignmentPanel[i].repaint();
		}
	}

	public void displayGroupEditingChanged(DisplayProperties dp) {
		// TODO Auto-generated method stub
//		System.out.println("In displayGroupEditingChanged");
	}

	public void displayHighlightsChanged(DisplayProperties dp, Sequence sequence) {
		// TODO Auto-generated method stub
		// System.out.println("In displayHighlightsChanged on Single Sequence");
		getContentPane().repaint();
		getContentPane().validate();
	}

	public void displayHighlightsChanged(DisplayProperties dp, Sequence[] sequenceArray) {
		// TODO Auto-generated method stub
		// System.out.println("In displayHighlightsChanged on Sequence Array");
		getContentPane().repaint();
		getContentPane().validate();
	}

	public void displayOverwriteChanged(DisplayProperties dp) {
		// TODO Auto-generated method stub
//		System.out.println("In displayOverwriteChanged");
	}

	public void displayRenderGapsChanged(DisplayProperties dp) {
		// Nothing to be done here
	}

	public void activityBackgroundChanged(DisplayProperties dp) {
		// Nothing to be done here
	}

	public void displaySeqSelectChanged(DisplayProperties dp, Sequence sequence, boolean select) {
		// for(JPanel clusterAlignmentPanel : clusterAlignmentPanelList)
		// clusterAlignmentPanel.revalidate();
//		System.out.println("In AlignmentFrame: displaySeqSelectChanged");
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		clusterAlignmentPanel[clusterNo].revalidate();
		clusterAlignmentPanel[clusterNo].repaint();
		// for(int i = 0; i < clusterAlignmentPanel.length; i++)
		// clusterAlignmentPanel[i].revalidate();
	}

	public void incrementalVerticalScroll(int rowsToScroll) {
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		int alignH = alignmentViewPort[clusterNo].getSize().height;
		int minScrollH = alignmentVerticalScrollBar[clusterNo].getMinimum();
		int maxScrollH = alignmentVerticalScrollBar[clusterNo].getMaximum() - alignH;

		int heightOfRow = displayProperties[clusterNo].getActivityHeight();
		int value = alignmentVerticalScrollBar[clusterNo].getValue();
		value += rowsToScroll * heightOfRow;
		if (value < minScrollH) {
			value = minScrollH;
		}
		if (value > maxScrollH) {
			value = maxScrollH;
		}

		alignmentVerticalScrollBar[clusterNo].setValue(value);
	}

	public void scrollTo(int row, int column, float lowerMargin, float upperMargin) {
		scrollTo(row, column, lowerMargin, upperMargin, false);
	}

	public void scrollTo(int row, int column, float lowerMargin, float upperMargin, boolean forceScroll) {
		// if row is non negative then ensure that row is visible...
		// if column is non negative ensure that column is visible...
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		if (row < 0) {
			row = displayProperties[clusterNo].getCursorRow();
		}

		if (column < 0) {
			column = displayProperties[clusterNo].getCursorColumn();
		}

		int alignW = alignmentViewPort[clusterNo].getSize().width;
		int alignH = alignmentViewPort[clusterNo].getSize().height;
		int scrollW = alignmentHorizontalScrollBar[clusterNo].getValue();
		int scrollH = alignmentVerticalScrollBar[clusterNo].getValue();
		int minScrollW = alignmentHorizontalScrollBar[clusterNo].getMinimum();
		int maxScrollW = alignmentHorizontalScrollBar[clusterNo].getMaximum() - alignW;
		int minScrollH = alignmentVerticalScrollBar[clusterNo].getMinimum();
		int maxScrollH = alignmentVerticalScrollBar[clusterNo].getMaximum() - alignH;

		int cursorX, cursorY;
		Point p = getXYForRowColumn(row, column);
		cursorX = p.x;
		cursorY = p.y;

		if ((cursorX < scrollW) || forceScroll) {
			alignmentHorizontalScrollBar[clusterNo].setValue((int) (Math
					.max(minScrollW, cursorX - lowerMargin * alignW)));
		} else if (cursorX > (scrollW + alignW)) {
			alignmentHorizontalScrollBar[clusterNo].setValue((int) (Math
					.min(maxScrollW, cursorX - upperMargin * alignW)));
		}
		if (((cursorY < scrollH) && (row != -1)) || forceScroll) {
			alignmentVerticalScrollBar[clusterNo]
					.setValue((int) (Math.max(minScrollH, cursorY - lowerMargin * alignH)));
		} else if ((cursorY > (scrollH + alignH)) && (row != -1)) {
			alignmentVerticalScrollBar[clusterNo]
					.setValue((int) (Math.min(maxScrollH, cursorY - upperMargin * alignH)));
		}
	}

	public Point getXYForRowColumn(int row, int column) {
		// ask all sequences before this one how tall they are
		// we are looking for upper left corner to DON'T count
		// ourselves...
		int clusterNo = clusterAlignmentTabbedPane.getSelectedIndex();
		int y = 0;
		for (int r = 0; r < row; r++) {
			SequenceComponent sc = alignmentPanel[clusterNo].getSequenceComponent(r);
			y += sc.getHeight();
		}

		int x = displayProperties[clusterNo].getXForColumn(column);

		Point p = new Point(x, y);
		return p;
	}
	
	public List<String> getActivityList(){
		return activityList;
	}
	
	public Map<String, String> getCharActivityMap(){
		return charActivityMap;
	}
	
	public Map<String, Color> getEncodedActivityColorMap(){
		return encodedActivityColorMap;
	}
	
	public int getEncodingLength(){
		return encodingLength;
	}
	
	public int getNumColumnSorts(){
		return numColumnSorts;
	}
	
	public int getMaxPriority(){
		return MaxPriority;
	}
	
	public int getNextFreePriority() {
//		System.out.println("Column Sorts Size: "+columnSorts.size());
		for (int i = 1; i <= MaxPriority; i++) {
			if (columnSorts.get(i - 1) == null) {
				return i;
			}
		}
		return 1;
	}
	
	public void makeRoom(int priority) {
		// if this slot is free then do nothing
		if (columnSorts.get(priority - 1) == null) {
			return;
		}

		// if there is a free spot below this spot
		// then move all things below it down
		// otherwise move all things above it up
		int freeSpace = 0;
		for (int i = priority + 1; i <= MaxPriority; i++) {
			if (columnSorts.get(i - 1) == null) {
				freeSpace = i;
				break;
			}
		}

		// if there is a free space below then use it
		if (freeSpace > 0) {
			makeRoomBetween(priority, freeSpace, 1);
			return;
		}

		// search for free space above
		for (int i = 1; i < priority; i++) {
			if (columnSorts.get(i - 1) == null) {
				freeSpace = i;
				break;
			}
		}

		if (freeSpace > 0) {
			makeRoomBetween(priority, freeSpace, -1);
		} else {
			System.err.println("ERROR: could not find free space in column sort priority list");
		}
	}
	
	private void makeRoomBetween(int priority, int freeSpace, int dir) {
		// move stuff over...
		for (int i = freeSpace; i != priority; i -= dir) {
			// move it over
			ColumnSort cs = (ColumnSort) columnSorts.get(i - dir - 1);
			columnSorts.set(i - 1, cs);
			cs.assignPriority(i);
		}

		columnSorts.set(priority - 1, null);
	}
	
	public void incrementNumColumnSorts(){
		numColumnSorts++;
	}
	
	public void decrementNumColumnSorts(){
		numColumnSorts--;
	}
	
	// called when this column sort is (about to be) "deleted"
	public void removed(int priority) {
		columnSorts.set(priority - 1, null);
		numColumnSorts--;
	}
	
	public void setColumnSort(int priority, ColumnSort cs){
		columnSorts.set(priority - 1, cs);
	}
}
