package org.processmining.plugins.tracealignment.swingx;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * Dialog for selection of (generic) items. This relies on the toString() method
 * of the given items for display within the JList.
 */
// This is based on the SelectNamesDialog class, but with changes to allow
// handling of non-String display items via a ListCellRenderer provided by a subclass.
@SuppressWarnings("serial")
public class ItemSelectionDialog extends JDialog {
	private JList list;

	private Object selectedItems[];

	private final JLabel countLabel = new JLabel();

	public ItemSelectionDialog(Frame owner, String title, String msg, String itemListTitle, Object[] items,
			int[] selectedIndices) {
		super(owner, title, true);
		init(owner, msg, itemListTitle, items, selectedIndices);
	}

	public ItemSelectionDialog(Dialog owner, String title, String msg, String itemListTitle, Object[] items,
			int[] selectedIndicies) {
		super(owner, title, true);
		init(owner, msg, itemListTitle, items, selectedIndicies);
	}

	public Object[] getSelectedItems() {
		return selectedItems;
	}

	protected ListCellRenderer getListCellRenderer() {
		return new DefaultListCellRenderer();
	}

	protected void init(Component owner, String msg, String itemListTitle, final Object[] items, int[] selectedIndicies) {
		Border emptyBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);

		JPanel subpanel = new JPanel();
		subpanel.setBorder(emptyBorder);

		JLabel label = new JLabel(msg);
		subpanel.add(label);
		getContentPane().add(subpanel, BorderLayout.NORTH);

		subpanel = new JPanel(new BorderLayout());
		subpanel.setBorder(emptyBorder);

		// Create and populate a Model for the JList
		final DefaultListModel listModel = new DefaultListModel();
		if (items != null) {
			for (int i = 0; i < items.length; i++) {
				listModel.addElement(items[i]);
			}
		}

		// list
		list = new JList(listModel);
		if ((selectedIndicies != null) && (selectedIndicies.length > 0)) {
			list.setSelectedIndices(selectedIndicies);
		}
		list.setCellRenderer(getListCellRenderer());
		JScrollPane scrollPane = new JScrollPane(list);
		subpanel.add(scrollPane, BorderLayout.CENTER);
		JLabel listLabel = new JLabel(itemListTitle);
		subpanel.add(listLabel, BorderLayout.NORTH);
		setCountLabelText();
		subpanel.add(countLabel, BorderLayout.SOUTH);

		getContentPane().add(subpanel, BorderLayout.CENTER);

		// buttons...
		subpanel = new JPanel();
		subpanel.setBorder(emptyBorder);

		JButton selectAllButton = new JButton("Select All");
		selectAllButton.setActionCommand("selectAll");
		selectAllButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				list.getSelectionModel().setSelectionInterval(0, listModel.getSize() - 1);
			}
		});
		subpanel.add(selectAllButton);

		JButton unselectAllButton = new JButton("Unselect All");
		unselectAllButton.setActionCommand("unselectAll");
		unselectAllButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectedItems = null;
				list.getSelectionModel().clearSelection();
			}
		});
		subpanel.add(unselectAllButton);

		subpanel.add(Box.createHorizontalStrut(20));

		JButton okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleOKAction();
			}
		});
		subpanel.add(okButton);

		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectedItems = null;
				dispose();
			}
		});
		subpanel.add(cancelButton);

		getContentPane().add(subpanel, BorderLayout.SOUTH);

		list.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				setCountLabelText();
			}
		});

		pack();
		setLocationRelativeTo(owner);
	}

	private void handleOKAction() {
		Object[] currentSelection = list.getSelectedValues();

		if (currentSelection.length < 1) {
			JOptionPane.showMessageDialog(this, "You must select at least one item from list");
		} else {
			selectedItems = currentSelection;
			dispose();
		}
	}

	private void setCountLabelText() {
		int selectedCount = list.getSelectedIndices().length;
		int totalCount = list.getModel().getSize();
		countLabel.setText(selectedCount + " item(s) selected (of " + totalCount + " total)");
	}
}
