package org.processmining.plugins.tracealignment.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Arrays;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import org.processmining.plugins.tracealignment.AlignmentFrame;
import org.processmining.plugins.tracealignment.swingx.ErrorDialog;
import org.processmining.plugins.tracealignment.swingx.Utils;
import org.processmining.plugins.tracealignment.ui.Sequence.LineAnnotation;

@SuppressWarnings( { "serial", "unused" })
public class AlignmentNamePanel extends JPanel implements AlignmentListener, MouseListener, ActionListener {

	// Underlying alignment
	protected Alignment alignment;

	// Underlying display properties
	protected DisplayProperties displayProperties;
	protected org.processmining.plugins.tracealignment.AlignmentFrame owner;

	//Upon right click, a pop-up window is shown; Define attributes for the popup menu and the options in the popup menu  
	private final JPopupMenu popupMenu;
	private final JMenuItem editSequenceMenuItem;
	private final JMenuItem colorSequenceMenuItem;

	// Line annotation related menu items
	private final JMenuItem addLineAnnMenuItem;
	private final JMenuItem showLineAnnMenuItem;
	private final JMenuItem hideLineAnnMenuItem;

	private final JMenu deleteLineAnnMenu;
	private final JMenu editLineAnnMenu;

	private final JMenuItem referenceSequenceMenuItem;
	private final JMenuItem clearReferenceSequenceMenuItem;

	//	private JMenuItem sortSequenceMenuItem;
	//	private JMenuItem addNewSequenceMenuItem;

	private boolean isMousePressed = false;
	private int mouseRowPos;
	private int mouseAnnPos;
	private int lastSelectedRow = -1;

	public AlignmentNamePanel(AlignmentFrame owner, Alignment alignment, DisplayProperties props) {
		super();
		setBackground(Color.white);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		displayProperties = props;
		this.owner = owner;
		this.alignment = alignment;
		alignment.addListener(this);

		int noSequencesInAlignment = alignment.getNoSequences();
		for (int i = 0; i < noSequencesInAlignment; i++) {
			add(new SequenceNameComponent(owner, alignment.getSequence(i), props));
		}

		popupMenu = new JPopupMenu();
		editSequenceMenuItem = new JMenuItem("Edit Trace");
		editSequenceMenuItem.addActionListener(this);
		popupMenu.add(editSequenceMenuItem);

		popupMenu.addSeparator();

		colorSequenceMenuItem = new JMenuItem("Color Trace Name");
		colorSequenceMenuItem.addActionListener(this);
		popupMenu.add(colorSequenceMenuItem);

		
		popupMenu.addSeparator();

		referenceSequenceMenuItem = new JMenuItem("Set as Reference Trace");
		referenceSequenceMenuItem.addActionListener(this);
		//TODO uncomment the following lines after this functionality is implemented
//		popupMenu.add(referenceSequenceMenuItem);

		clearReferenceSequenceMenuItem = new JMenuItem("Unset Reference Trace Setting");
		clearReferenceSequenceMenuItem.addActionListener(this);
		clearReferenceSequenceMenuItem.setEnabled(false);
//		popupMenu.add(clearReferenceSequenceMenuItem);

//		popupMenu.addSeparator();

		addLineAnnMenuItem = new JMenuItem("Add Line Annotation");
		addLineAnnMenuItem.addActionListener(this);
		popupMenu.add(addLineAnnMenuItem);

		showLineAnnMenuItem = new JMenuItem("Show Line Annotation");
		showLineAnnMenuItem.addActionListener(this);
		popupMenu.add(showLineAnnMenuItem);

		hideLineAnnMenuItem = new JMenuItem("Hide Line Annotation");
		hideLineAnnMenuItem.addActionListener(this);
		popupMenu.add(hideLineAnnMenuItem);

		editLineAnnMenu = new JMenu("Edit Line Annotation");
		editLineAnnMenu.addActionListener(this);
		popupMenu.add(editLineAnnMenu);

		deleteLineAnnMenu = new JMenu("Delete Line Annotation");
		deleteLineAnnMenu.addActionListener(this);
		popupMenu.add(deleteLineAnnMenu);

		addMouseListener(this);

		addMouseMotionListener(new MouseMotionListener() {
			public void mouseDragged(MouseEvent e) {
				handleMouseDrag(e);
			}

			public void mouseMoved(MouseEvent e) {
			}
		});

	}

	private void handleMouseDrag(MouseEvent e) {
		shiftSequence(e);
	}

	private void shiftSequence(MouseEvent e) {
		if (editWarning()) {
			return;
		}
		int index = findRow(e.getX(), e.getY());
		if (index == mouseRowPos) {
			return;
		}

		int shift = index - mouseRowPos;
		boolean isInSelection = false;
		Sequence seq = alignment.getSequence(mouseRowPos);

		Sequence[] seqs = displayProperties.getAllSelected();
		for (int i = 0; i < seqs.length; i++) {
			if (seq.equals(seqs[i])) {
				isInSelection = true;
				break;
			}
		}
		if (!displayProperties.isGroupEditing() || (displayProperties.getSelectedCount() <= 0) || !isInSelection) {
			seqs = new Sequence[1];
			seqs[0] = alignment.getSequence(mouseRowPos);
		}

		try {
			int[] indexArray = new int[seqs.length];

			for (int i = indexArray.length - 1; i >= 0; i--) {
				indexArray[i] = alignment.getIndex(seqs[i]);
			}
			Arrays.sort(indexArray);
			if ((shift < 0) && (indexArray[0] + shift >= 0)) {
				for (int i = 0; i < indexArray.length; i++) {
					alignment.swapSequence(indexArray[i], indexArray[i] + shift);
				}
				mouseRowPos = index;
			} else if ((shift > 0) && (indexArray[indexArray.length - 1] + shift < alignment.getNoSequences())) {
				for (int i = indexArray.length - 1; i >= 0; i--) {
					alignment.swapSequence(indexArray[i], indexArray[i] + shift);
				}
				mouseRowPos = index;
			} else {
				Toolkit.getDefaultToolkit().beep();
			}
		} catch (Exception exp) {
			ErrorDialog.showErrorDialog(this, "Unable to move " + "sequence in alignment.", exp);
		}
	}

	private boolean editWarning() {
		if (displayProperties.isMapping()) {
			JOptionPane.showMessageDialog(owner, "You cannot edit an alignment while columns are hidden");
			return true;
		}
		return false;
	}

	private int findRow(int x, int y) {
		Component c = findComponentAt(x, y);
		Component[] comps = getComponents();
		int index;

		for (index = comps.length - 1; index >= 0; index--) {
			if (comps[index] == c) {
				break;
			}
		}
		return index;
	}

	private boolean maybeShowPopup(MouseEvent e) {
		if (e.isPopupTrigger()) {
			// really this should operate on the SELECTED row yes???
			SequenceNameComponent sequenceNameComponent = (SequenceNameComponent) getComponent(mouseRowPos);
			Sequence sequence = sequenceNameComponent.getSequence();

			//			System.out.println("popup: mouseAnnPos = " + mouseAnnPos);            

			// rebuild the menus...
			rebuildAnnMenu(sequence, "editLineAnnotation", editLineAnnMenu);
			rebuildAnnMenu(sequence, "deleteLineAnnotation", deleteLineAnnMenu);

			enableShowHideLineAnnotationMenus(sequence);

			// Reference Sequence Clear item enabled/disabled ->
			clearReferenceSequenceMenuItem.setEnabled(alignment.getRefSequence() != null);

			int popupHeight = (int) (popupMenu.getComponent()).getSize().getHeight();
			int aHeight = (int) owner.getSize().getHeight();
			int yOffset = -popupHeight * e.getY() / aHeight;

			yOffset = Math.min(0, Math.max(yOffset, -popupHeight));
			Utils.showPopup(popupMenu, e.getComponent(), e.getX(), e.getY() + yOffset);
			return true;
		}
		return false;
	}

	private void rebuildAnnMenu(Sequence seq, String action, JMenu menu) {
		int noLineAnnotations = seq.getLineAnnotationsCount();
		menu.setEnabled(noLineAnnotations > 0);
		menu.removeAll();

		for (int i = 0; i < noLineAnnotations; i++) {
			String lineAnnotationName = seq.getLineAnnotation(i).getName();
			JMenuItem menuItem = new JMenuItem(lineAnnotationName);
			menuItem.setName("" + i);
			menuItem.setActionCommand(action);
			menuItem.addActionListener(this);
			menu.add(menuItem);
		}
	}

	public void setAllSequenceCollapsed(boolean collapsed, boolean repaint) {
		Component[] components = getComponents();
		for (int i = 0; i < components.length; i++) {
			Component component = components[i];
			if (component instanceof SequenceNameComponent) {
				SequenceNameComponent snc = (SequenceNameComponent) component;
				snc.setCollapsed(collapsed);
			}
		}

		if (repaint) {
			revalidate();
			repaint();
		}
	}

	public void setSequenceCollapsed(int idx, boolean collapsed, boolean repaint) {
		SequenceNameComponent snc = getSequenceNameComponent(idx);
		snc.setCollapsed(collapsed);

		if (repaint) {
			revalidate();
			repaint();
		}
	}

	private void enableShowHideLineAnnotationMenus(Sequence sequence) {
		List<LineAnnotation> visibleLineAnnotationList = displayProperties.getVisibleLineAnnotations(sequence);
		List<LineAnnotation> hiddenLineAnnotationList = displayProperties.getHiddenLineAnnotations(sequence);

		showLineAnnMenuItem.setEnabled(!hiddenLineAnnotationList.isEmpty());
		hideLineAnnMenuItem.setEnabled(!visibleLineAnnotationList.isEmpty());
	}

	private boolean clickInLineAnnotationExpandBox(MouseEvent e, int index) {
		SequenceNameComponent sequenceNameComponent = getSequenceNameComponent(index);

		// if it's inside the expand box...
		Point p = sequenceNameComponent.getLocation();

		Point localP = e.getPoint();
		localP.translate(-p.x, -p.y);

		return sequenceNameComponent.pointInExpandBox(localP);
	}

	public SequenceNameComponent getSequenceNameComponent(int index) {
		Component[] components = getComponents();
		return (SequenceNameComponent) components[index];
	}

	// AlignmentListener interface implementation
	public void alignmentSeqDeleted(Alignment alignment, int i, Sequence sequence) {
		if (alignment != this.alignment) {
			throw new RuntimeException("bound to incorrect alignment");
		}
		remove(i);
		revalidate();
		repaint();
	}

	public void alignmentSeqInserted(Alignment alignment, int i) {
		if (alignment != this.alignment) {
			throw new RuntimeException("bound to incorrect alignment");
		}
		add(new SequenceNameComponent(owner, alignment.getSequence(i), displayProperties), i);
		revalidate();
		repaint();
	}

	public void alignmentSeqSwapped(Alignment alignment, int i, int j) {
		if (alignment != this.alignment) {
			throw new RuntimeException("bound to incorrect alignment");
		}
		((SequenceNameComponent) getComponent(i)).setSequence(alignment.getSequence(i));
		((SequenceNameComponent) getComponent(j)).setSequence(alignment.getSequence(j));

		//TODO added revalidate and repaint
		revalidate();
		repaint();
	}

	public void alignmentNameChanged(Alignment alignment) {
		// Nothing to be done here
	}

	public void alignmentSeqActivityChanged(Alignment alignment, Sequence sequence) {
		// Nothing to be done here
	}

	//MouseListener interface implementation
	public void mousePressed(MouseEvent e) {
		mouseRowPos = findRow(e.getX(), e.getY());
		isMousePressed = mouseRowPos >= 0;
		if (!isMousePressed) {
			return;
		}
		SequenceNameComponent sequenceNameComponent = (SequenceNameComponent) getComponent(mouseRowPos);
		Point p = sequenceNameComponent.getLocation();

		mouseAnnPos = sequenceNameComponent.findLineAnnotation(e.getX() - p.x, e.getY() - p.y);

		if (maybeShowPopup(e)) {
			return;
		} else if (clickInLineAnnotationExpandBox(e, mouseRowPos)) {
			Sequence seq = alignment.getSequence(mouseRowPos);
			// any number of clicks inside the expand box of
			// a line annotation expands it
			if (displayProperties.getAnnView(seq)) {
				displayProperties.setAnnView(seq, false);
//				System.out.println("In LineAnnotationExpandBox; Need to implement it");
				//TODO commented the redrawing in the frame; need to incorporate it 
				//				owner.redrawAlignmentAnnPanel();
			} else {
				displayProperties.setAnnView(seq, true);
			}
		} else if (e.getClickCount() >= 2) {
			sequenceNameComponent.edit(sequenceNameComponent.getLocationOnScreen());
		} else if (e.getClickCount() == 1) {
			Sequence seq = alignment.getSequence(mouseRowPos);
			// meta down
			if (e.isControlDown() || e.isMetaDown()) {
				if (displayProperties.getSeqSelect(seq)) {
					displayProperties.setSeqSelect(seq, false);
				} else {
					displayProperties.setSeqSelect(seq, true);
					lastSelectedRow = mouseRowPos;
				}
			} else if (e.isShiftDown() && (lastSelectedRow >= 0) && (lastSelectedRow < alignment.getNoSequences())) {
				displayProperties.clearSelections();

				int start = Math.min(mouseRowPos, lastSelectedRow);
				int end = Math.max(mouseRowPos, lastSelectedRow);

				for (int i = start; i <= end; i++) {
					displayProperties.setSeqSelect(alignment.getSequence(i), true);
				}
			} else {
				displayProperties.clearSelections();
				displayProperties.setSeqSelect(seq, true);
				lastSelectedRow = mouseRowPos;
			}
		}
	}

	public void mouseReleased(MouseEvent e) {
		if (maybeShowPopup(e)) {
			return;
		}
		if (popupMenu.isVisible()) {
			return;
		}

		isMousePressed = false;
	}

	public void mouseClicked(MouseEvent e) {
		// Nothing to be done here
	}

	public void mouseEntered(MouseEvent e) {
		// Nothing to be done here
	}

	public void mouseExited(MouseEvent e) {
		// Nothing to be done here
	}

	//ActionListener interface implementation
	@SuppressWarnings("deprecation")
	public void actionPerformed(ActionEvent e) {
		if (!isMousePressed) {
			return;
		}

		if (e.getSource() == editSequenceMenuItem) {
			Sequence seq = alignment.getSequence(mouseRowPos);
			EditSequenceDialog editSequenceDialog = new EditSequenceDialog(owner, seq);
			editSequenceDialog.show();
		} else if (e.getSource() == colorSequenceMenuItem) {
			Sequence seq = alignment.getSequence(mouseRowPos);
			ColorSequenceDialog d = new ColorSequenceDialog(owner, seq);
			d.show();
		} else if (e.getSource() == referenceSequenceMenuItem) {
			Sequence seq = alignment.getSequence(mouseRowPos);
			setRefSequence(seq);
		} else if (e.getSource() == clearReferenceSequenceMenuItem) {
			clearRefSequence();
		} else if (e.getSource() == showLineAnnMenuItem) {
			Sequence sequence = alignment.getSequence(mouseRowPos);
			List<LineAnnotation> hiddenLineAnnotationList = displayProperties.getHiddenLineAnnotations(sequence);
			if (hiddenLineAnnotationList.isEmpty()) {
				return;
			}

			LineAnnotation[] lineAnnotationArray = new LineAnnotation[hiddenLineAnnotationList.size()];
			hiddenLineAnnotationList.toArray(lineAnnotationArray);

			LineAnnotationSelectionDialog dialog = new LineAnnotationSelectionDialog(JOptionPane
					.getFrameForComponent(owner), "Show line annotations", "Hidden line annotations",
					"Line annotations to show", lineAnnotationArray, new int[0]);
			dialog.show();

			LineAnnotation[] selectedLineAnnotationArray = dialog.getSelectedLineAnnotations();
			if ((selectedLineAnnotationArray == null) || (selectedLineAnnotationArray.length < 1)) {
				return;
			}
			for (int i = 0; i < selectedLineAnnotationArray.length; i++) {
				displayProperties.setVisibleLineAnnotation(sequence, selectedLineAnnotationArray[i]);
			}
			//TODO commenting the following line
			//			owner.enableLineAnnotationMenus();
		} else if (e.getSource() == hideLineAnnMenuItem) {
			Sequence sequence = alignment.getSequence(mouseRowPos);
			List<LineAnnotation> visibleLineAnnotationList = displayProperties.getVisibleLineAnnotations(sequence);
			if (visibleLineAnnotationList.isEmpty()) {
				return;
			}

			LineAnnotation[] lineAnnotationArray = new LineAnnotation[visibleLineAnnotationList.size()];
			visibleLineAnnotationList.toArray(lineAnnotationArray);
			LineAnnotationSelectionDialog dialog = new LineAnnotationSelectionDialog(JOptionPane
					.getFrameForComponent(owner), "Hide line annotations", "Visible line annotations",
					"Line annotations to hide", lineAnnotationArray, new int[0]);
			dialog.show();
			LineAnnotation[] selectedLineAnnotationArray = dialog.getSelectedLineAnnotations();
			if ((selectedLineAnnotationArray == null) || (selectedLineAnnotationArray.length < 1)) {
				return;
			}
			for (int i = 0; i < selectedLineAnnotationArray.length; i++) {
				displayProperties.setHiddenLineAnnotation(sequence, selectedLineAnnotationArray[i]);
			}
			//TODO commenting the following line
			//			owner.enableLineAnnotationMenus();

		} else if (e.getSource() == addLineAnnMenuItem) {
			Sequence seq = alignment.getSequence(mouseRowPos);
			EditLineAnnDialog d = new EditLineAnnDialog(JOptionPane.getFrameForComponent(owner), seq,
					displayProperties, -1);

			d.show();
		} else if (e.getActionCommand().equals("editLineAnnotation")) {
			int line = getSourceNameAsNumber(e);
			Sequence seq = alignment.getSequence(mouseRowPos);
			EditLineAnnDialog d = new EditLineAnnDialog(JOptionPane.getFrameForComponent(owner), seq,
					displayProperties, line);

			d.show();
		} else if (e.getActionCommand().equals("deleteLineAnnotation")) {
			int line = getSourceNameAsNumber(e);
			Sequence seq = alignment.getSequence(mouseRowPos);
			String name = seq.getLineAnnotation(line).getName();
			if (JOptionPane.showConfirmDialog(owner, "Delete line annotation: " + name, "Confirm Delete",
					JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
				seq.deleteLineAnnotation(line);
			}
		}
	}

	private int getSourceNameAsNumber(ActionEvent e) {
		Component c = (Component) e.getSource();
		int num = 0;
		try {
			num = Integer.parseInt(c.getName());
		} catch (NumberFormatException ex) {
			ex.printStackTrace();
		}

		return num;
	}

	public void setRefSequence(Sequence seq) {
		alignment.setRefSequence(seq);

		// GUI needs to be updated after this...
		displayProperties.fireSeqSelect(seq);
		displayProperties.updateCursor(displayProperties.getCursorRow(), displayProperties.getCursorColumn());
		validate();
	}

	public void clearRefSequence() {
		Sequence refSeq = alignment.getRefSequence();
		if (refSeq == null) {
			return;
		}
		alignment.setRefSequence(null);

		// GUI needs to be updated after this...
		displayProperties.fireSeqSelect(refSeq);
		displayProperties.updateCursor(displayProperties.getCursorRow(), displayProperties.getCursorColumn());
	}
}
