package org.processmining.plugins.tracealignment.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import org.processmining.plugins.tracealignment.AlignmentFrame;

@SuppressWarnings( { "serial", "unused", "cast" })
public class ColumnFilterComponent extends JPanel implements MouseListener, ActionListener, PopupMenuListener {

	// underlying alignment
	protected Alignment alignment;

	// display properties
	protected DisplayProperties displayProperties;

	protected AlignmentFrame alignmentFrame;

	//TODO changed this from Frame to AlignmentFrame
	protected AlignmentFrame owner;

	private JPopupMenu popupMenu = null;

	private boolean isMousePressed = false;
	private int clickedColumnIndex;

	public ColumnFilterComponent(AlignmentFrame owner, Alignment alignment, DisplayProperties displayProperties,
			AlignmentFrame alignmentFrame) {
		super();

		setBackground(Color.white);

		this.alignment = alignment;
		this.displayProperties = displayProperties;
		this.owner = owner;
		this.alignmentFrame = alignmentFrame;

		setToolTipText("Filter Sequences by Column - to turn off use View menu");

		addMouseListener(this);
	}

	public Dimension getMinimumSize() {
		return getPreferredSize();
	}

	public Dimension getMaximumSize() {
		Dimension d = getPreferredSize();
		d.width = Integer.MAX_VALUE;
		return d;
	}

	public Dimension getPreferredSize() {
		int height = displayProperties.getActivityHeight();
		// TODO changed from original; pertaining to Mapping
		int width = displayProperties.getActivityWidth() * displayProperties.getAlignment().maxLength();

		return new Dimension(width, height);
	}

	// get a position good for the columnfilter dialog for editing
	// this column
	public Point getColumnLocationRelativeToOwner(int col) {
		// get location of this column in local space
		int x = displayProperties.getXForColumn(col);
		int y = displayProperties.getActivityHeight();

		// get this location on screen
		Point p = new Point(getLocationOnScreen());
		p.translate(x, y);

		// get location of owner on screen
		Point ownerLoc = owner.getLocationOnScreen();

		// return relative location...
		p.translate(-ownerLoc.x, -ownerLoc.y);

		return p;
	}

	private void popup(MouseEvent e) {
		if (popupMenu != null) {
			popupMenu.setVisible(false);
		}

		alignment.setColumnFilterCurrentColumn(clickedColumnIndex);
		repaint();

		popupMenu = buildPopupMenu(e);
		popupMenu.addPopupMenuListener(this);
		popupMenu.setVisible(true);
	}

	// popup menu
	private JPopupMenu buildPopupMenu(MouseEvent e) {
		JPopupMenu popupMenu = new JPopupMenu();
		popupMenu.setInvoker(this);

		Point p = getColumnLocationRelativeToOwner(clickedColumnIndex);
		Point ownerLoc = owner.getLocation();
		// Shift the popup one residue to the right so we can see the current
		// column
		p.translate(ownerLoc.x + displayProperties.getActivityWidth(), ownerLoc.y);
		popupMenu.setLocation(p);

		JMenuItem menuItem;

		popupMenu.add(menuItem = new JMenuItem(ColumnFilter.getDisplayNone()));
		menuItem.setName(ColumnFilter.getDisplayNone());
		menuItem.addActionListener(this);

		popupMenu.add(menuItem = new JMenuItem(ColumnFilter.getDisplayCustom()));
		menuItem.setName(ColumnFilter.getDisplayCustom());
		menuItem.addActionListener(this);

		String[] uniqueActivitiesInColumnArray = alignment.getUniqueActivitiesInColumn(clickedColumnIndex);
		int[] allSequencesCounts = alignment.getUniqueActivityCounts(clickedColumnIndex, uniqueActivitiesInColumnArray,
				alignment.getAllSequences());
		int[] selectedSequenceCounts = null;
		if (displayProperties.getSelectedCount() > 1) {
			selectedSequenceCounts = alignment.getUniqueActivityCounts(clickedColumnIndex,
					uniqueActivitiesInColumnArray, displayProperties.getAllSelected());
		}
		for (int i = 0; i < uniqueActivitiesInColumnArray.length; i++) {
			String menuItemLabel = uniqueActivitiesInColumnArray[i] + "  (" + allSequencesCounts[i] + ")";
			if (selectedSequenceCounts != null) {
				menuItemLabel += " [" + selectedSequenceCounts[i] + "]";
			}
			popupMenu.add(menuItem = new JMenuItem(menuItemLabel));
			menuItem.addActionListener(this);
			menuItem.setName(uniqueActivitiesInColumnArray[i]);
		}

		return popupMenu;
	}

	public int findColumn(int x, int y) {
		int idx = x / displayProperties.getActivityWidth();

		idx = idx >= alignment.maxLength() ? -1 : idx;

		//TODO removed isMapping related
		return idx;
	}

	private class RenderIterator {
		private final int activityWidth;
		private int currentIndex;
		private int startIndex, endIndex;
		private int currentX;

		public RenderIterator(Rectangle clip) {
			activityWidth = displayProperties.getActivityWidth();

			int length = alignment.maxLength();

			//TODO changed pertaining to Mapping
			if (clip == null) {
				//				ErrorDialog.showErrorDialog(this, "CLip is Null");
				startIndex = 0;
				endIndex = length - 1;

				clip = new Rectangle(0, 0, getPreferredSize().width, getPreferredSize().height);
			} else {
				startIndex = (int) Math.floor(((float) clip.x) / ((float) activityWidth)) - 1;

				//TODO changed condition from < 1 to < 0 
				if (startIndex < 0) {
					startIndex = 0;
				}

				//TODO changed pertaining to Mapping
				endIndex = (int) Math.floor(((float) clip.x + clip.width) / ((float) activityWidth)) + 1;
				if (endIndex >= length) {
					endIndex = length - 1;
				}
			}
		}

		private int curr_idx() {
			return currentIndex;
		}

		public boolean hasNext() {
			return currentIndex < endIndex;
		}

		public void reset() {
			currentIndex = startIndex - 1;
			//TODO changed mapping
			currentX = (currentIndex) * activityWidth;
		}

		public int getIndex() {
			return curr_idx();
		}

		public int getX() {
			return currentX;
		}

		public int getStartIndex() {
			return startIndex;
		}

		public int getEndIndex() {
			return endIndex;
		}

		public int getStartX() {
			//TODO change mapping
			return (startIndex) * activityWidth;
		}

		public void next() {
			//TODO changed non contiguous columns
			currentIndex++;
			currentX += activityWidth;
		}

		private String getText() {

			ColumnFilter filter = alignment.getColumnFilter(new Integer(curr_idx()));
			String text;
			if (filter == null) {
				text = " ";
			} else {
				text = filter.getDisplayText();
			}
			return text;
		}
	}

	public void paint(Graphics g) {
		g.setColor(Color.white);
		Dimension d = getSize();

		//TODO changed pertaining to Mapping
		g.fillRect(0, 0, d.width, d.height);

		// figure out the range to render
		Rectangle clip = g.getClipBounds();
		int activityWidth = displayProperties.getActivityWidth();
		int activityHeight = displayProperties.getActivityHeight();
		RenderIterator ri = new RenderIterator(clip);

		int font_x = displayProperties.getFontXOffset();
		int font_y = displayProperties.getFontYOffset();

		int cfCurrCol = alignment.getColumnFilterCurrentColumn();

		// try to draw as few boxes as possible
		g.setFont(displayProperties.getFont());
		for (ri.reset(); ri.hasNext();) {
			ri.next();
			int col = ri.getIndex();

			if ((cfCurrCol > 0) && (cfCurrCol == col)) {
				g.setColor(Color.lightGray);
				g.fillRect(ri.getX(), 0, activityWidth, activityHeight - 1);
			}

			String text = ri.getText();

			g.setColor(Color.black);

			g.drawRect(ri.getX(), 0, activityWidth, activityHeight - 1);
			g.drawString(text, ri.getX() + font_x, font_y);
		}
	}

	// MouseListener interface implementation
	public void mousePressed(MouseEvent e) {
		clickedColumnIndex = findColumn(e.getX(), e.getY());

		//TODO changed the condition >=1 to >=0
		isMousePressed = clickedColumnIndex >= 0;

		if ((e.getClickCount() == 1) && isMousePressed) {
			popup(e);
		}

		if ((e.getClickCount() >= 2) && isMousePressed) {
		}
	}

	public void mouseReleased(MouseEvent arg0) {
		isMousePressed = false;
	}

	public void mouseClicked(MouseEvent arg0) {
		// Nothing to be done here
	}

	public void mouseEntered(MouseEvent arg0) {
		// Nothing to be done here
	}

	public void mouseExited(MouseEvent arg0) {
		// Nothing to be done here
	}

	// ActionListener interface implementation
	public void actionPerformed(ActionEvent e) {
		alignment.setColumnFilterCurrentColumn(clickedColumnIndex);
		repaint();

		//TODO removed the code pertaining to custom option in columnFilter
		Component c = (Component) e.getSource();
		String val = c.getName();
		if (val.equals(ColumnFilter.getDisplayNone())) {
			alignmentFrame.removeColumnFilter(clickedColumnIndex);
		} else if (val.equals(ColumnFilter.getDisplayCustom())) {
			Point p = getColumnLocationRelativeToOwner(clickedColumnIndex);

			ColumnFilter currFilter = alignment.getCurrentColumnFilter();
			ColumnFilter filter = ColumnFilterDialog.showDialog(alignmentFrame, displayProperties, clickedColumnIndex,
					currFilter, p);
			if (filter != null) {
				alignmentFrame.addColumnFilter(filter, clickedColumnIndex);
			}
		} else {
			alignmentFrame.addColumnFilter(new ColumnFilter(alignmentFrame, val), clickedColumnIndex);
		}

		popupMenu.setVisible(false);

		// reset
		alignment.setColumnFilterCurrentColumn(-1);
		repaint();
	}

	// PopupMenuListener interface implementation
	public void popupMenuCanceled(PopupMenuEvent arg0) {
		alignment.setColumnFilterCurrentColumn(-1);
		repaint();
	}

	public void popupMenuWillBecomeInvisible(PopupMenuEvent arg0) {
		alignment.setColumnFilterCurrentColumn(-1);
		repaint();
	}

	public void popupMenuWillBecomeVisible(PopupMenuEvent arg0) {
		// Nothing to be done here
	}

}
