package org.processmining.plugins.tracealignment.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.processmining.plugins.tracealignment.AlignmentFrame;

@SuppressWarnings("serial")
public class ColumnFilterDialog extends JDialog implements ActionListener, InputMethodListener {

	ColumnFilter retVal = null;

	AlignmentFrame owner;
	DisplayProperties displayProperties;
	int column;
	ColumnFilter prevVal;

	// controls
	final JButton okButton;
	JButton cancelButton;
	final JTextField valueField;
	JComboBox optionComboBox;

	// access through static method
	public static ColumnFilter showDialog(AlignmentFrame owner, DisplayProperties props, int column,
			ColumnFilter prevVal, Point location) {
		ColumnFilterDialog dialog = new ColumnFilterDialog(owner, props, column, prevVal, location);
		dialog.setVisible(true);

		// if we hit "ok" button this will be seutp...otherwise will be null
		return dialog.retVal;
	}

	// constructor
	@SuppressWarnings("deprecation")
	protected ColumnFilterDialog(AlignmentFrame owner, DisplayProperties props, int column, ColumnFilter prevVal,
			Point location) {
		//TODO changed the following line
		//		super(owner, "Column Filter Dialog");
		super();

		setModal(true);
		this.owner = owner;
		displayProperties = props;
		this.column = column;
		this.prevVal = prevVal;

		getContentPane().setLayout(new BorderLayout());

		JPanel subpanel = new JPanel();
		Dimension size = new Dimension(500, 120);
		subpanel.setPreferredSize(size);
		getContentPane().add(subpanel, BorderLayout.CENTER);

		JPanel midpanel = new JPanel(new BorderLayout());
		subpanel.add(midpanel, BorderLayout.CENTER);

		JPanel panel = new JPanel();
		midpanel.add(panel, BorderLayout.NORTH);

		panel.add(new JLabel("Show sequences where Column: " + column));

		panel = new JPanel();
		midpanel.add(panel, BorderLayout.SOUTH);

		optionComboBox = new JComboBox(ColumnFilter.getDisplayOpNames());
		panel.add(optionComboBox);

		valueField = new JTextField("", 20);
		valueField.setFont(props.getFont());
		valueField.addInputMethodListener(this);
		valueField.requestDefaultFocus();

		panel.add(valueField);

		subpanel = new JPanel();
		okButton = new JButton("Ok");
		getRootPane().setDefaultButton(okButton);
		okButton.addActionListener(this);
		subpanel.add(okButton);
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		subpanel.add(cancelButton);

		getContentPane().add(subpanel, BorderLayout.SOUTH);

		setOptions(prevVal);

		// if this location puts us off the end then move us over
		int overlap = (location.x + size.width) - owner.getWidth();
		if (overlap > 0) {
			location.translate(-overlap, 0);
		}

		Point ownerLoc = owner.getLocation();
		location.translate(ownerLoc.x, ownerLoc.y);
		setLocation(location);

		pack();

		valueField.requestFocus();
	}

	private void setOptions(ColumnFilter val) {
		if (val == null) {
			return;
		}
		optionComboBox.setSelectedIndex(val.getSelectedOption());
		valueField.setText(val.getFilterActivitiesString());
	}

	private boolean getOptions() {
		int selectedOption = optionComboBox.getSelectedIndex();
		String filterActivitiesString = valueField.getText();

		if (!hasAnyActivities(filterActivitiesString)) {
			JOptionPane.showMessageDialog(this,
					"You must spectify at least one activity. Activities should be of length "
							+ owner.getEncodingLength()
							+ ". If multiple activities are to be specified, use , as delimiter", "Error",
					JOptionPane.ERROR_MESSAGE);
			return false;
		}

		ColumnFilter opt = new ColumnFilter(owner, selectedOption, filterActivitiesString);

		// this is what we return...
		retVal = opt;
		return true;
	}

	public boolean hasAnyActivities(String s) {
		if (s.length() < owner.getEncodingLength()) {
			return false;
		} else if ((s.length() > owner.getEncodingLength()) && !s.contains(",")) {
			return false;
		}

		String[] sSplit = s.split(",");
		for (int i = 0; i < sSplit.length; i++) {
			if (sSplit[i].length() != owner.getEncodingLength()) {
				return false;
			}
		}

		return true;
	}
	
	// ActionListener interface implementation
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == cancelButton) {
			dispose();
		} else if (e.getSource() == okButton) {
			if (getOptions()) {
				dispose();
			}
		}
	}

	// InputMethodListener interface implementation
	//	@Override
	public void caretPositionChanged(InputMethodEvent arg0) {
		// TODO Auto-generated method stub

	}

	//	@Override
	public void inputMethodTextChanged(InputMethodEvent arg0) {
		// TODO Auto-generated method stub

	}

}
