package org.processmining.plugins.tracealignment.ui;

import org.processmining.plugins.tracealignment.AlignmentFrame;
import org.processmining.plugins.tracealignment.swingx.ErrorDialog;

@SuppressWarnings("cast")
public class ColumnSort {
	public static final int EqualsThisLetter = 0;
	public static final int HasAnyOfTheseLetters = 1;
	public static final int HasNoneOfTheseLetters = 2;
	public static final int noOptions = 3;

//	public static final int MaxPriority = 10;

	public static String[] optionNames = { "Equals This Letter", "Has Any Of These Letters",
			"Has None Of These Letters", };

	String[] filterActivitiesArray; // if selectedOption == EqualsThisLetter
	// then only filterActivitiesArray[0] is
	// used
	int selectedOption;
	int priority; // 1...? the smaller the # the greater the priority

	AlignmentFrame alignmentFrame;
	// we track all instances of column sort...max of MaxPriority numbered 1
	// through MaxPriority
	// sorted in the vector in index 0..MaxPriority-1
//	private static int numColumnSorts;
//	private static Vector<ColumnSort> columnSorts;
//	static {
//		numColumnSorts = 0;
//		columnSorts = new Vector<ColumnSort>();
//		for (int i = 0; i < MaxPriority; i++) {
//			columnSorts.add(null);
//		}
//	}

	public ColumnSort(AlignmentFrame alignmentFrame, int op, String filterActivitiesString, int priority) {
		this.alignmentFrame = alignmentFrame;
		alignmentFrame.incrementNumColumnSorts();
		if (alignmentFrame.getNumColumnSorts() > alignmentFrame.getMaxPriority() + 1) {
			ErrorDialog.showErrorDialog(alignmentFrame, "ERROR: Too many column sorts, max = " + alignmentFrame.getNumColumnSorts() + 1);
			return;
		}

		setFilterActivities(op, filterActivitiesString);

		setPriority(priority);
	}

	public ColumnSort(AlignmentFrame alignmentFrame, String letter, int priority) {
		this(alignmentFrame, EqualsThisLetter, letter, priority);
	}

	public ColumnSort(AlignmentFrame alignmentFrame, String letter) {
		this(alignmentFrame, EqualsThisLetter, letter, alignmentFrame.getNextFreePriority());
	}

	public void setFilterActivities(int op, String filterActivitiesString) {
		if ((op < 0) || (op >= noOptions)) {
			op = 0;
		}
		if ((filterActivitiesString == null) || (filterActivitiesString.length() < 1)) {
			filterActivitiesString = new String("!");
		}

		selectedOption = op;
		filterActivitiesArray = getArrayFromString(filterActivitiesString);
	}

	public void setPriority(int priority) {
		if ((priority <= 0) || (priority > alignmentFrame.getMaxPriority())) {
			return;
		}

		// make space in list of priorities
		alignmentFrame.makeRoom(priority);
		this.priority = priority;
		alignmentFrame.setColumnSort(priority, this);
	}

	// called when this column sort is (about to be) "deleted"
	public void removed() {
		alignmentFrame.removed(priority);
	}

	public static final String[] getDisplayOpNames() {
		return optionNames;
	}

	public static final String getDisplayNone() {
		return "None";
	}

	public static final String getDisplayCustom() {
		return "Edit";
	}

	public static final String[] getDisplayCustomOpNames() {
		String[] s = new String[optionNames.length - 1];
		for (int i = 1; i < optionNames.length; i++) {
			s[i - 1] = optionNames[i];
		}
		return s;
	}

	private static String[] optionText = { " ", // actually whatever is in
			// filterActivitiesArray[0]
			"+", "-", };

	public String getDisplayText() {
		if (selectedOption == EqualsThisLetter) {
			return getFilterActivitiesString();
		}
		return optionText[selectedOption];
	}

	public static String[] getArrayFromString(String filterActivitiesString) {
		// count how many activities

		String[] filterActivitiesStringSplit = filterActivitiesString.split(",");
		return filterActivitiesStringSplit;
	}

	public int getSelectedOption() {
		return selectedOption;
	}

	public String getFilterActivitiesString() {
		StringBuilder strBuilder = new StringBuilder();

		for (int i = 0; i < filterActivitiesArray.length - 1; i++) {
			strBuilder.append(filterActivitiesArray[i]).append(",");
		}
		strBuilder.append(filterActivitiesArray[filterActivitiesArray.length - 1]);

		return strBuilder.toString();
	}

	public boolean optionTest(String encodedActivity) {
		String filterActivity;
		if ((encodedActivity == null) || (encodedActivity.length() < 1)) {
			filterActivity = "!";
		} else {
			filterActivity = encodedActivity;
		}

		if (selectedOption == EqualsThisLetter) {
			return filterActivitiesArray[0].equals(filterActivity);
		}

		if ((selectedOption == HasAnyOfTheseLetters) || (selectedOption == HasNoneOfTheseLetters)) {
			for (int i = 0; i < filterActivitiesArray.length; i++) {
				if (filterActivitiesArray[i].equals(filterActivity)) {
					return selectedOption == HasAnyOfTheseLetters;
				}
			}
			return selectedOption != HasAnyOfTheseLetters;
		}

		// we pass any unknown test...
		return true;
	}

	public int optionTestGetPriorityBit(String activity) {
		boolean passesTest = optionTest(activity);

		if (!passesTest) {
			return 0;
		}

		// highest priority is "1" , then "2" etc
		// so priority of 1 sets the HIGHEST bit
		// and then we sort DESCENDING order
		return 1 << (alignmentFrame.getMaxPriority() - priority);
	}

	public int getPriority() {
		return priority;
	}
	
	public void assignPriority(int priority){
		this.priority = priority;
	}
}
