package org.processmining.plugins.tracealignment.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.processmining.plugins.tracealignment.AlignmentFrame;

@SuppressWarnings("serial")
public class ColumnSortDialog extends JDialog implements ActionListener, InputMethodListener {

	ColumnSort retVal = null;

	AlignmentFrame owner;
	DisplayProperties displayProperties;
	int column;
	ColumnSort prevVal;

	// controls
	final JButton okButton;
	JButton cancelButton;
	final JTextField valueField;

	JComboBox optionComboBox;
	JComboBox priorityComboBox;

	// access through static method
	public static ColumnSort showDialog(org.processmining.plugins.tracealignment.AlignmentFrame owner,
			DisplayProperties props, int column, ColumnSort prevVal, Point location) {
		ColumnSortDialog dialog = new ColumnSortDialog(owner, props, column, prevVal, location);
		dialog.setLocationRelativeTo(owner);
		dialog.setVisible(true);

		// if we hit "ok" button this will be seutp...otherwise will be null
		return dialog.retVal;
	}

	// constructor
	@SuppressWarnings("deprecation")
	protected ColumnSortDialog(AlignmentFrame owner, DisplayProperties displayProperties, int column,
			ColumnSort prevVal, Point location) {
		super(JOptionPane.getFrameForComponent(owner), "Column Sort Dialog");
		setModal(true);
		this.owner = owner;
		this.displayProperties = displayProperties;
		this.column = column;
		this.prevVal = prevVal;

		getContentPane().setLayout(new BorderLayout());

		JPanel subpanel = new JPanel();
		Dimension size = new Dimension(500, 120);
		subpanel.setPreferredSize(size);
		getContentPane().add(subpanel, BorderLayout.CENTER);

		JPanel midpanel = new JPanel(new BorderLayout());
		subpanel.add(midpanel, BorderLayout.CENTER);

		JPanel panel = new JPanel();
		midpanel.add(panel, BorderLayout.NORTH);

		panel.add(new JLabel("Sort sequences where Column: " + column));

		panel = new JPanel();
		midpanel.add(panel, BorderLayout.CENTER);

		optionComboBox = new JComboBox(ColumnSort.getDisplayOpNames());
		panel.add(optionComboBox);

		valueField = new JTextField("", 20);
		valueField.setFont(displayProperties.getFont());
		valueField.addInputMethodListener(this);
		valueField.requestDefaultFocus();

		panel.add(valueField);

		panel = new JPanel();
		midpanel.add(panel, BorderLayout.SOUTH);

		panel.add(new JLabel("Priority: "));
		String[] priorityNames = { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" };
		priorityComboBox = new JComboBox(priorityNames);
		panel.add(priorityComboBox);

		subpanel = new JPanel();
		okButton = new JButton("Ok");
		getRootPane().setDefaultButton(okButton);
		okButton.addActionListener(this);
		subpanel.add(okButton);
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		subpanel.add(cancelButton);

		getContentPane().add(subpanel, BorderLayout.SOUTH);

		// setup defaults if provided...
		setOptions(prevVal);

		// if this location puts us off the end then move us over
		int overlap = (location.x + size.width) - owner.getWidth();
		if (overlap > 0) {
			location.translate(-overlap, 0);
		}

		Point ownerLoc = owner.getLocation();
		location.translate(ownerLoc.x, ownerLoc.y);
		setLocation(location);

		pack();

		valueField.requestFocus();
	}

	private void setOptions(ColumnSort columnSort) {
		if (columnSort == null) {
			return;
		}
		optionComboBox.setSelectedIndex(columnSort.getSelectedOption());
		valueField.setText(columnSort.getFilterActivitiesString());
		priorityComboBox.setSelectedIndex(columnSort.getPriority() - 1);
	}

	private boolean getOptions() {
		int selectedOption = optionComboBox.getSelectedIndex();
		String filterActivitiesString = valueField.getText();
		int priority = priorityComboBox.getSelectedIndex() + 1;

		if (!hasAnyActivities(filterActivitiesString)) {
			JOptionPane.showMessageDialog(this,
					"You must spectify at least one activity. Activities should be of length "
							+ owner.getEncodingLength()
							+ ". If multiple activities are to be specified, use , as delimiter.", "Error",
					JOptionPane.ERROR_MESSAGE);
			return false;
		}

		ColumnSort columnSort = new ColumnSort(owner, selectedOption, filterActivitiesString, priority);

		// this is what we return...
		retVal = columnSort;
		return true;
	}

	public boolean hasAnyActivities(String s) {
		if (s.length() < owner.getEncodingLength()) {
			return false;
		} else if ((s.length() > owner.getEncodingLength()) && !s.contains(",")) {
			return false;
		}

		String[] sSplit = s.split(",");
		for (int i = 0; i < sSplit.length; i++) {
			if (sSplit[i].length() != owner.getEncodingLength()) {
				return false;
			}
		}

		return true;
	}
	// ActionListener interface implementation
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == cancelButton) {
			dispose();
		} else if (e.getSource() == okButton) {
			if (getOptions()) {
				dispose();
			}
		}
	}

	public void caretPositionChanged(InputMethodEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void inputMethodTextChanged(InputMethodEvent arg0) {
		// TODO Auto-generated method stub

	}

}
