package org.processmining.plugins.tracealignment.ui;

import java.util.Arrays;
import java.util.List;

import org.processmining.plugins.tracealignment.AlignmentFrame;

public class ConsensusSequence extends Sequence {
//	private AlignmentFrame alignmentFrame;
	private final Alignment alignment;

	private final Sequence[] allSequencesArray;

	List<String> activityList;

	int[][] activityFrequencyInColumnTable; // of dimension no of columns x no distinct activities
	int[] noActivitiesInColumn;
	double[] informationInColumnGapsExcluded;
	double[] informationInColumnGapsIncluded;

	int[] predominantActivityIndex;

	// constructor
	public ConsensusSequence(AlignmentFrame alignmentFrame, DisplayProperties props) {
		this(alignmentFrame, props, props.getAlignment().getAllSequences());
	}

	protected ConsensusSequence(AlignmentFrame alignmentFrame, DisplayProperties displayProperties, Sequence[] seqs) {
//		this.alignmentFrame = alignmentFrame;
		allSequencesArray = seqs;
		//		this.displayProperties = displayProperties;
		alignment = displayProperties.getAlignment();

		initialize("Consensus", null);
		// System.out.println("constructor: length=" +seqs.length);
		activityList = alignmentFrame.getActivityList();
		recalc();
	}

	public void recalc() {
		generateFrequencyTable();
		generateInformationTable();

		String dash = "-";
		for (int i = 1; i < alignment.getEncodingLength(); i++) {
			dash += "-";
		}

		int noColumns = activityFrequencyInColumnTable.length;
		String[] predominantActivitySeq = new String[noColumns];
		predominantActivityIndex = new int[noColumns];
		for (int k = noColumns - 1; k >= 0; k--) {
			predominantActivitySeq[k] = dash;
			int maxCount = 0;
			for (int i = activityFrequencyInColumnTable[k].length - 1; i >= 0; i--) {
				if (activityFrequencyInColumnTable[k][i] >= maxCount) {
					maxCount = activityFrequencyInColumnTable[k][i];
					predominantActivitySeq[k] = activityList.get(i);
					predominantActivityIndex[k] = i;
				}
			}
		}

		//		generateSimilarityTable(allSeq);
		//		generatePredominantResidues();

		setActivity(predominantActivitySeq);
	}

	private void generateFrequencyTable() {
		activityFrequencyInColumnTable = new int[alignment.maxLength()][activityList.size()];

		for (int k = activityFrequencyInColumnTable.length - 1; k >= 0; k--) {
			Arrays.fill(activityFrequencyInColumnTable[k], 0);
		}

		noActivitiesInColumn = new int[activityFrequencyInColumnTable.length];
		Arrays.fill(noActivitiesInColumn, 0);

		Sequence seq;
		for (int i = allSequencesArray.length - 1; i >= 0; i--) {
			seq = allSequencesArray[i];
			for (int k = seq.getLengthWithGaps() - 1; k >= 0; k--) {
				activityFrequencyInColumnTable[k][activityList.indexOf(seq.getEncodedActivity(k))]++;
				// TODO something wrong here; at the end of the iterations,
				// noActivitiesInCol would be just equal to to the no of
				// Sequences; Ideally should ignore gaps in the column
				noActivitiesInColumn[k]++;
			}
		}
	}

	@SuppressWarnings("cast")
	private void generateInformationTable() {
		double gapsExcludedProbability;
		double gapsIncludedProbability;
		double entropyGapsExcluded = 0;
		double entropyGapsIncluded = 0;
		double maxEntropyGapsExcluded = 0;
		double maxEntropyGapsIncluded = 0;

		informationInColumnGapsExcluded = new double[alignment.maxLength()];
		informationInColumnGapsIncluded = new double[alignment.maxLength()];

		for (int i = activityFrequencyInColumnTable.length - 1; i >= 0; i--) {
			entropyGapsExcluded = 0;
			entropyGapsIncluded = 0;
			for (int k = activityList.size() - 1; k >= 0; k--) {
				if ((activityFrequencyInColumnTable[i][k] > 0) && (noActivitiesInColumn[i] > 0)) {

					gapsIncludedProbability = (double) activityFrequencyInColumnTable[i][k]
							/ (double) noActivitiesInColumn[i];
					entropyGapsIncluded -= gapsIncludedProbability * log2(gapsIncludedProbability);

					// assumes that dash is the last activity in the list
					if (k < (activityList.size() - 1)) {
						gapsExcludedProbability = (double) activityFrequencyInColumnTable[i][k]
								/ (double) (noActivitiesInColumn[i] - activityFrequencyInColumnTable[i][activityList
										.size() - 1]);
						entropyGapsExcluded -= gapsExcludedProbability * log2(gapsExcludedProbability);
					}

				}
			}

			maxEntropyGapsExcluded = log2((double) (activityList.size() - 1)); // Fixed an obvious error that was here. Note this formula still treats B and X etc as real amino acids!
			maxEntropyGapsIncluded = log2((double) (activityList.size())); // Made the same fix here. This is fine now. Much better than before. - SS

			informationInColumnGapsExcluded[i] = 1.0 - entropyGapsExcluded / maxEntropyGapsExcluded;
			informationInColumnGapsIncluded[i] = 1.0 - entropyGapsIncluded / maxEntropyGapsIncluded;
		}

		//TODO removed the randomized gaps information code
	}

	public double log2(double x) {
		return Math.log(x) / Math.log(2.0);
	}

	public double[] getInformationInColumnGapsExcluded() {
		return informationInColumnGapsExcluded;
	}

	public double[] getInformationInColumnGapsIncluded() {
		return informationInColumnGapsIncluded;
	}
}
