package org.processmining.plugins.tracealignment.ui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import org.processmining.plugins.tracealignment.AlignmentFrame;
import org.processmining.plugins.tracealignment.swingx.ErrorDialog;
import org.processmining.plugins.tracealignment.swingx.Utils;

@SuppressWarnings( { "serial", "unused" })
public class EditActivityAnnDialog extends JDialog implements CaretListener {

	private final AlignmentFrame owner;
	private final Sequence sequence;
	private final Sequence.ColumnAnnotation columnAnnotation;
	private final Alignment alignment;
	private final DisplayProperties displayProperties;

	int index;

	public EditActivityAnnDialog(AlignmentFrame owner, DisplayProperties props, Alignment alignment, Sequence s,
			int index) {
		super(JOptionPane.getFrameForComponent(owner), "Edit Activity Annotation: " + s.getName()
				+ " for alignment position " + index);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		sequence = s;
		this.index = index;
		columnAnnotation = s.getColumnAnnotation(index);
		this.owner = owner;
		this.alignment = alignment;
		displayProperties = props;

		init();
		setSize(400, 300);
		pack();
		setLocationRelativeTo(this.owner);
	}

	private void init() {
		JPanel subpanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();

		subpanel.setLayout(gb);

		int row = 0;
		String encodedActivity = sequence.getEncodedActivity(index);

		Utils.addToGridBag(subpanel, gb, new JLabel("Encoded Activity: "), 0, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.EAST);
		Utils.addToGridBag(subpanel, gb, new JLabel(encodedActivity), 1, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.CENTER);
		//		Utils.addToGridBag(subpanel, gb, new JLabel(sequence.getEncodedActivity(index)), 1, row, 1, 1, 1, 1,
		//				GridBagConstraints.BOTH, GridBagConstraints.CENTER);
		row++;

		Utils.addToGridBag(subpanel, gb, new JLabel(" "), 0, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.EAST);
		row++;

		String decodedActivity = "null";
		if (owner.getCharActivityMap().containsKey(encodedActivity)) {
			decodedActivity = owner.getCharActivityMap().get(encodedActivity);
		} else {
			ErrorDialog.showErrorDialog(owner, "Encoded Activity " + encodedActivity + " not found in charActivityMap");
		}
		Utils.addToGridBag(subpanel, gb, new JLabel("Actual Activity: "), 0, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.EAST);
		Utils.addToGridBag(subpanel, gb, new JLabel(decodedActivity), 1, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.CENTER);
		row++;

		Utils.addToGridBag(subpanel, gb, new JLabel(" "), 0, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.EAST);
		row++;

		Utils.addToGridBag(subpanel, gb, new JLabel("Alignment Index: "), 0, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.EAST);
		Utils.addToGridBag(subpanel, gb, new JLabel((index + 1) + ""), 1, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.CENTER);
		row++;

		Utils.addToGridBag(subpanel, gb, new JLabel(" "), 0, row, 1, 1, 1, 1, GridBagConstraints.BOTH,
				GridBagConstraints.EAST);
		row++;

		Utils.addToGridBag(subpanel, gb, new JLabel("Activity Index in Trace: "), 0, row, 1, 1, 1, 1,
				GridBagConstraints.BOTH, GridBagConstraints.EAST);
		Utils.addToGridBag(subpanel, gb, new JLabel(sequence.getActivityIndex(index) + ""), 1, row, 1, 1, 1, 1,
				GridBagConstraints.BOTH, GridBagConstraints.CENTER);
		row++;

		JScrollPane scroll = new JScrollPane(subpanel);
		getContentPane().add(scroll, BorderLayout.CENTER);
		subpanel.setVisible(false);
		subpanel.setVisible(true);
		//        getContentPane().setLayout(new BorderLayout());
		//TODO need to fill more; check original
	}

	public void caretUpdate(CaretEvent arg0) {
		// TODO Auto-generated method stub

	}

}
