package org.processmining.plugins.tracealignment.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.WindowConstants;

import org.processmining.plugins.tracealignment.swingx.ErrorDialog;
import org.processmining.plugins.tracealignment.swingx.FixedWidthTextField;
import org.processmining.plugins.tracealignment.swingx.Utils;

@SuppressWarnings("serial")
public class EditLineAnnDialog extends JDialog implements ActionListener {

	private final FixedWidthTextField oldSequenceAsStr, annotationName, annotationText;
	private final JButton okButton, cancelButton;
	private final Sequence sequence;
	private final DisplayProperties displayProperties;
	private final int index;

	public EditLineAnnDialog(Frame owner, Sequence sequence, DisplayProperties displayProperties, int index) {
		super(owner, "Edit Line Annotation: " + sequence.getName());
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout());

		this.sequence = sequence;
		this.displayProperties = displayProperties;
		this.index = index;

		JPanel subpanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();

		subpanel.setLayout(gb);

		String seqStr = sequence.toString();

		Sequence.LineAnnotation lineAnnotation = index >= 0 ? sequence.getLineAnnotation(index) : null;

		Utils.addToGridBag(subpanel, gb, new JLabel("Annotation Name: "), 0, 0, 1, 1, 1, 1, GridBagConstraints.NONE,
				GridBagConstraints.WEST);

		annotationName = new FixedWidthTextField(lineAnnotation != null ? lineAnnotation.getName() : "",
				lineAnnotation != null ? Math.max(lineAnnotation.getName().length(), 25) : 25);

		Utils.addToGridBag(subpanel, gb, annotationName, 1, 0, 1, 1, 1, 1, GridBagConstraints.NONE,
				GridBagConstraints.WEST);

		StringBuffer lineAnnotationText = new StringBuffer(lineAnnotation == null ? "" : lineAnnotation.getAnnotation());
		StringBuffer newLineAnnotationText = new StringBuffer("");

		for (int i = 0; i < seqStr.length(); i++) {
			newLineAnnotationText.append(' ');
		}

		for (int i = 0; (i < lineAnnotationText.length()) && (i < seqStr.length()); i++) {
			newLineAnnotationText.setCharAt(i, lineAnnotationText.charAt(i));
		}

		Utils.addToGridBag(subpanel, gb, new JLabel("Annotation Text: "), 0, 1, 1, 1, 1, 1, GridBagConstraints.NONE,
				GridBagConstraints.EAST);
		annotationText = new FixedWidthTextField(newLineAnnotationText.toString(), seqStr.length());

		Utils.addToGridBag(subpanel, gb, annotationText, 1, 1, 1, 1, 1, 1, GridBagConstraints.NONE,
				GridBagConstraints.WEST);

		Utils.addToGridBag(subpanel, gb, new JLabel("Sequence: "), 0, 2, 1, 1, 1, 1, GridBagConstraints.NONE,
				GridBagConstraints.EAST);
		oldSequenceAsStr = new FixedWidthTextField(seqStr, seqStr.length());
		oldSequenceAsStr.setEditable(false);
		Utils.addToGridBag(subpanel, gb, oldSequenceAsStr, 1, 2, 1, 1, 1, 1, GridBagConstraints.NONE,
				GridBagConstraints.WEST);

		Utils.addClipboardBindings(oldSequenceAsStr);
		Utils.addClipboardBindings(annotationName);
		Utils.addClipboardBindings(annotationText);

		JScrollPane scrollPane = new JScrollPane(subpanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);

		scrollPane.getHorizontalScrollBar().setUnitIncrement(annotationText.getCharWidth());

		getContentPane().add(scrollPane, BorderLayout.CENTER);

		subpanel = new JPanel();
		okButton = new JButton("OK");
		okButton.addActionListener(this);
		subpanel.add(okButton);
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		subpanel.add(cancelButton);
		getContentPane().add(subpanel, BorderLayout.SOUTH);

		setLocationRelativeTo(owner);
		pack();
		Dimension d = getSize();

		setSize(600, d.height);
	}

	public StringBuffer trimEnd(StringBuffer sb) {
		while ((sb.length() > 0) && (sb.charAt(sb.length() - 1) == ' ')) {
			sb.deleteCharAt(sb.length() - 1);
		}
		return sb;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == okButton) {
			String name = annotationName.getText();

			if (name.length() < 1) {
				ErrorDialog.showErrorDialog(this, "Annotation must have a name.");
				return;
			}
			String ann = trimEnd(new StringBuffer(annotationText.getText())).toString();

			if (index >= 0) {
				if (ann.length() < 1) {
					sequence.deleteLineAnnotation(index);
				} else {
					sequence.setLineAnnotation(index, name, ann);
				}
			} else {
				if (ann.length() < 1) {
					ErrorDialog.showErrorDialog(this, "Annotation cannot be empty.");
					return;
				}
				sequence.addLineAnnotation(name, ann);
			}

			// Make sure that the new line annotation is showing
//			System.out.println("calling setAnnView: " + sequence.getLineAnnotationsCount());
			displayProperties.setAnnView(sequence, true);
			dispose();
		} else if (e.getSource() == cancelButton) {
			dispose();
		}

	}

}
