package org.processmining.plugins.tracealignment.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;

import javax.swing.JPanel;

@SuppressWarnings("serial")
public class RulerComponent extends JPanel implements AlignmentListener, DisplayPropertiesListener {

	// underlying alignment
	private final Alignment alignment;

	// rendering properties
	private final DisplayProperties displayProperties;

	// size
	private int size;

	public RulerComponent(Alignment alignment, DisplayProperties displayProperties) {
		super();
		this.alignment = alignment;
		this.displayProperties = displayProperties;

		size = alignment.maxLength();

		alignment.addListener(this);
		displayProperties.addListener(this);

		setBackground(Color.white);
	}

	private void resizeToAlignment() {
		int newSize = alignment.maxLength();

		if (newSize != size) {
			size = newSize;
			revalidate();
			repaint();
		}
	}

	public Dimension getPreferredSize() {
		int height = displayProperties.getActivityHeight();
		int width;

		//TODO changed from original
		width = displayProperties.getActivityWidth() * size;

		return new Dimension(width, height);
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		int y = 0;

		int fontY = y + displayProperties.getFontYOffset() - 2;
		int activityHeight = displayProperties.getActivityHeight();
		int activityWidth = displayProperties.getActivityWidth();

		int lineHeight = activityHeight - displayProperties.getFontMetrics().getMaxAscent()
				- displayProperties.getFontMetrics().getMaxDescent();

		int lineYStart = y + activityHeight - lineHeight;

		FontMetrics fm = displayProperties.getFontMetrics();

		g.setFont(displayProperties.getFont());
		g.setColor(Color.black);

		// for now this ruler LIES...it just draws the MAPPED index range...

		// When dealing with non-contiguous columns, we can't use the index at all
		// as it does not line up properly.
		// Instead, we'll just tick off ruler marks at a regular interval.
		//TODO changed from original the if condition
		//       	int width = getSize().width;
		//       	int tickIncrement = 5 * activityWidth - 1;
		//        System.out.println(tickIncrement+" @ "+width);
		//       	for (int x = tickIncrement; x < width; x += tickIncrement) {
		//			// we don't draw the #s cause they are misleading
		//			g.drawLine(x, lineYStart, x, lineYStart + lineHeight);
		//		}

		int maxLength = displayProperties.getAlignment().maxLength();

		for (int index = maxLength; index > 0; index--) {
			if (index % 4 == 0) {
				int x = index * activityWidth - 1;

				g.drawLine(x, lineYStart, x, lineYStart + 2 * lineHeight);
				String str = Integer.toString(index);
				int strWidth = fm.stringWidth(str);

				// we don't draw the #s if we are mapping cause they are
				// misleading
				g.drawString(str, x - strWidth + 2, fontY);
			} else if (index % 2 == 0) {
				int x = index * activityWidth - 1;

				g.drawLine(x, lineYStart, x, lineYStart + lineHeight);
			}
		}

	}

	// AlignmentListener interface implementation
	public void alignmentSeqActivityChanged(Alignment alignment, Sequence sequence) {
		if (alignment != this.alignment) {
			throw new RuntimeException("Bound to incorrect alignment");
		}
		resizeToAlignment();
	}

	public void alignmentSeqDeleted(Alignment alignment, int i, Sequence sequence) {
		if (alignment != this.alignment) {
			throw new RuntimeException("Bound to incorrect alignment");
		}
		resizeToAlignment();
	}

	public void alignmentSeqInserted(Alignment alignment, int i) {
		if (alignment != this.alignment) {
			throw new RuntimeException("Bound to incorrect alignment");
		}
		resizeToAlignment();
	}

	public void alignmentNameChanged(Alignment alignment) {
		// Nothing to be done here
	}

	public void alignmentSeqSwapped(Alignment alignment, int i, int j) {
		// Nothing to be done here
	}

	// DisplayPropertiesListener interface implementation

	public void displayFontChanged(DisplayProperties dp) {
		if (dp != displayProperties) {
			throw new RuntimeException("Bound to incorrect DisplayProperties");
		}
		revalidate();
		repaint();
	}

	public void displayAnnViewChanged(DisplayProperties dp, Sequence sequence, boolean show) {
		// Nothing to be done here
	}

	public void displayColorSchemeChanged(DisplayProperties dp, ColorScheme old) {
		// Nothing to be done here
	}

	public void displayGroupEditingChanged(DisplayProperties dp) {
		// Nothing to be done here
	}

	public void displayHighlightsChanged(DisplayProperties dp, Sequence sequence) {
		// Nothing to be done here
	}

	public void displayHighlightsChanged(DisplayProperties dp, Sequence[] sequenceArray) {
		// Nothing to be done here
	}

	public void displayOverwriteChanged(DisplayProperties dp) {
		// Nothing to be done here
	}

	public void displayRenderGapsChanged(DisplayProperties dp) {
		// Nothing to be done here
	}

	public void activityBackgroundChanged(DisplayProperties dp) {
		// Nothing to be done here
	}

	public void displaySeqSelectChanged(DisplayProperties dp, Sequence sequence, boolean select) {
		// Nothing to be done here
	}

}
