package org.processmining.plugins.tracealignment.ui;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class Sequence {
	int encodingLength;
	String name;
	String[] sequenceWithGaps;
	String[] sequenceWithoutGaps;
	int[] activityIndexArray;

	List<String> duplicateProcessInstanceNameList;

	Color color;

	String dash = "-";

	String displayedAnnotationName = "SeqName";

	private final List<LineAnnotation> lineAnnotationList = new ArrayList<LineAnnotation>();
	List<ColumnAnnotation> columnAnnotationList = new ArrayList<ColumnAnnotation>();

	private final List<SequenceListener> listenersList = new ArrayList<SequenceListener>();
	//	private List annotationListenersList = new ArrayList();

	// Need this mapping to display the activity details in the edit annotation
	// dialog box
	int[] alignmentIndex2ActivityIndex;
	int[] activityIndex2AlignmentIndex;

	// sequence annotation
	private String annotation;

	private int sortValue;

	protected Sequence() {
	}

	public Sequence(int encodingLength, String name, List<String> duplicateSequenceNameList, String[] sequenceWithGaps) {
		this.encodingLength = encodingLength;
		this.name = name;
		duplicateProcessInstanceNameList = duplicateSequenceNameList;
		this.sequenceWithGaps = sequenceWithGaps;

		for (int i = 1; i < encodingLength; i++) {
			dash += "-";
		}

		setSequenceWithoutGaps();
	}

	private void setSequenceWithoutGaps() {
		int alignedSequenceLength = sequenceWithGaps.length;

		int gapCount = 0;
		for (int i = 0; i < alignedSequenceLength; i++) {
			if (sequenceWithGaps[i].equals(dash)) {
				gapCount++;
			}
		}

		sequenceWithoutGaps = new String[alignedSequenceLength - gapCount];
		activityIndexArray = new int[alignedSequenceLength];
		int i = 0, j = 0;
		for (String symbol : sequenceWithGaps) {
			if (!symbol.equals(dash)) {
				//TODO added the activity index addition here
				activityIndexArray[j++] = (i + 1);
				sequenceWithoutGaps[i++] = symbol;
			} else {
				activityIndexArray[j++] = -1;
			}
		}
	}

	protected void initialize(String name, String[] sequence) {
		this.name = name;
		annotation = null;
		setActivityInternal(sequence, null);

	}

	public int getLengthWithGaps() {
		return sequenceWithGaps.length;
	}

	public int getLengthWithoutGaps() {
		return sequenceWithoutGaps.length;
	}

	public String getSymbol(int index) {
		if (index < sequenceWithGaps.length) {
			return sequenceWithGaps[index];
		}

		return null;
	}

	public String[] getSequenceWithGaps() {
		return sequenceWithGaps;
	}

	public String[] getSequenceWithoutGaps() {
		return sequenceWithoutGaps;
	}

	public Color getColor() {
		return color;
	}

	public void addDuplicateProcessInstance(String processInstanceName) {
		duplicateProcessInstanceNameList.add(processInstanceName);
	}

	public List<String> getDuplicateProcessInstanceNameList() {
		return duplicateProcessInstanceNameList;
	}

	// get the number of line annotation
	public int getLineAnnotationsCount() {
		return lineAnnotationList.size();
	}

	// get one line annotation
	public LineAnnotation getLineAnnotation(int index) {
		return lineAnnotationList.get(index);
	}

	public LineAnnotation getLineAnnotation(String annotationName) {
		for (LineAnnotation lineAnnotation : lineAnnotationList) {
			if (lineAnnotation.getName().equals(annotationName)) {
				return lineAnnotation;
			}
		}
		return null;
	}

	public int getColumnAnnotationsCount() {
		return columnAnnotationList.size();
	}

	@SuppressWarnings("cast")
	public ColumnAnnotation[] getColumnAnnotations() {
		ColumnAnnotation[] ca = new ColumnAnnotation[columnAnnotationList.size()];

		ca = (ColumnAnnotation[]) columnAnnotationList.toArray(ca);
		return ca;
	}

	// get the line annotations
	@SuppressWarnings("cast")
	public LineAnnotation[] getLineAnnotations() {
		LineAnnotation[] la = new LineAnnotation[lineAnnotationList.size()];

		la = (LineAnnotation[]) lineAnnotationList.toArray(la);
		return la;
	}

	// shift the sequence
	// shift column and line annotations as well
	@SuppressWarnings("cast")
	public void shiftActivity(int idx, int shift, boolean shiftActivityIndex) throws Exception {
		// TODO changed from original; original also had an array to keep track
		// of "numbers"

		if ((idx + shift < 0) || (idx < 0) || (idx > sequenceWithGaps.length)) {
			throw new Exception("invalid amino acid shift specified");
		}
		if (shift == 0) {
			return;
		}

		String[] newSequenceWithGaps = new String[sequenceWithGaps.length + shift];

		int[] newActivityIndexArray = null;
		if (shiftActivityIndex && (activityIndexArray != null)) {
			newActivityIndexArray = new int[activityIndexArray.length + shift];
		}

		if (shift > 0) {
			System.arraycopy(sequenceWithGaps, 0, newSequenceWithGaps, 0, idx);
			Arrays.fill(newSequenceWithGaps, idx, idx + shift, dash);

			//TODO changed the Arrays.fill idx-1 to idx
			if (shiftActivityIndex) {
				System.arraycopy(activityIndexArray, 0, newActivityIndexArray, 0, idx - 1);
				Arrays.fill(newActivityIndexArray, idx, idx + shift, -1);
			}

			if (idx < sequenceWithGaps.length) {
				System
						.arraycopy(sequenceWithGaps, idx, newSequenceWithGaps, idx + shift, sequenceWithGaps.length
								- idx);
				if (shiftActivityIndex) {
					System.arraycopy(activityIndexArray, idx, newActivityIndexArray, idx + shift,
							activityIndexArray.length - idx);
				}
			}
		} else {
			System.arraycopy(sequenceWithGaps, 0, newSequenceWithGaps, 0, idx + shift);

			if (shiftActivityIndex) {
				System.arraycopy(activityIndexArray, 0, newActivityIndexArray, 0, idx + shift);
			}

			if (idx < sequenceWithGaps.length) {
				System.arraycopy(sequenceWithGaps, idx, newSequenceWithGaps, idx + shift, newSequenceWithGaps.length
						- (idx + shift));
				if (shiftActivityIndex) {
					System.arraycopy(activityIndexArray, idx, newActivityIndexArray, idx + shift,
							newActivityIndexArray.length - (idx + shift));
				}
			}
		}

		// adjust column annotations
		Set<Integer> changedColumnSet = new HashSet<Integer>(11);
		int shiftStartIdx = idx + shift;

		for (int i = columnAnnotationList.size() - 1; i >= 0; i--) {
			ColumnAnnotation ca = (ColumnAnnotation) columnAnnotationList.get(i);

			if (ca.getColumn() >= idx) {
				// TODO haven't added extraANnotation and isFound attribute to
				// ColumnAnnotation; Add when needed and modify this
				// accordingly; Check original code
				ColumnAnnotation new_ca = new ColumnAnnotation(ca.getColumn() + shift, ca.getSymbol(), ca
						.getAnnotation(), ca.getColor(), ca.isShown());

				changedColumnSet.add(new Integer(ca.getColumn()));
				changedColumnSet.add(new Integer(new_ca.getColumn()));

				columnAnnotationList.set(i, new_ca);
			} else if (ca.getColumn() >= shiftStartIdx) {
				// delete overwritten annotations
				changedColumnSet.add(new Integer(ca.getColumn()));
				columnAnnotationList.remove(i);
			} else {
				break;
			}
		}

		// TODO adjust line annotations not added; basically this has to take
		// care of the length of the annotation text

		// set the new sequence and perform notifications
		setActivityInternal(newSequenceWithGaps, newActivityIndexArray);

		if (changedColumnSet.size() > 0) {
			for (SequenceListener listener : listenersList) {
				for (Integer column : changedColumnSet) {
					listener.sequenceColumnAnnotationsChanged(this, column);
				}
			}
		}

	}

	// set the amino acid sequence
	// delete column annotations which are out of range
	public void setActivity(String[] sequenceWithGaps) {
		ArrayList<ColumnAnnotation> oldColumnAnnotationList = new ArrayList<ColumnAnnotation>();

		for (int i = columnAnnotationList.size() - 1; i >= 0; i--) {
			ColumnAnnotation ca = columnAnnotationList.get(i);

			if (ca.getColumn() >= sequenceWithGaps.length) {
				oldColumnAnnotationList.add(ca);
				columnAnnotationList.remove(i);
			} else {
				break;
			}
		}

		// set the new sequence and perform notifications
		setActivityInternal(sequenceWithGaps, null);
		if (oldColumnAnnotationList.size() > 0) {
			for (SequenceListener listener : listenersList) {
				for (int j = oldColumnAnnotationList.size() - 1; j >= 0; j--) {
					ColumnAnnotation ca = columnAnnotationList.get(j);
					listener.sequenceColumnAnnotationsChanged(this, ca.getColumn());
				}
			}
		}
	}

	public void deleteActivity(int idx) throws Exception {
		shiftActivity(idx + 1, -1, true);
	}

	private void setActivityInternal(String[] newSequenceWithGaps, int[] newActivityIndexArray) {
		sequenceWithGaps = newSequenceWithGaps;

		if (sequenceWithGaps != null) {

			int gapCount = 0;

			for (int i = sequenceWithGaps.length - 1; i >= 0; i--) {
				if (sequenceWithGaps[i].equals(dash)) {
					gapCount++;
				}
			}
			sequenceWithoutGaps = new String[sequenceWithGaps.length - gapCount];
			alignmentIndex2ActivityIndex = new int[sequenceWithGaps.length];
			activityIndex2AlignmentIndex = new int[sequenceWithoutGaps.length];

			int aligned_cnt = 0;
			int raw_cnt = 0;

			while (aligned_cnt < sequenceWithGaps.length) {
				String encodedActivity = sequenceWithGaps[aligned_cnt];

				if (encodedActivity.equals(dash)) {
					alignmentIndex2ActivityIndex[aligned_cnt] = -1;
				} else {
					sequenceWithoutGaps[raw_cnt] = encodedActivity;
					activityIndex2AlignmentIndex[raw_cnt] = aligned_cnt;
					alignmentIndex2ActivityIndex[aligned_cnt] = raw_cnt;
					raw_cnt++;
				}
				aligned_cnt++;
			}

			//TODO added activityIndexArray related code
			if (newActivityIndexArray != null) {
				activityIndexArray = newActivityIndexArray;
			} else {
				activityIndexArray = new int[sequenceWithGaps.length];
				int j = 0;
				for (int i = 0; i < sequenceWithGaps.length; i++) {
					if (sequenceWithGaps[i].equals(dash)) {
						activityIndexArray[i] = -1;
					} else {
						activityIndexArray[i] = j;
						j++;
					}
				}
			}

		} else {
			sequenceWithoutGaps = null;
			alignmentIndex2ActivityIndex = null;
			activityIndex2AlignmentIndex = null;
			activityIndexArray = null;
		}

		for (SequenceListener listener : listenersList) {
			listener.sequenceActivityChanged(this);
		}
	}

	public String getEncodedActivity(int index) {
		return sequenceWithGaps[index];
	}

	public String getName() {
		return name;
	}

	// set the name
	public void setName(String name) throws Exception {
		String old_name = this.name;

		this.name = name;
		if ((name == old_name) || ((name != null) && name.equals(old_name))) {
			return; // no notification necessary
		}
		int idx = listenersList.size() - 1;

		try {
			while (idx >= 0) {
				listenersList.get(idx).sequenceNameChanged(this, old_name);
				idx--;
			}
		} catch (Exception e) {
			this.name = old_name;
			while (++idx < listenersList.size()) {
				listenersList.get(idx).sequenceNameChanged(this, name);
			}
			throw e;
		}
	}

	// get the annotation
	public String getAnnotation() {
		return annotation;
	}

	public String getDisplayedAnnotation() {
		if (displayedAnnotationName == null) {
			return "";
		}

		if (displayedAnnotationName.equalsIgnoreCase("SeqName")) {
			return getName();
		}

		return "";
		// TODO changed from original;
	}

	/**
	 * deletes sequence annotations
	 * 
	 */
	public void deleteAnnotation() {
		annotation = null;
		// TODO commented the next line
		// extraAnnotations.clear();
		for (SequenceListener listener : listenersList) {
			listener.sequenceAnnotationChanged(this);
		}
	}

	// get the column annotation for a specific column
	public ColumnAnnotation getColumnAnnotation(int column) {
		// binary search for position
		int idx = findColumnAnnotationIndex(column);

		return idx >= 0 ? columnAnnotationList.get(idx) : null;
	}

	// binary search for a column annotation position
	private int findColumnAnnotationIndex(int column) {
		int low = 0;
		int high = columnAnnotationList.size() - 1;

		while (low <= high) {
			int mid = (low + high) / 2;
			int midVal = (columnAnnotationList.get(mid)).getColumn();

			if (midVal < column) {
				low = mid + 1;
			} else if (midVal > column) {
				high = mid - 1;
			} else {
				return mid; // key found
			}
		}
		return -(low + 1); // key not found.
	}

	// set the color
	public void setColor(Color color) {
		if ((this.color == null) || (color == null) || !color.equals(this.color)) {
			this.color = color;
			for (SequenceListener listener : listenersList) {
				listener.sequenceColorChanged(this);
			}
		}
	}

	public int getSortValue() {
		return sortValue;
	}

	public void setSortValue(int sortValue) {
		this.sortValue = sortValue;
	}

	// add a line annotation
	public LineAnnotation addLineAnnotation(String name, String ann) {
		LineAnnotation lineAnnotation = new LineAnnotation(name, ann, this);
		addLineAnnotation(lineAnnotation);
		return lineAnnotation;
	}

	public void addLineAnnotation(LineAnnotation lineAnnotation) {
		lineAnnotationList.add(lineAnnotation);
		lineAnnotation.setSequence(this);
		for (SequenceListener listener : listenersList) {
			listener.sequenceLineAnnotationsChanged(this);
		}
	}

	/**
	 * Deletes the specified line annotation
	 * 
	 * @param lineAnnotName
	 */
	public void deleteLineAnnotation(String lineAnnotName) {
		for (int i = 0; i < lineAnnotationList.size(); i++) {
			LineAnnotation lineAnnotation = lineAnnotationList.get(i);
			if (lineAnnotName.equals(lineAnnotation.getName())) {
				deleteLineAnnotation(i);
			}
		}
	}

	// delete a line annotation
	public void deleteLineAnnotation(int idx) {
		lineAnnotationList.remove(idx);
		for (SequenceListener listener : listenersList) {
			listener.sequenceLineAnnotationsChanged(this);
		}
	}

	// set a line annotation
	public void setLineAnnotation(int idx, String name, String ann) {
		LineAnnotation lineAnnotation = this.getLineAnnotation(name);
		if (lineAnnotation == null) {
			lineAnnotation = new LineAnnotation(name, ann, this);
		}
		lineAnnotation.setAnnotation(ann);

		lineAnnotationList.set(idx, lineAnnotation);
		for (SequenceListener listener : listenersList) {
			listener.sequenceLineAnnotationsChanged(this);
		}
	}

	public String getDisplayedAnnotationName() {
		// TODO changed temporarily; need to study original in detail
		return "SeqName";
	}

	/**
	 * Returns sequence (with gaps) as a string
	 * 
	 * @param String
	 *            sequenceWithGaps
	 */
	public String toString() {
		StringBuilder sequenceStrBuilder = new StringBuilder();
		for (String encodedActivity : sequenceWithGaps) {
			sequenceStrBuilder.append(encodedActivity);
		}

		return sequenceStrBuilder.toString();
	}

	public void addListener(SequenceListener listener) {
		listenersList.add(listener);
	}

	public void removeListener(SequenceListener listener) {
		listenersList.remove(listener);
	}

	class SequenceAnnotation {
		int noDuplicates;
		List<String> identicalTracesNameList;

		public SequenceAnnotation(int noDuplicates, List<String> identicalTracesNameList) {
			this.noDuplicates = noDuplicates;
			this.identicalTracesNameList = identicalTracesNameList;
		}
	}

	public static class LineAnnotation {
		private final String name;
		private String annotation;
		private Sequence sequence;

		public LineAnnotation(String name, String annotation, Sequence seq) {
			this.name = name;
			this.annotation = annotation;
			sequence = seq;
		}

		public String getName() {
			return name;
		}

		public String getAnnotation() {
			return annotation;
		}

		public void setAnnotation(String newAnnotation) {
			annotation = newAnnotation;
		}

		public void setSequence(Sequence sequence) {
			this.sequence = sequence;

		}

		public Sequence getSequence() {
			return sequence;
		}

		public boolean equals(Object obj) {
			if (obj == this) {
				return true;
			}
			if ((obj == null) || (obj.getClass() != this.getClass())) {
				return false;
			}
			LineAnnotation otherAnnotation = (LineAnnotation) obj;
			if ((otherAnnotation.name == null) && (name != null)) {
				return false;
			}
			if ((otherAnnotation.name != null) && (name == null)) {
				return false;
			}
			if ((otherAnnotation.annotation == null) && (annotation != null)) {
				return false;
			}
			if ((otherAnnotation.annotation != null) && (annotation == null)) {
				return false;
			}
			if ((otherAnnotation.sequence == null) && (sequence != null)) {
				return false;
			}
			if ((otherAnnotation.sequence != null) && (sequence == null)) {
				return false;
			}
			return (otherAnnotation.name.equals(name) && otherAnnotation.annotation.equals(annotation) && (otherAnnotation.sequence == sequence));
		}

		public int hashCode() {
			int hash = 9;
			hash = 31 * hash + (null == name ? 0 : name.hashCode());
			hash = 31 * hash + (null == annotation ? 0 : annotation.hashCode());
			hash = 31 * hash + (null == sequence ? 0 : sequence.hashCode());
			return hash;
		}

	}

	public static class ColumnAnnotation {

		private final int column;
		private final String symbol;
		private final Color color;
		private final String annotation;

		public boolean shown = true;

		public ColumnAnnotation(int column, String symbol, String annotation, Color color, boolean shown) {
			this.column = column;
			this.symbol = symbol;
			this.annotation = annotation;
			this.color = color;
			this.shown = shown;
		}

		public String getAnnotation() {
			return annotation;
		}

		public int getColumn() {
			return column;
		}

		public String getSymbol() {
			return symbol;
		}

		public Color getColor() {
			return color;
		}

		public boolean isShown() {
			return shown;
		}

		public void getFormattedAnnotationInfo(Sequence seq, StringBuffer sb) {
			sb.append("Working Perfectly; Need to put the actual details now");
			// TODO
		}
	}

	public int[] getActivityIndexArray() {
		return activityIndexArray;
	}

	public int getActivityIndex(int index) {
		return activityIndexArray[index];
	}

	public int getNoDuplicates() {
		return duplicateProcessInstanceNameList.size() - 1;
	}

	public String getSequenceWithoutGapsString() {
		StringBuilder strBuilder = new StringBuilder();
		for (String activity : sequenceWithoutGaps) {
			strBuilder.append(activity);
		}

		return strBuilder.toString();
	}
}
