package org.processmining.plugins.tracealignment.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;

import javax.swing.JPanel;

import org.processmining.plugins.tracealignment.AlignmentFrame;
import org.processmining.plugins.tracealignment.ui.Sequence.LineAnnotation;

@SuppressWarnings("serial")
public class SequenceAnnComponent extends JPanel implements SequenceListener, DisplayPropertiesListener, MouseListener {
	// underlying sequence
	private Sequence sequence = null;
	// properties
	private final DisplayProperties displayProperties;

	boolean collapsed = false;
	AlignmentFrame owner;

	public SequenceAnnComponent(AlignmentFrame owner, Sequence sequence, DisplayProperties displayProperties) {
		super();
		this.owner = owner;
		setBackground(Color.white);
		setLayout(new BorderLayout());

		this.displayProperties = displayProperties;
		displayProperties.addListener(this);

		addMouseListener(this);

		setSequence(sequence);
	}

	public void setSequence(Sequence sequence) {
		if (this.sequence != null) {
			this.sequence.removeListener(this);
		}
		this.sequence = sequence;
		this.sequence.addListener(this);
		revalidate();
		repaint();
	}

	@SuppressWarnings("deprecation")
	public void edit(Point loc) {
		removeAll();

		displayProperties.setSeqAnnEditing(true);
		displayProperties.setRulerEditing(false);
		displayProperties.setCursorHidden(true);

		// TODO check if this frame can be replaced with Panel

		//		Frame frame = JOptionPane.getFrameForComponent(this);
		//TODO changed the following line
		//		EditSeqAnnDialog editSeqAnnDialog = new EditSeqAnnDialog(
		//				(AlignmentFrame) frame, displayProperties, sequence);
		EditSeqAnnDialog editSeqAnnDialog = new EditSeqAnnDialog(owner, displayProperties, sequence);
		editSeqAnnDialog.setLocation(loc);
		editSeqAnnDialog.show();

		revalidate();
		repaint();
		displayProperties.setSeqAnnEditing(false);
		displayProperties.setCursorHidden(false);
	}

	public Sequence getSequence() {
		return sequence;
	}

	// caller must revalidate...
	public void setCollapsed(boolean collapsed) {
		this.collapsed = collapsed;
	}

	public boolean getCollapsed() {
		return collapsed;
	}

	public Dimension getPreferredSize() {
		// System.out.println("***** Inside seq ann comp: getPreferredSize");

		FontMetrics font_metrics = displayProperties.getFontMetrics();
		int height = displayProperties.getActivityHeight();

		if (collapsed) {
			height = 0;
		} else if (displayProperties.getAnnView(sequence)) {
			List<LineAnnotation> visibleLineAnnotations = displayProperties.getVisibleLineAnnotations(sequence);

			height *= 1 + visibleLineAnnotations.size();
		}

		int width = 2 * displayProperties.getActivityWidth();

		String ann = sequence.getDisplayedAnnotation();

		if (ann != null) {
			width += font_metrics.stringWidth(ann) + 2;
		}

		return new Dimension(width, height);
	}

	public Dimension getSize() {
		Dimension d = super.getSize();
		if (collapsed) {
			d.height = 0;
		}

		return d;
	}

	public void paint(Graphics g) {
		g.setColor(Color.white);
		Dimension d = getPreferredSize();

		g.fillRect(0, 0, d.width, d.height);

		String ann = sequence.getDisplayedAnnotation();

		if (ann == null) {
			return;
		}

		Font font = displayProperties.getFont();
		int x = displayProperties.getFontXOffset();
		int font_y = displayProperties.getFontYOffset();

		g.setFont(font);
		g.setColor(Color.black);
		g.drawString(ann, x, font_y);
	}

	// SequenceListener interface implementation
	public void sequenceAnnotationChanged(Sequence sequence) {
		if (this.sequence != sequence) {
			throw new RuntimeException("bound to incorrect Sequence");
		}
		revalidate();
		repaint();
	}

	public void sequenceActivityChanged(Sequence aaseq) {
		// Nothing to be done here
	}

	public void sequenceColorChanged(Sequence sequence) {
		// Nothing to be done here
	}

	public void sequenceColumnAnnotationsChanged(Sequence sequence, int column) {
		// Nothing to be done here
	}

	public void sequenceLineAnnotationsChanged(Sequence aaseq) {
		// Nothing to be done here
	}

	public void sequenceNameChanged(Sequence sequence, String oldName) throws Exception {
		// Nothing to be done here
	}

	// DisplayPropertiesListener interface implementation
	public void displayAnnViewChanged(DisplayProperties dp, Sequence sequence, boolean show) {
		if (sequence == this.sequence) {
			revalidate();
			this.repaint();
		}
	}

	public void displayColorSchemeChanged(DisplayProperties dp, ColorScheme old) {
		// Nothing to be done here
	}

	public void displayFontChanged(DisplayProperties dp) {
		if (dp != displayProperties) {
			throw new RuntimeException("bound to incorrect DisplayProperties");
		}
		revalidate();
		repaint();
	}

	public void displayGroupEditingChanged(DisplayProperties dp) {
		// Nothing to be done here		
	}

	public void displayHighlightsChanged(DisplayProperties dp, Sequence sequence) {
		// Nothing to be done here
	}

	public void displayHighlightsChanged(DisplayProperties dp, Sequence[] sequenceArray) {
		// Nothing to be done here
	}

	public void displayOverwriteChanged(DisplayProperties dp) {
		// Nothing to be done here		
	}

	public void displayRenderGapsChanged(DisplayProperties dp) {
		// Nothing to be done here		
	}

	public void activityBackgroundChanged(DisplayProperties dp) {
		// Nothing to be done here
	}

	public void displaySeqSelectChanged(DisplayProperties dp, Sequence sequence, boolean select) {
		// Nothing to be done here		
	}

	// MouseListener interface implementation
	//	@Override
	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	//	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	//	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	//	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	//	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

}
